@@ -372,7 +372,7 @@ void MyMoneyQifReader::slotProcessData(void)
     m_userAbort = true;
   }
 
-  signalProgress(0, m_qifLines.count(), i18n("Importing QIF ..."));
+  signalProgress(0, m_qifLines.count(), i18n("Importing QIF..."));
   QStringList::iterator it;
   for (it = m_qifLines.begin(); m_userAbort == false && it != m_qifLines.end(); ++it) {
     ++m_linenumber;
@@ -447,8 +447,8 @@ bool MyMoneyQifReader::startImport(void)
       m_filter << "type";
       m_filter << QDir::toNativeSeparators(m_filename);
 #else
-      m_filter << "cat";
-      m_filter << "-";
+    m_filter << "cat";
+    m_filter << "-";
 #endif
     } else {
       m_filter << m_qifProfile.filterScriptImport().split(' ', QString::KeepEmptyParts);
@@ -458,13 +458,13 @@ bool MyMoneyQifReader::startImport(void)
     m_filter.setOutputChannelMode(KProcess::MergedChannels);
     m_filter.start();
     if (m_filter.waitForStarted()) {
-      signalProgress(0, m_file->size(), i18n("Reading QIF ..."));
+      signalProgress(0, m_file->size(), i18n("Reading QIF..."));
       slotSendDataToFilter();
       rc = true;
     } else {
       KMessageBox::detailedError(0, i18n("Error while running the filter '%1'.", m_filter.program().join(QLatin1String(" "))),
-                                    m_filter.errorString(),
-                                    i18n("Filter error"));
+                                 m_filter.errorString(),
+                                 i18n("Filter error"));
     }
 #endif
   }
@@ -547,10 +547,9 @@ bool MyMoneyQifReader::finishImport(void)
     }
     if (rc)
       ft.commit();
-  } catch (MyMoneyException *e) {
+  } catch (const MyMoneyException &e) {
     KMessageBox::detailedSorry(0, i18n("Unable to add transactions"),
-                               i18n("%1 thrown in %2:%3", e->what(), e->file(), e->line()));
-    delete e;
+                               i18n("%1 thrown in %2:%3", e.what(), e.file(), e.line()));
     rc = false;
   }
 #endif
@@ -705,13 +704,12 @@ void MyMoneyQifReader::processQifEntry(void)
         kDebug(2) << "Line " << m_linenumber << ": EntryType " << m_entryType << " not yet implemented!";
         break;
     }
-  } catch (MyMoneyException *e) {
-    if (e->what() != "USERABORT") {
-      kDebug(2) << "Line " << m_linenumber << ": Unhandled error: " << e->what();
+  } catch (const MyMoneyException &e) {
+    if (e.what() != "USERABORT") {
+      kDebug(2) << "Line " << m_linenumber << ": Unhandled error: " << e.what();
     } else {
       m_userAbort = true;
     }
-    delete e;
   }
 }
 
@@ -732,27 +730,44 @@ const QString MyMoneyQifReader::extractLine(const QChar& id, int cnt)
   return QString();
 }
 
-void MyMoneyQifReader::extractSplits(QList<qSplit>& listqSplits) const
+bool MyMoneyQifReader::extractSplits(QList<qSplit>& listqSplits) const
 {
 //     *** With apologies to QString MyMoneyQifReader::extractLine ***
 
   QStringList::ConstIterator it;
+  bool ret = false;
+  bool memoPresent = false;
+  int neededCount = 0;
+  qSplit q;
 
   for (it = m_qifEntry.constBegin(); it != m_qifEntry.constEnd(); ++it) {
-    if ((*it)[0] == 'S') {
-      qSplit q;
-      q.m_strCategoryName = (*it++).mid(1);       //   'S'
+    if (((*it)[0] == 'S') || ((*it)[0] == '$') || ((*it)[0] == 'E')) {
+      memoPresent = false;  //                      in case no memo line in this split
       if ((*it)[0] == 'E') {
-        q.m_strMemo = (*it++).mid(1); //           'E'
+        q.m_strMemo = (*it).mid(1);  //             'E' = Memo
         d->fixMultiLineMemo(q.m_strMemo);
+        memoPresent = true;  //                     This transaction contains memo
+      } else if ((*it)[0] == 'S') {
+        q.m_strCategoryName = (*it).mid(1);  //   'S' = CategoryName
+        neededCount ++;
+      } else if ((*it)[0] == '$') {
+        q.m_amount = (*it).mid(1);  //            '$' = Amount
+        neededCount ++;
       }
-      if ((*it)[0] == '$') {
-        q.m_amount = (*it).mid(1); //              '$'
+      if (neededCount > 1) {  //                         CategoryName & Amount essential
+        listqSplits += q;  //                       Add valid split
+        if (!memoPresent) {  //                     If no memo, clear previous
+          q.m_strMemo.clear();
+        }
+        qSplit q;  //                               Start new split
+        neededCount = 0;
+        ret = true;
       }
-      listqSplits += q;
     }
   }
+  return ret;
 }
+
 #if 0
 void MyMoneyQifReader::processMSAccountEntry(const MyMoneyAccount::accountTypeE accountType)
 {
@@ -891,7 +906,7 @@ void MyMoneyQifReader::processCategoryEntry(void)
   }
 
   // check if we can find the account already in the file
-  MyMoneyAccount acc = kmymoney->findAccount(account, MyMoneyAccount());
+  MyMoneyAccount acc = kmymoney->findAccount(account, parentAccount);
 
   // if not, we just create it
   if (acc.id().isEmpty()) {
@@ -991,12 +1006,11 @@ void MyMoneyQifReader::createOpeningBalance(MyMoneyAccount::_accountTypeE accTyp
 
     // remember which account we created
     d->st.m_accountId = m_account.id();
-  } catch (MyMoneyException* e) {
+  } catch (const MyMoneyException &e) {
     KMessageBox::detailedError(0,
                                i18n("Error while creating opening balance transaction"),
-                               QString("%1(%2):%3").arg(e->file()).arg(e->line()).arg(e->what()),
+                               QString("%1(%2):%3").arg(e.file()).arg(e.line()).arg(e.what()),
                                i18n("File access error"));
-    delete e;
   }
 }
 
@@ -1070,7 +1084,7 @@ void MyMoneyQifReader::processTransactionEntry(void)
         break;
 
       case KMessageBox::Cancel:
-        throw new MYMONEYEXCEPTION("USERABORT");
+        throw MYMONEYEXCEPTION("USERABORT");
         break;
     }
   }
@@ -1133,7 +1147,11 @@ void MyMoneyQifReader::processTransactionEntry(void)
   s1.m_strMemo = tr.m_strMemo;
   // tr.m_listSplits.append(s1);
 
-  if (extractLine('$').isEmpty()) {
+  //             split transaction
+  //      ****** ensure each field is ******
+  //      *   attached to correct split    *
+  QList<qSplit> listqSplits;
+  if (! extractSplits(listqSplits)) {
     MyMoneyAccount account;
     // use the same values for the second split, but clear the ID and reverse the value
     MyMoneyStatement::Split s2 = s1;
@@ -1172,10 +1190,9 @@ void MyMoneyQifReader::processTransactionEntry(void)
           accountId.clear();
         }
 
-      } catch (MyMoneyException *e) {
+      } catch (const MyMoneyException &) {
         kDebug(0) << "Line " << m_linenumber << ": Account with id " << accountId.data() << " not found";
         accountId.clear();
-        delete e;
       }
     }
 
@@ -1186,14 +1203,8 @@ void MyMoneyQifReader::processTransactionEntry(void)
     }
 
   } else {
-    // split transaction
-    QList<qSplit> listqSplits;
-
-    extractSplits(listqSplits);   //      ****** ensure each field is ******
-    //      *   attached to correct split    *
     int   count;
-
-    for (count = 1; !extractLine('$', count).isEmpty(); ++count) {
+    for (count = 1; count <= listqSplits.count(); ++count) {                     // Use true splits count
       MyMoneyStatement::Split s2 = s1;
       s2.m_amount = (-m_qifProfile.value('$', listqSplits[count-1].m_amount));   // Amount of split
       s2.m_strMemo = listqSplits[count-1].m_strMemo;                             // Memo in split
@@ -1205,7 +1216,6 @@ void MyMoneyQifReader::processTransactionEntry(void)
       } else {
         pos = tmp.lastIndexOf("--");
         if (pos != -1) {
-///          t.setValue("Dialog", tmp.mid(pos+2));
           tmp = tmp.left(pos);
         }
         tmp = tmp.trimmed();
@@ -1227,10 +1237,9 @@ void MyMoneyQifReader::processTransactionEntry(void)
             accountId.clear();
           }
 
-        } catch (MyMoneyException *e) {
+        } catch (const MyMoneyException &) {
           kDebug(0) << "Line " << m_linenumber << ": Account with id " << accountId.data() << " not found";
           accountId.clear();
-          delete e;
         }
       }
       if (!accountId.isEmpty()) {
@@ -1303,7 +1312,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry(void)
         break;
 
       case KMessageBox::Cancel:
-        throw new MYMONEYEXCEPTION("USERABORT");
+        throw MYMONEYEXCEPTION("USERABORT");
         break;
     }
   }
@@ -1446,8 +1455,29 @@ void MyMoneyQifReader::processInvestmentTransactionEntry(void)
     xAction = true;
   }
 
+  tmp = extractLine('L');
+  // if the action ends in an X, the L-Record contains the asset account
+  // to which the dividend should be transferred. In the other cases, it
+  // may contain a category that identifies the income category for the
+  // dividend payment
+  if ((xAction == true)
+      || (d->isTransfer(tmp, m_qifProfile.accountDelimiter().left(1), m_qifProfile.accountDelimiter().mid(1, 1)) == true)) {
+    tmp = tmp.remove(QRegExp("[\\[\\]]"));                 //  xAction != true so ignore any'[ and ]'
+    if (!tmp.isEmpty()) {                                  // use 'L' record name
+      tr.m_strBrokerageAccount = tmp;
+      transferAccount(tmp);                                // make sure the account exists
+    } else {
+      tr.m_strBrokerageAccount = m_account.brokerageName();// use brokerage account
+      transferAccount(m_account.brokerageName());          // make sure the account exists
+    }
+  } else {
+    tmp = tmp.remove(QRegExp("[\\[\\]]"));                 //  xAction != true so ignore any'[ and ]'
+    tr.m_strInterestCategory = tmp;
+    tr.m_strBrokerageAccount = m_account.brokerageName();
+  }
+
   // Whether to create a cash split for the other side of the value
-  QString accountname ;//= extractLine('L');
+  QString accountname; //= extractLine('L');
   if (action == "reinvdiv" || action == "reinvlg" || action == "reinvsh") {
     d->st.m_listPrices += price;
     tr.m_shares = quantity;
@@ -1461,21 +1491,6 @@ void MyMoneyQifReader::processInvestmentTransactionEntry(void)
   } else if (action == "div" || action == "cgshort" || action == "cgmid" || action == "cglong" || action == "rtrncap") {
     tr.m_eAction = (MyMoneyStatement::Transaction::eaCashDividend);
 
-    QString tmp = extractLine('L');
-    // if the action ends in an X, the L-Record contains the asset account
-    // to which the dividend should be transferred. In the other cases, it
-    // may contain a category that identifies the income category for the
-    // dividend payment
-    if ((xAction == true)
-        && (d->isTransfer(tmp, m_qifProfile.accountDelimiter().left(1), m_qifProfile.accountDelimiter().mid(1, 1)) == true)) {
-      tr.m_strBrokerageAccount = tmp;
-      transferAccount(tmp);           // make sure the account exists
-    } else {
-      tmp = tmp.remove(QRegExp("[\\[\\]]"));//  xAction != true so ignore any'[ and ]'
-      tr.m_strInterestCategory = tmp;
-      tr.m_strBrokerageAccount = m_account.brokerageName();
-    }
-
     // make sure, we have valid category. Either taken from the L-Record above,
     // or derived from the action code
     if (tr.m_strInterestCategory.isEmpty()) {
@@ -1498,20 +1513,6 @@ void MyMoneyQifReader::processInvestmentTransactionEntry(void)
     if (action == "miscexp")
       tr.m_eAction = (MyMoneyStatement::Transaction::eaFees);
 
-    QString tmp = extractLine('L');
-    // if the action ends in an X, the L-Record contains the asset account
-    // to which the dividend should be transferred. In the other cases, it
-    // may contain a category that identifies the income category for the
-    // payment
-    if ((xAction == true)
-        && (d->isTransfer(tmp, m_qifProfile.accountDelimiter().left(1), m_qifProfile.accountDelimiter().mid(1, 1)) == true)) {
-      tr.m_strBrokerageAccount = tmp;
-      transferAccount(tmp);           // make sure the account exists
-    } else {
-      tmp = tmp.remove(QRegExp("[\\[\\]]"));//  xAction != true so ignore any '[ and ]'
-      tr.m_strInterestCategory = tmp;
-      tr.m_strBrokerageAccount = m_account.brokerageName();
-    }
     // make sure, we have a valid category. Either taken from the L-Record above,
     // or derived from the action code
     if (tr.m_strInterestCategory.isEmpty()) {
@@ -1557,22 +1558,10 @@ void MyMoneyQifReader::processInvestmentTransactionEntry(void)
     s2.m_amount = -tr.m_amount;
     tr.m_listSplits.append(s2);
   } else if (action == "buy") {
-    QString tmp = extractLine('L');
-    if (d->isTransfer(tmp, m_qifProfile.accountDelimiter().left(1), m_qifProfile.accountDelimiter().mid(1, 1)) == true) {
-      tr.m_strBrokerageAccount = tmp;
-      transferAccount(tmp);           // make sure the account exists
-    }
-
     d->st.m_listPrices += price;
     tr.m_shares = quantity;
     tr.m_eAction = (MyMoneyStatement::Transaction::eaBuy);
   } else if (action == "sell") {
-    QString tmp = extractLine('L');
-    if (d->isTransfer(tmp, m_qifProfile.accountDelimiter().left(1), m_qifProfile.accountDelimiter().mid(1, 1)) == true) {
-      tr.m_strBrokerageAccount = tmp;
-      transferAccount(tmp);           // make sure the account exists
-    }
-
     d->st.m_listPrices += price;
     tr.m_shares = -quantity;
     tr.m_amount = -amount;
@@ -1599,7 +1588,7 @@ void MyMoneyQifReader::processInvestmentTransactionEntry(void)
     kDebug(0) << "Line " << m_linenumber << ": Unsupported transaction action (" << action << ")";
     return;
   }
-  d->st.m_strAccountName = accountname;
+  d->st.m_strAccountName = accountname;  //  accountname appears not to get set
   d->st.m_listTransactions += tr;
 
   /*************************************************************************
@@ -2037,8 +2026,9 @@ const QString MyMoneyQifReader::processAccountEntry(bool resetAccountId)
 
     // investment accounts will receive a brokerage account, as KMyMoney
     // currently does not allow to store funds in the investment account directly
+    // but only create it (not here, but later) if it is needed
     if (account.accountType() == MyMoneyAccount::Investment) {
-      brokerage.setName(account.brokerageName());
+      brokerage.setName(QString());  //                           brokerage name empty so account not created yet
       brokerage.setAccountType(MyMoneyAccount::Checkings);
       brokerage.setCurrencyId(MyMoneyFile::instance()->baseCurrency().id());
     }
@@ -2053,7 +2043,7 @@ const QString MyMoneyQifReader::processAccountEntry(bool resetAccountId)
     // possibly start a new statement
     d->finishStatement();
     m_account = acc;
-    d->st.m_accountId = m_account.id();
+    d->st.m_accountId = m_account.id();  //                      needed here for account selection
     d->transactionType = transactionType;
   }
   return acc.id();
@@ -2101,11 +2091,9 @@ void MyMoneyQifReader::selectOrCreateAccount(const SelectCreateMode mode, MyMone
       delete accountSelect;
       return;
 
-    } catch (MyMoneyException *e) {
-      QString message(i18n("Account \"%1\" disappeared: ", account.name()));
-      message += e->what();
+    } catch (const MyMoneyException &e) {
+      const QString message(i18n("Account \"%1\" disappeared: %2", account.name(), e.what()));
       KMessageBox::error(0, message);
-      delete e;
     }
   }
 
@@ -2207,7 +2195,7 @@ void MyMoneyQifReader::selectOrCreateAccount(const SelectCreateMode mode, MyMone
           }
 
         } else if (accountSelect->aborted())
-          throw new MYMONEYEXCEPTION("USERABORT");
+          throw MYMONEYEXCEPTION("USERABORT");
 
         if (typeStr == i18n("account")) {
           KMessageBox::error(0, i18n("You must select or create an account."));
@@ -2298,5 +2286,3 @@ void MyMoneyQifReader::processSecurityEntry(void)
 
   d->st.m_listSecurities += security;
 }
-
-#include "mymoneyqifreader.moc"
