@@ -49,7 +49,7 @@ bool Activity::isComplete(QString& reason) const
   bool rc = false;
   KMyMoneySecurity* security = dynamic_cast<KMyMoneySecurity*>(haveWidget("security"));
   if (!security->currentText().isEmpty()) {
-    rc = security->selector()->contains(security->currentText());
+    rc = (security->selector()->contains(security->currentText()) || (isMultiSelection() && m_memoChanged));
   }
   return rc;
 }
@@ -90,7 +90,7 @@ bool Activity::haveCategoryAndAmount(const QString& category, const QString& amo
 bool Activity::haveShares(void) const
 {
   kMyMoneyEdit* amount = dynamic_cast<kMyMoneyEdit*>(haveWidget("shares"));
-  if (isMultiSelection() && amount->text().isEmpty())
+  if (isMultiSelection() && amount->value().isZero())
     return true;
 
   return !amount->value().isZero();
@@ -99,7 +99,7 @@ bool Activity::haveShares(void) const
 bool Activity::havePrice(void) const
 {
   kMyMoneyEdit* amount = dynamic_cast<kMyMoneyEdit*>(haveWidget("price"));
-  if (isMultiSelection() && amount->text().isEmpty())
+  if (isMultiSelection() && amount->value().isZero())
     return true;
 
   return !amount->value().isZero();
@@ -240,13 +240,13 @@ bool Buy::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
   if (!s0.shares().isZero())
     price = (s0.value() / s0.shares()).reduce();
 
-  if (!isMultiSelection() || (isMultiSelection() && !sharesEdit->text().isEmpty())) {
+  if (!isMultiSelection() || (isMultiSelection() && !sharesEdit->value().isZero())) {
     shares = sharesEdit->value().abs();
     s0.setShares(shares);
     s0.setValue((shares * price).reduce());
     s0.setPrice(price);
   }
-  if (!isMultiSelection() || (isMultiSelection() && !priceEdit->text().isEmpty())) {
+  if (!isMultiSelection() || (isMultiSelection() && !priceEdit->value().isZero())) {
     price = priceEdit->value().abs();
     if (priceMode() == InvestTransactionEditor::PricePerTransaction) {
       s0.setValue(price.reduce());
@@ -279,6 +279,10 @@ bool Buy::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
 void Sell::showWidgets(void) const
 {
   KMyMoneyCategory* cat;
+  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account"));
+  cat->show();
+  cat->splitButton()->show();
+
   cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account"));
   cat->show();
   cat->splitButton()->show();
@@ -287,14 +291,15 @@ void Sell::showWidgets(void) const
   shareEdit->setPrecision(MyMoneyMoney::denomToPrec(m_parent->security().smallestAccountFraction()));
 
   QStringList widgets;
-  widgets << "asset-account" << "shares" << "price" << "total";
+  widgets << "asset-account" << "interest-amount" << "shares" << "price" << "total";
   QStringList::const_iterator it_w;
   for (it_w = widgets.constBegin(); it_w != widgets.constEnd(); ++it_w) {
     QWidget* w = haveWidget(*it_w);
     if (w)
       w->show();
   }
-
+  setLabelText("interest-amount-label", i18n("Interest"));
+  setLabelText("interest-label", i18n("Interest"));
   setLabelText("fee-label", i18n("Fees"));
   setLabelText("asset-label", i18n("Account"));
   setLabelText("shares-label", i18n("Shares"));
@@ -307,6 +312,7 @@ bool Sell::isComplete(QString& reason) const
   bool rc = Activity::isComplete(reason);
   rc &= haveAssetAccount();
   rc &= haveFees(true);
+  rc &= haveInterest(true);
   rc &= haveShares();
   rc &= havePrice();
   return rc;
@@ -332,13 +338,13 @@ bool Sell::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpl
   if (!s0.shares().isZero())
     price = (s0.value() / s0.shares()).reduce();
 
-  if (!isMultiSelection() || (isMultiSelection() && !sharesEdit->text().isEmpty())) {
+  if (!isMultiSelection() || (isMultiSelection() && !sharesEdit->value().isZero())) {
     shares = -sharesEdit->value().abs();
     s0.setShares(shares);
     s0.setValue((shares * price).reduce());
     s0.setPrice(price);
   }
-  if (!isMultiSelection() || (isMultiSelection() && !priceEdit->text().isEmpty())) {
+  if (!isMultiSelection() || (isMultiSelection() && !priceEdit->value().isZero())) {
     price = priceEdit->value().abs();
     if (priceMode() == InvestTransactionEditor::PricePerTransaction) {
       price = -price;
@@ -354,11 +360,12 @@ bool Sell::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpl
   if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("fee-amount")), MyMoneyMoney(1, 1), feeSplits, m_feeSplits))
     return false;
 
+  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("interest-amount")), MyMoneyMoney(-1, 1), interestSplits, m_interestSplits))
+    return false;
+
   createAssetAccountSplit(assetAccountSplit, s0);
 
-  interestSplits.clear();
-
-  MyMoneyMoney total = sumSplits(s0, feeSplits, QList<MyMoneySplit>());
+  MyMoneyMoney total = sumSplits(s0, feeSplits, interestSplits);
   assetAccountSplit.setValue(-total);
 
   if (!m_parent->setupPrice(t, assetAccountSplit))
@@ -406,7 +413,9 @@ bool Div::isComplete(QString& reason) const
 {
   Q_UNUSED(reason)
 
-  bool rc = haveAssetAccount();
+  bool rc = Activity::isComplete(reason);
+  rc &= haveAssetAccount();
+  rc &= haveCategoryAndAmount("interest-account", QString(), false);
   rc &= haveInterest(false);
   return rc;
 }
@@ -429,6 +438,9 @@ bool Div::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
   s0.setValue(shares);
   s0.setPrice(MyMoneyMoney(1, 1));
 
+  if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("fee-amount")), MyMoneyMoney(1, 1), feeSplits, m_feeSplits))
+    return false;
+
   if (!createCategorySplits(t, dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account")), dynamic_cast<kMyMoneyEdit*>(haveWidget("interest-amount")), MyMoneyMoney(-1, 1), interestSplits, m_interestSplits))
     return false;
 
@@ -436,7 +448,10 @@ bool Div::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpli
 
   MyMoneyMoney total = sumSplits(s0, feeSplits, interestSplits);
   assetAccountSplit.setValue(-total);
-  assetAccountSplit.setShares(-total);
+
+  if (!m_parent->setupPrice(t, assetAccountSplit))
+    return false;
+
   return true;
 }
 
@@ -501,13 +516,13 @@ bool Reinvest::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMone
   if (!s0.shares().isZero())
     price = (s0.value() / s0.shares()).reduce();
 
-  if (!isMultiSelection() || (isMultiSelection() && !sharesEdit->text().isEmpty())) {
+  if (!isMultiSelection() || (isMultiSelection() && !sharesEdit->value().isZero())) {
     shares = sharesEdit->value().abs();
     s0.setShares(shares);
     s0.setValue((shares * price).reduce());
     s0.setPrice(price);
   }
-  if (!isMultiSelection() || (isMultiSelection() && !priceEdit->text().isEmpty())) {
+  if (!isMultiSelection() || (isMultiSelection() && !priceEdit->value().isZero())) {
     price = priceEdit->value().abs();
     if (priceMode() == InvestTransactionEditor::PricePerTransaction) {
       s0.setValue(price.reduce());
@@ -715,7 +730,9 @@ bool IntInc::isComplete(QString& reason) const
 {
   Q_UNUSED(reason)
 
-  bool rc = haveAssetAccount();
+  bool rc = Activity::isComplete(reason);
+  rc &= haveAssetAccount();
+  rc &= haveCategoryAndAmount("interest-account", QString(), false);
   rc &= haveInterest(false);
   return rc;
 }
@@ -746,7 +763,9 @@ bool IntInc::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneyS
 
   MyMoneyMoney total = sumSplits(s0, feeSplits, interestSplits);
   assetAccountSplit.setValue(-total);
-  assetAccountSplit.setShares(-total);
+
+  if (!m_parent->setupPrice(t, assetAccountSplit))
+    return false;
 
   return true;
 }
