@@ -36,7 +36,6 @@
 #include <kcombobox.h>
 #include <kpushbutton.h>
 #include <kmessagebox.h>
-#include <kstdguiitem.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -194,8 +193,7 @@ void InvestTransactionEditor::dissectTransaction(const MyMoneyTransaction& trans
   currency.setTradingSymbol("???");
   try {
     currency = file->security(transaction.commodity());
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
   }
 }
 
@@ -212,22 +210,22 @@ void InvestTransactionEditor::createEditWidgets(void)
   security->setClickMessage(i18n("Security"));
   m_editWidgets["security"] = security;
   connect(security, SIGNAL(itemSelected(QString)), this, SLOT(slotUpdateSecurity(QString)));
-  connect(security, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
+  connect(security, SIGNAL(editTextChanged(QString)), this, SLOT(slotUpdateButtonState()));
   connect(security, SIGNAL(createItem(QString,QString&)), this, SLOT(slotCreateSecurity(QString,QString&)));
   connect(security, SIGNAL(objectCreation(bool)), this, SIGNAL(objectCreation(bool)));
 
   KMyMoneyCategory* asset = new KMyMoneyCategory(0, false);
   asset->setClickMessage(i18n("Asset account"));
   m_editWidgets["asset-account"] = asset;
-  connect(asset, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
+  connect(asset, SIGNAL(editTextChanged(QString)), this, SLOT(slotUpdateButtonState()));
   connect(asset, SIGNAL(objectCreation(bool)), this, SIGNAL(objectCreation(bool)));
 
   KMyMoneyCategory* fees = new KMyMoneyCategory(0, true);
   fees->setClickMessage(i18n("Fees"));
   m_editWidgets["fee-account"] = fees;
   connect(fees, SIGNAL(itemSelected(QString)), this, SLOT(slotUpdateFeeCategory(QString)));
-  connect(fees, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
-  connect(fees, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateFeeVisibility(QString)));
+  connect(fees, SIGNAL(editTextChanged(QString)), this, SLOT(slotUpdateButtonState()));
+  connect(fees, SIGNAL(editTextChanged(QString)), this, SLOT(slotUpdateFeeVisibility(QString)));
   connect(fees, SIGNAL(createItem(QString,QString&)), this, SLOT(slotCreateFeeCategory(QString,QString&)));
   connect(fees, SIGNAL(objectCreation(bool)), this, SIGNAL(objectCreation(bool)));
   connect(fees->splitButton(), SIGNAL(clicked()), this, SLOT(slotEditFeeSplits()));
@@ -236,15 +234,28 @@ void InvestTransactionEditor::createEditWidgets(void)
   interest->setClickMessage(i18n("Interest"));
   m_editWidgets["interest-account"] = interest;
   connect(interest, SIGNAL(itemSelected(QString)), this, SLOT(slotUpdateInterestCategory(QString)));
-  connect(interest, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
-  connect(interest, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateInterestVisibility(QString)));
+  connect(interest, SIGNAL(editTextChanged(QString)), this, SLOT(slotUpdateButtonState()));
+  connect(interest, SIGNAL(editTextChanged(QString)), this, SLOT(slotUpdateInterestVisibility(QString)));
   connect(interest, SIGNAL(createItem(QString,QString&)), this, SLOT(slotCreateInterestCategory(QString,QString&)));
   connect(interest, SIGNAL(objectCreation(bool)), this, SIGNAL(objectCreation(bool)));
   connect(interest->splitButton(), SIGNAL(clicked()), this, SLOT(slotEditInterestSplits()));
 
+  KTagContainer* tag = new KTagContainer;
+  tag->tagCombo()->setClickMessage(i18n("Tag"));
+  tag->tagCombo()->setObjectName(QLatin1String("Tag"));
+  m_editWidgets["tag"] = tag;
+  connect(tag->tagCombo(), SIGNAL(editTextChanged(QString)), this, SLOT(slotUpdateButtonState()));
+  connect(tag->tagCombo(), SIGNAL(createItem(QString,QString&)), this, SIGNAL(createTag(QString,QString&)));
+  connect(tag->tagCombo(), SIGNAL(objectCreation(bool)), this, SIGNAL(objectCreation(bool)));
+
   KTextEdit* memo = new KTextEdit;
   memo->setTabChangesFocus(true);
   m_editWidgets["memo"] = memo;
+  connect(memo, SIGNAL(textChanged()), this, SLOT(slotUpdateInvestMemoState()));
+  connect(memo, SIGNAL(textChanged()), this, SLOT(slotUpdateButtonState()));
+
+  d->m_activity->m_memoText.clear();
+  d->m_activity->m_memoChanged = false;
 
   kMyMoneyEdit* value = new kMyMoneyEdit;
   value->setClickMessage(i18n("Shares"));
@@ -377,9 +388,8 @@ int InvestTransactionEditor::editSplits(const QString& categoryWidgetName, const
     try {
       d->m_phonyAccount.setCurrencyId(m_transaction.commodity());
       d->m_phonyAccount.fraction(MyMoneyFile::instance()->security(m_transaction.commodity()));
-    } catch (MyMoneyException *e) {
+    } catch (const MyMoneyException &) {
       qDebug("Unable to setup precision");
-      delete e;
     }
 
     if (createPseudoTransaction(transaction, splits)) {
@@ -490,8 +500,8 @@ void InvestTransactionEditor::slotUpdateFeeVisibility(const QString& txt)
   QLabel* l = dynamic_cast<QLabel*>(haveWidget("fee-amount-label"));
 
   const bool hideFee = txt.isEmpty() || d->m_activity->type() == MyMoneySplit::AddShares ||
-    d->m_activity->type() == MyMoneySplit::RemoveShares ||
-    d->m_activity->type() == MyMoneySplit::SplitShares;
+                       d->m_activity->type() == MyMoneySplit::RemoveShares ||
+                       d->m_activity->type() == MyMoneySplit::SplitShares;
   //  no fee expected so hide
   if (hideFee) {
     if (l) {
@@ -517,19 +527,22 @@ void InvestTransactionEditor::slotUpdateInterestVisibility(const QString& txt)
   w->setHidden(txt.isEmpty());
   QLabel* l = dynamic_cast<QLabel*>(haveWidget("interest-amount-label"));
 
-  const bool showInterest = !txt.isEmpty() && (d->m_activity->type() == MyMoneySplit::Dividend ||
-    d->m_activity->type() == MyMoneySplit::InterestIncome || d->m_activity->type() == MyMoneySplit::Yield);
-  if (showInterest) {
+  KMyMoneyCategory* interest = dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account"));
+  const bool showInterest = !txt.isEmpty() && (d->m_activity->type() == MyMoneySplit::SellShares || d->m_activity->type() == MyMoneySplit::Dividend ||
+                            d->m_activity->type() == MyMoneySplit::InterestIncome || d->m_activity->type() == MyMoneySplit::Yield);
+  if (interest && showInterest) {
+    interest->splitButton()->show();
     w->show();
     if (l)
       l->setText(i18n("Interest"));
   } else {
     // for the following activity->types -
-    //  ReinvestDividend, BuyShares, SellShares, AddShares, RemoveShares and SplitShares
-    KMyMoneyCategory* interest = dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account"));
+    //  ReinvestDividend, BuyShares, AddShares, RemoveShares and SplitShares
     if (interest) {
       interest->splitButton()->hide();  //no interest-amount so no splits
       w->hide();
+      if (l)
+        l->setText(QString());
     }
   }
 }
@@ -541,7 +554,6 @@ void InvestTransactionEditor::slotCreateInterestCategory(const QString& name, QS
 
   emit createCategory(acc, MyMoneyFile::instance()->income());
 
-  // return id
   id = acc.id();
 }
 
@@ -612,6 +624,11 @@ void InvestTransactionEditor::loadEditWidgets(KMyMoneyRegister::Action /* action
   aSet.clear();
   aSet.load(security->selector(), i18n("Security"), m_account.accountList(), true);
 
+  // memo
+  memo->setText(m_split.memo());
+  d->m_activity->m_memoText = m_split.memo();
+  d->m_activity->m_memoChanged = false;
+
   if (!isMultiSelection()) {
     // date
     if (m_transaction.postDate().isValid())
@@ -648,9 +665,6 @@ void InvestTransactionEditor::loadEditWidgets(KMyMoneyRegister::Action /* action
     setupCategoryWidget(fees, m_feeSplits, id, SLOT(slotEditFeeSplits()));
     slotUpdateFeeVisibility(fees->currentText());
 
-    // memo
-    memo->setText(m_split.memo());
-
     // shares
     // don't set the value if the number of shares is zero so that
     // we can see the hint
@@ -685,7 +699,6 @@ void InvestTransactionEditor::loadEditWidgets(KMyMoneyRegister::Action /* action
   } else {
     postDate->loadDate(QDate());
     reconcile->setState(MyMoneySplit::Unknown);
-    memo->setText(QString());
 
     // We don't allow to change the activity
     activity->setActivity(d->m_activity->type());
@@ -772,6 +785,7 @@ void InvestTransactionEditor::slotUpdateSecurity(const QString& stockId)
   haveWidget("interest-amount")->setEnabled(currencyKnown);
 
   slotUpdateTotalAmount();
+  slotUpdateButtonState();
 }
 
 void InvestTransactionEditor::totalAmount(MyMoneyMoney& amount) const
@@ -1024,7 +1038,7 @@ bool InvestTransactionEditor::createTransaction(MyMoneyTransaction& t, const MyM
   //       by the user
   KTextEdit* memo = dynamic_cast<KTextEdit*>(m_editWidgets["memo"]);
   if (memo) {
-    if (!isMultiSelection() || (isMultiSelection() && !memo->toPlainText().isEmpty()))
+    if (!isMultiSelection() || (isMultiSelection() && d->m_activity->m_memoChanged))
       s0.setMemo(memo->toPlainText());
   }
 
@@ -1145,5 +1159,10 @@ void InvestTransactionEditor::setupFinalWidgets(void)
   addFinalWidget(haveWidget("memo"));
 }
 
-#include "investtransactioneditor.moc"
-
+void InvestTransactionEditor::slotUpdateInvestMemoState(void)
+{
+  KTextEdit* memo = dynamic_cast<KTextEdit*>(m_editWidgets["memo"]);
+  if (memo) {
+    d->m_activity->m_memoChanged = (memo->toPlainText() != d->m_activity->m_memoText);
+  }
+}
