@@ -33,7 +33,6 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kpushbutton.h>
-#include <kstdguiitem.h>
 #include <knuminput.h>
 #include <klineedit.h>
 #include <KToolInvocation>
@@ -201,10 +200,10 @@ TransactionEditor* KEditScheduleDlg::startEdit(void)
     connect(MyMoneyFile::instance(), SIGNAL(dataChanged()), editor, SLOT(slotReloadEditWidgets()));
     // connect(editor, SIGNAL(finishEdit(KMyMoneyRegister::SelectedTransactions)), this, SLOT(slotLeaveEditMode(KMyMoneyRegister::SelectedTransactions)));
     connect(editor, SIGNAL(createPayee(QString,QString&)), kmymoney, SLOT(slotPayeeNew(QString,QString&)));
+    connect(editor, SIGNAL(createTag(QString,QString&)), kmymoney, SLOT(slotTagNew(QString,QString&)));
     connect(editor, SIGNAL(createCategory(MyMoneyAccount&,MyMoneyAccount)), kmymoney, SLOT(slotCategoryNew(MyMoneyAccount&,MyMoneyAccount)));
     connect(editor, SIGNAL(createSecurity(MyMoneyAccount&,MyMoneyAccount)), kmymoney, SLOT(slotInvestmentNew(MyMoneyAccount&,MyMoneyAccount)));
     connect(MyMoneyFile::instance(), SIGNAL(dataChanged()), editor, SLOT(slotReloadEditWidgets()));
-    connect(editor, SIGNAL(operationTypeChanged(int)), this, SLOT(slotFilterPaymentType(int)));
 
     // create the widgets, place them in the parent and load them with data
     // setup tab order
@@ -301,13 +300,13 @@ TransactionEditor* KEditScheduleDlg::startEdit(void)
     if (dateEdit)
       connect(dateEdit, SIGNAL(dateChanged(QDate)), this, SLOT(slotPostDateChanged(QDate)));
 
-
     m_nameEdit->setFocus();
 
     // add the required fields to the mandatory group
     d->m_requiredFields->add(m_nameEdit);
     d->m_requiredFields->add(editor->haveWidget("account"));
     d->m_requiredFields->add(editor->haveWidget("category"));
+    d->m_requiredFields->add(editor->haveWidget("amount"));
 
     // fix labels
     QLabel* label = dynamic_cast<QLabel*>(editor->haveWidget("date-label"));
@@ -319,6 +318,7 @@ TransactionEditor* KEditScheduleDlg::startEdit(void)
     slotSetPaymentMethod(d->m_schedule.paymentType());
 
     connect(m_paymentMethodEdit, SIGNAL(itemSelected(int)), this, SLOT(slotSetPaymentMethod(int)));
+    connect(editor, SIGNAL(operationTypeChanged(int)), this, SLOT(slotFilterPaymentType(int)));
   }
 
   return editor;
@@ -602,5 +602,3 @@ void KEditScheduleDlg::slotFilterPaymentType(int index)
   }
 
 }
-
-#include <keditscheduledlg.moc>
