@@ -49,7 +49,6 @@
 #include <kcompletionbox.h>
 #include <kpushbutton.h>
 #include <kmenu.h>
-#include <kstdaccel.h>
 #include <kshortcut.h>
 
 // ----------------------------------------------------------------------------
@@ -71,6 +70,7 @@ kMyMoneySplitTable::kMyMoneySplitTable(QWidget *parent) :
     QTableWidget(parent),
     m_currentRow(0),
     m_maxRows(0),
+    m_precision(2),
     m_editCategory(0),
     m_editMemo(0),
     m_editAmount(0)
@@ -111,11 +111,11 @@ kMyMoneySplitTable::kMyMoneySplitTable(QWidget *parent) :
   // setup the context menu
   m_contextMenu = new KMenu(this);
   m_contextMenu->setTitle(i18n("Split Options"));
-  m_contextMenu->setIcon(KIcon("transaction"));
+  m_contextMenu->setIcon(KIcon("view-financial-transfer"));
   m_contextMenu->addAction(KIcon("document-edit"), i18n("Edit..."), this, SLOT(slotStartEdit()));
   m_contextMenuDuplicate = m_contextMenu->addAction(KIcon("edit-copy"), i18nc("To duplicate a split", "Duplicate"), this, SLOT(slotDuplicateSplit()));
   m_contextMenuDelete = m_contextMenu->addAction(KIcon("edit-delete"),
-                        i18n("Delete ..."),
+                        i18n("Delete..."),
                         this, SLOT(slotDeleteSplit()));
 
   connect(this, SIGNAL(clicked(QModelIndex)),
@@ -140,9 +140,10 @@ int kMyMoneySplitTable::currentRow() const
   return m_currentRow;
 }
 
-void kMyMoneySplitTable::setup(const QMap<QString, MyMoneyMoney>& priceInfo)
+void kMyMoneySplitTable::setup(const QMap<QString, MyMoneyMoney>& priceInfo, int precision)
 {
   m_priceInfo = priceInfo;
+  m_precision = precision;
 }
 
 bool kMyMoneySplitTable::eventFilter(QObject *o, QEvent *e)
@@ -153,11 +154,6 @@ bool kMyMoneySplitTable::eventFilter(QObject *o, QEvent *e)
   int row = currentRow();
   int lines = viewport()->height() / rowHeight(0);
 
-  if (o == this && e->type() == QEvent::Resize && isEditMode()) {
-    rc = false;
-    endEdit(false);
-  }
-
   if (e->type() == QEvent::KeyPress && !isEditMode()) {
     rc = true;
     switch (k->key()) {
@@ -353,8 +349,8 @@ void kMyMoneySplitTable::slotSetFocus(const QModelIndex& index, int button)
   scrollTo(model()->index(row, 0));
 
   if (isEditMode()) {                   // in edit mode?
-    if (KMyMoneyGlobalSettings::focusChangeIsEnter())
-      slotEndEdit();
+    if (isEditSplitValid() && KMyMoneyGlobalSettings::focusChangeIsEnter())
+      endEdit(false/*keyboard driven*/, false/*set focus to next row*/);
     else
       slotCancelEdit();
   }
@@ -403,7 +399,7 @@ void kMyMoneySplitTable::mouseDoubleClickEvent(QMouseEvent *e)
   slotSetFocus(model()->index(rowAt(e->pos().y()), col), e->button());
   createEditWidgets(false);
 
-  QLineEdit* editWidget = 0;
+  QLineEdit* editWidget = 0;    //krazy:exclude=qmethods
   switch (col) {
     case 0:
       editWidget = m_editCategory->lineEdit();
@@ -506,9 +502,8 @@ void kMyMoneySplitTable::slotUpdateData(const MyMoneyTransaction& t)
     if (!(*it).accountId().isEmpty()) {
       try {
         colText = MyMoneyFile::instance()->accountToCategory((*it).accountId());
-      } catch (MyMoneyException *e) {
+      } catch (const MyMoneyException &) {
         qDebug("Unexpected exception in kMyMoneySplitTable::slotUpdateData()");
-        delete e;
       }
     }
     QString amountTxt = value.formatMoney(m_account.fraction());
@@ -583,7 +578,11 @@ void kMyMoneySplitTable::updateTransactionTableSize(void)
 void kMyMoneySplitTable::resizeEvent(QResizeEvent* ev)
 {
   QTableWidget::resizeEvent(ev);
-  updateTransactionTableSize();
+  if (!isEditMode()) {
+    // update the size of the transaction table only if a split is not being edited
+    // otherwise the height of the editors would be altered in an undesired way
+    updateTransactionTableSize();
+  }
 }
 
 void kMyMoneySplitTable::slotDuplicateSplit(void)
@@ -596,9 +595,8 @@ void kMyMoneySplitTable::slotDuplicateSplit(void)
     try {
       m_transaction.addSplit(split);
       emit transactionChanged(m_transaction);
-    } catch (MyMoneyException *e) {
-      qDebug("Cannot duplicate split: %s", qPrintable(e->what()));
-      delete e;
+    } catch (const MyMoneyException &e) {
+      qDebug("Cannot duplicate split: %s", qPrintable(e.what()));
     }
   }
 }
@@ -622,9 +620,8 @@ void kMyMoneySplitTable::slotDeleteSplit(void)
         else
           selectRow(m_currentRow);
         emit transactionChanged(m_transaction);
-      } catch (MyMoneyException *e) {
-        qDebug("Cannot remove split: %s", qPrintable(e->what()));
-        delete e;
+      } catch (const MyMoneyException &e) {
+        qDebug("Cannot remove split: %s", qPrintable(e.what()));
       }
     }
   }
@@ -646,14 +643,14 @@ void kMyMoneySplitTable::slotEndEditKeyboard(void)
   endEdit(true);
 }
 
-void kMyMoneySplitTable::endEdit(bool keyBoardDriven)
+void kMyMoneySplitTable::endEdit(bool keyboardDriven, bool setFocusToNextRow)
 {
   MyMoneyFile* file = MyMoneyFile::instance();
 
   MYMONEYTRACER(tracer);
   MyMoneySplit s1 = m_split;
 
-  if (m_editCategory->selectedItem().isEmpty()) {
+  if (!isEditSplitValid()) {
     KMessageBox::information(this, i18n("You need to assign a category to this split before it can be entered."), i18n("Enter split"), "EnterSplitWithEmptyCategory");
     m_editCategory->setFocus();
     return;
@@ -740,18 +737,19 @@ void kMyMoneySplitTable::endEdit(bool keyBoardDriven)
         m_transaction.modifySplit(m_split);
       }
       emit transactionChanged(m_transaction);
-    } catch (MyMoneyException *e) {
-      qDebug("Cannot add/modify split: %s", qPrintable(e->what()));
-      delete e;
+    } catch (const MyMoneyException &e) {
+      qDebug("Cannot add/modify split: %s", qPrintable(e.what()));
     }
   }
   this->setFocus();
   destroyEditWidgets();
-  slotSetFocus(model()->index(currentRow() + 1, 0));
+  if (setFocusToNextRow) {
+    slotSetFocus(model()->index(currentRow() + 1, 0));
+  }
 
   // if we still have more splits, we start editing right away
   // in case we have selected 'enter moves between fields'
-  if (keyBoardDriven
+  if (keyboardDriven
       && currentRow() < m_transaction.splits().count() - 1
       && KMyMoneyGlobalSettings::enterMovesBetweenFields()) {
     slotStartEdit();
@@ -770,7 +768,13 @@ void kMyMoneySplitTable::slotCancelEdit(void)
 
 bool kMyMoneySplitTable::isEditMode(void) const
 {
-  return state() == QAbstractItemView::EditingState;
+  // while the edit widgets exist we're in edit mode
+  return m_editAmount || m_editMemo || m_editCategory;
+}
+
+bool kMyMoneySplitTable::isEditSplitValid() const
+{
+  return isEditMode() && !m_editCategory->selectedItem().isEmpty();
 }
 
 void kMyMoneySplitTable::destroyEditWidgets(void)
@@ -783,8 +787,6 @@ void kMyMoneySplitTable::destroyEditWidgets(void)
   destroyEditWidget(m_currentRow, 1);
   destroyEditWidget(m_currentRow, 2);
   destroyEditWidget(m_currentRow + 1, 0);
-  setState(QAbstractItemView::NoState);
-  QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
 }
 
 void kMyMoneySplitTable::destroyEditWidget(int r, int c)
@@ -805,6 +807,7 @@ KMyMoneyCategory* kMyMoneySplitTable::createEditWidgets(bool setFocus)
   m_editAmount = new kMyMoneyEdit(0);
   m_editAmount->setFont(cellFont);
   m_editAmount->setResetButtonVisible(false);
+  m_editAmount->setPrecision(m_precision);
 
   m_editCategory = new KMyMoneyCategory();
   m_editCategory->setClickMessage(i18n("Category"));
@@ -818,13 +821,13 @@ KMyMoneyCategory* kMyMoneySplitTable::createEditWidgets(bool setFocus)
 
   // create buttons for the mouse users
   m_registerButtonFrame = new QFrame(this);
-  QPalette palette = m_registerButtonFrame->palette();
-  //palette.setColor(QPalette::Background, rowBackgroundColor(m_currentRow+1) );
-  m_registerButtonFrame->setPalette(palette);
+  m_registerButtonFrame->setContentsMargins(0, 0, 0, 0);
+  m_registerButtonFrame->setAutoFillBackground(true);
 
   QHBoxLayout* l = new QHBoxLayout(m_registerButtonFrame);
+  l->setContentsMargins(0, 0, 0, 0);
+  l->setSpacing(0);
   m_registerEnterButton = new KPushButton(KIcon("dialog-ok"), QString(), m_registerButtonFrame);
-
   m_registerCancelButton = new KPushButton(KIcon("dialog-cancel"), QString(), m_registerButtonFrame);
 
   l->addWidget(m_registerEnterButton);
@@ -884,8 +887,6 @@ KMyMoneyCategory* kMyMoneySplitTable::createEditWidgets(bool setFocus)
     m_editCategory->lineEdit()->selectAll();
   }
 
-  setState(QAbstractItemView::EditingState);
-
   // resize the rows so the added edit widgets would fit appropriately
   resizeRowsToContents();
 
@@ -966,5 +967,3 @@ bool kMyMoneySplitTable::focusNextPrevChild(bool next)
     rc = QTableWidget::focusNextPrevChild(next);
   return rc;
 }
-
-#include "kmymoneysplittable.moc"
