@@ -15,8 +15,6 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <config-kmymoney.h>
-
 #include "knewaccountdlg.h"
 
 // ----------------------------------------------------------------------------
@@ -118,7 +116,7 @@ QModelIndex HierarchyFilterProxyModel::getSelectedParentAccountIndex() const
 bool HierarchyFilterProxyModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
 {
   if (!source_parent.isValid()) {
-    QVariant data = sourceModel()->data(sourceModel()->index(source_row, 0, source_parent), AccountsModel::AccountIdRole);
+    QVariant data = sourceModel()->index(source_row, 0, source_parent).data(AccountsModel::AccountIdRole);
     if (data.isValid() && data.toString() == AccountsModel::favoritesAccountId)
       return false;
   }
@@ -396,9 +394,8 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
       institutionName = file->institution(account.institutionId()).name();
     else
       institutionName.clear();
-  } catch (MyMoneyException *e) {
-    qDebug("exception in init for account dialog: %s", qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    qDebug("exception in init for account dialog: %s", qPrintable(e.what()));
   }
 
   if (m_account.isInvest())
@@ -472,8 +469,7 @@ KNewAccountDlg::KNewAccountDlg(const MyMoneyAccount& account, bool isEditing, bo
           m_grossAmount->setChecked(true);
           if (m_account.value("VatAmount") == "Net")
             m_netAmount->setChecked(true);
-        } catch (MyMoneyException *e) {
-          delete e;
+        } catch (const MyMoneyException &) {
         }
       }
     }
@@ -494,6 +490,7 @@ void KNewAccountDlg::setOpeningBalance(const MyMoneyMoney& balance)
 
 void KNewAccountDlg::setOpeningBalanceShown(bool shown)
 {
+  m_openingBalanceLabel->setVisible(shown);
   m_openingBalanceEdit->setVisible(shown);
 }
 
@@ -526,9 +523,8 @@ void KNewAccountDlg::okClicked()
           if ((*institutionIterator).name() == institutionNameText)
             m_account.setInstitutionId((*institutionIterator).id());
         }
-      } catch (MyMoneyException *e) {
-        qDebug("Exception in account institution set: %s", qPrintable(e->what()));
-        delete e;
+      } catch (const MyMoneyException &e) {
+        qDebug("Exception in account institution set: %s", qPrintable(e.what()));
       }
     } else {
       m_account.setInstitutionId(QString());
@@ -756,9 +752,8 @@ void KNewAccountDlg::slotLoadInstitutions(const QString& name)
     }
 
     m_qcomboboxInstitutions->setCurrentItem(name, false);
-  } catch (MyMoneyException *e) {
-    qDebug("Exception in institution load: %s", qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    qDebug("Exception in institution load: %s", qPrintable(e.what()));
   }
 }
 
@@ -776,8 +771,7 @@ void KNewAccountDlg::slotNewClicked()
       file->addInstitution(institution);
       ft.commit();
       slotLoadInstitutions(institution.name());
-    } catch (MyMoneyException *e) {
-      delete e;
+    } catch (const MyMoneyException &) {
       KMessageBox::information(this, i18n("Cannot add institution"));
     }
   }
@@ -788,37 +782,17 @@ void KNewAccountDlg::slotAccountTypeChanged(const QString& typeStr)
 {
   MyMoneyAccount::accountTypeE type;
   MyMoneyAccount::accountTypeE oldType;
-  MyMoneyFile* file = MyMoneyFile::instance();
 
   type = KMyMoneyUtils::stringToAccountType(typeStr);
   try {
     oldType = m_account.accountType();
     if (oldType != type) {
-      QString parentId;
-      switch (MyMoneyAccount::accountGroup(type)) {
-        case MyMoneyAccount::Asset:
-          parentId = file->asset().id();
-          break;
-        case MyMoneyAccount::Liability:
-          parentId = file->liability().id();
-          break;
-        case MyMoneyAccount::Expense:
-          parentId = file->expense().id();
-          break;
-        case MyMoneyAccount::Income:
-          parentId = file->income().id();
-          break;
-        default:
-          qWarning("Unknown account group in KNewAccountDlg::slotAccountTypeChanged()");
-          break;
-      }
       m_account.setAccountType(type);
       // update the account group displayed in the accounts hierarchy
       m_filterProxyModel->clear();
       m_filterProxyModel->addAccountGroup(m_account.accountGroup());
     }
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     qWarning("Unexpected exception in KNewAccountDlg::slotAccountTypeChanged()");
   }
 }
@@ -905,7 +879,3 @@ void KNewAccountDlg::addTab(QWidget* w, const QString& name)
     m_tab->addTab(w, name);
   }
 }
-
-
-#include "knewaccountdlg.moc"
-// vim:cin:si:ai:et:ts=2:sw=2:
