@@ -38,7 +38,6 @@
 #include <kcombobox.h>
 #include <kpushbutton.h>
 #include <kmessagebox.h>
-#include <kstdguiitem.h>
 #include <kiconloader.h>
 #include <kguiitem.h>
 
@@ -81,6 +80,13 @@ TransactionEditor::~TransactionEditor()
 {
   // Make sure the widgets do not send out signals to the editor anymore
   // After all, the editor is about to die
+
+  //disconnect first tagCombo:
+  KTagContainer *w = dynamic_cast<KTagContainer*>(haveWidget("tag"));
+  if (w && w->tagCombo()) {
+    w->tagCombo()->disconnect(this);
+  }
+
   QMap<QString, QWidget*>::iterator it_w;
   for (it_w = m_editWidgets.begin(); it_w != m_editWidgets.end(); ++it_w) {
     (*it_w)->disconnect(this);
@@ -125,8 +131,13 @@ void TransactionEditor::setup(QWidgetList& tabOrderWidgets, const MyMoneyAccount
   m_regForm->arrangeEditWidgets(m_editWidgets, m_item);
   m_regForm->tabOrder(tabOrderWidgets, m_item);
   QWidget* w = haveWidget("tabbar");
-  if (w)
+  if (w) {
     tabOrderWidgets.append(w);
+    TabBar* tabbar = dynamic_cast<TabBar*>(w);
+    if ((tabbar) && (action == KMyMoneyRegister::ActionNone)) {
+      action = static_cast<KMyMoneyRegister::Action>(tabbar->currentIndex());
+    }
+  }
   loadEditWidgets(action);
 
   // remove all unused widgets and don't forget to remove them
@@ -249,17 +260,28 @@ bool TransactionEditor::eventFilter(QObject* o, QEvent* e)
 
 void TransactionEditor::slotNumberChanged(const QString& txt)
 {
+  QString next = txt;
   kMyMoneyLineEdit* number = dynamic_cast<kMyMoneyLineEdit*>(haveWidget("number"));
 
-  if (number) {
-    if (MyMoneyFile::instance()->checkNoUsed(m_account.id(), txt)) {
-      if (KMessageBox::questionYesNo(m_regForm, QString("<qt>") + i18n("The number <b>%1</b> has already been used in account <b>%2</b>. Do you want to replace it with the next available number?", txt, m_account.name()) + QString("</qt>"), i18n("Duplicate number")) == KMessageBox::Yes) {
-        number->loadText(KMyMoneyUtils::nextCheckNumber(m_account));
-      }
+  while (MyMoneyFile::instance()->checkNoUsed(m_account.id(), next)) {
+    if (KMessageBox::questionYesNo(m_regForm, QString("<qt>") + i18n("The number <b>%1</b> has already been used in account <b>%2</b>.""<center>Do you want to replace it with the next available number?</center>", next, m_account.name()) + QString("</qt>"), i18n("Duplicate number")) == KMessageBox::Yes) {
+      assignNextNumber();
+      next = KMyMoneyUtils::nextCheckNumber(m_account);
+    } else {
+      number->setText(QString());
+      break;
     }
   }
 }
 
+void TransactionEditor::slotUpdateMemoState(void)
+{
+  KTextEdit* memo = dynamic_cast<KTextEdit*>(m_editWidgets["memo"]);
+  if (memo) {
+    m_memoChanged = (memo->toPlainText() != m_memoText);
+  }
+}
+
 void TransactionEditor::slotUpdateButtonState(void)
 {
   QString reason;
@@ -319,9 +341,8 @@ bool TransactionEditor::fixTransactionCommodity(const MyMoneyAccount& account)
             splitB.setValue(splitB.shares());
             (*it_t).transaction().modifySplit(splitB);
 
-          } catch (MyMoneyException *e) {
-            qDebug("Unable to update commodity to second splits currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e->what()));
-            delete e;
+          } catch (const MyMoneyException &e) {
+            qDebug("Unable to update commodity to second splits currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e.what()));
           }
           break;
 
@@ -356,9 +377,8 @@ bool TransactionEditor::fixTransactionCommodity(const MyMoneyAccount& account)
             (*it_t).transaction().modifySplit(splitA);
             (*it_t).transaction().modifySplit(splitB);
 
-          } catch (MyMoneyException *e) {
-            qDebug("Unable to update commodity to second splits currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e->what()));
-            delete e;
+          } catch (const MyMoneyException &e) {
+            qDebug("Unable to update commodity to second splits currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e.what()));
           }
           break;
 
@@ -402,9 +422,8 @@ bool TransactionEditor::fixTransactionCommodity(const MyMoneyAccount& account)
                 (*it_t).transaction().modifySplit(s);
               }
             }
-          } catch (MyMoneyException *e) {
-            qDebug("Unable to update commodity of split currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e->what()));
-            delete e;
+          } catch (const MyMoneyException &e) {
+            qDebug("Unable to update commodity of split currency in %s: '%s'", qPrintable((*it_t).transaction().id()), qPrintable(e.what()));
           }
           break;
       }
@@ -425,14 +444,32 @@ void TransactionEditor::assignNextNumber(void)
 {
   if (canAssignNumber()) {
     kMyMoneyLineEdit* number = dynamic_cast<kMyMoneyLineEdit*>(haveWidget("number"));
-    number->loadText(KMyMoneyUtils::nextCheckNumber(m_account));
+    QString num = KMyMoneyUtils::nextCheckNumber(m_account);
+    bool showMessage = true;
+    int rc = KMessageBox::No;
+    while (MyMoneyFile::instance()->checkNoUsed(m_account.id(), num)) {
+      if (showMessage) {
+        rc = KMessageBox::questionYesNo(m_regForm, QString("<qt>") + i18n("The expected next check number <b>%1</b> has already been used in account <b>%2</b>." "<center>Do you want to replace it with the next available number?</center>", num, m_account.name()) + QString("</qt>"), i18n("Duplicate number"));
+        showMessage = false;
+      }
+      if (rc == KMessageBox::Yes) {
+        num = KMyMoneyUtils::nextCheckNumber(m_account);
+        KMyMoneyUtils::updateLastNumberUsed(m_account, num);
+        m_account.setValue("lastNumberUsed", num);
+        number->loadText(num);
+      } else {
+        num = QString();
+        break;
+      }
+    }
+    number->setText(num);
   }
 }
 
 bool TransactionEditor::canAssignNumber(void) const
 {
   kMyMoneyLineEdit* number = dynamic_cast<kMyMoneyLineEdit*>(haveWidget("number"));
-  return (number != 0) && (number->text().isEmpty());
+  return (number != 0);
 }
 
 void TransactionEditor::setupCategoryWidget(KMyMoneyCategory* category, const QList<MyMoneySplit>& splits, QString& categoryId, const char* splitEditSlot, bool /* allowObjectCreation */)
@@ -585,14 +622,7 @@ bool TransactionEditor::enterTransactions(QString& newId, bool askForSchedule, b
             t = (*it_ts);
 
             // if a new transaction has a valid number, keep it with the account
-            QString number;
-            if (!(*it_ts).splits().isEmpty())
-              number = (*it_ts).splits().front().number();
-            if (!number.isEmpty()) {
-              m_account.setValue("lastNumberUsed", number);
-              file->modifyAccount(m_account);
-            }
-
+            keepNewNumber((*it_ts));
           } else {
             // turn object creation on, so that moving the focus does
             // not screw up the dialog that might be popping up
@@ -607,6 +637,9 @@ bool TransactionEditor::enterTransactions(QString& newId, bool askForSchedule, b
           emit lastPostDateUsed((*it_ts).postDate());
         } else {
           // modify existing transaction
+          // its number might have been edited
+          // bearing in mind it could contain alpha characters
+          keepNewNumber((*it_ts));
           file->modifyTransaction(*it_ts);
         }
       }
@@ -689,9 +722,8 @@ bool TransactionEditor::enterTransactions(QString& newId, bool askForSchedule, b
           }
         }
       }
-    } catch (MyMoneyException * e) {
-      qDebug("Unable to store transaction within engine: %s", qPrintable(e->what()));
-      delete e;
+    } catch (const MyMoneyException &e) {
+      qDebug("Unable to store transaction within engine: %s", qPrintable(e.what()));
       newTransactionCreated = false;
     }
 
@@ -702,6 +734,28 @@ bool TransactionEditor::enterTransactions(QString& newId, bool askForSchedule, b
   return storeTransactions;
 }
 
+void TransactionEditor::keepNewNumber(const MyMoneyTransaction& tr)
+{
+  // verify that new number, possibly containing alpha, is valid
+  MyMoneyTransaction txn = tr;
+  MyMoneyFile* file = MyMoneyFile::instance();
+  if (!txn.splits().isEmpty()) {
+    QString number = txn.splits().first().number();
+    if (KMyMoneyUtils::numericPart(number) > 0) {
+      // numeric is valid
+      kMyMoneyLineEdit* numberEdit = dynamic_cast<kMyMoneyLineEdit*>(haveWidget("number"));
+      if (numberEdit) {
+        numberEdit->loadText(number);
+        MyMoneySplit split = txn.splits().first();
+        split.setNumber(number);
+        txn.modifySplit(split);
+        m_account.setValue("lastNumberUsed", number);
+        file->modifyAccount(m_account);
+      }
+    }
+  }
+}
+
 void TransactionEditor::resizeForm(void)
 {
   // force resizeing of the columns in the form
@@ -735,7 +789,7 @@ void StdTransactionEditor::createEditWidgets(void)
     account->setClickMessage(i18n("Account"));
     account->setObjectName(QLatin1String("Account"));
     m_editWidgets["account"] = account;
-    connect(account, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
+    connect(account, SIGNAL(editTextChanged(QString)), this, SLOT(slotUpdateButtonState()));
     connect(account, SIGNAL(itemSelected(QString)), this, SLOT(slotUpdateAccount(QString)));
   }
 
@@ -743,17 +797,18 @@ void StdTransactionEditor::createEditWidgets(void)
   payee->setClickMessage(i18n("Payer/Receiver"));
   payee->setObjectName(QLatin1String("Payee"));
   m_editWidgets["payee"] = payee;
-  connect(payee, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
+
   connect(payee, SIGNAL(createItem(QString,QString&)), this, SIGNAL(createPayee(QString,QString&)));
   connect(payee, SIGNAL(objectCreation(bool)), this, SIGNAL(objectCreation(bool)));
   connect(payee, SIGNAL(itemSelected(QString)), this, SLOT(slotUpdatePayee(QString)));
+  connect(payee, SIGNAL(editTextChanged(QString)), this, SLOT(slotUpdateButtonState()));
 
   KMyMoneyCategory* category = new KMyMoneyCategory(0, true);
   category->setClickMessage(i18n("Category/Account"));
   category->setObjectName(QLatin1String("Category/Account"));
   m_editWidgets["category"] = category;
   connect(category, SIGNAL(itemSelected(QString)), this, SLOT(slotUpdateCategory(QString)));
-  connect(category, SIGNAL(textChanged(QString)), this, SLOT(slotUpdateButtonState()));
+  connect(category, SIGNAL(editTextChanged(QString)), this, SLOT(slotUpdateButtonState()));
   connect(category, SIGNAL(createItem(QString,QString&)), this, SLOT(slotCreateCategory(QString,QString&)));
   connect(category, SIGNAL(objectCreation(bool)), this, SIGNAL(objectCreation(bool)));
   connect(category->splitButton(), SIGNAL(clicked()), this, SLOT(slotEditSplits()));
@@ -761,11 +816,21 @@ void StdTransactionEditor::createEditWidgets(void)
   if (category->splitButton())
     category->splitButton()->setDisabled(m_account.id().isEmpty());
 
+  KTagContainer* tag = new KTagContainer;
+  tag->tagCombo()->setClickMessage(i18n("Tag"));
+  tag->tagCombo()->setObjectName(QLatin1String("Tag"));
+  m_editWidgets["tag"] = tag;
+  connect(tag->tagCombo(), SIGNAL(createItem(QString,QString&)), this, SIGNAL(createTag(QString,QString&)));
+  connect(tag->tagCombo(), SIGNAL(objectCreation(bool)), this, SIGNAL(objectCreation(bool)));
+
   KTextEdit* memo = new KTextEdit;
   memo->setObjectName(QLatin1String("Memo"));
   memo->setTabChangesFocus(true);
+  connect(memo, SIGNAL(textChanged()), this, SLOT(slotUpdateMemoState()));
   connect(memo, SIGNAL(textChanged()), this, SLOT(slotUpdateButtonState()));
   m_editWidgets["memo"] = memo;
+  m_memoText.clear();
+  m_memoChanged = false;
 
   bool showNumberField = true;
   switch (m_account.accountType()) {
@@ -800,8 +865,8 @@ void StdTransactionEditor::createEditWidgets(void)
   kMyMoneyDateInput* postDate = new kMyMoneyDateInput;
   m_editWidgets["postdate"] = postDate;
   postDate->setObjectName(QLatin1String("PostDate"));
-  postDate->setDate(QDate());
   connect(postDate, SIGNAL(dateChanged(QDate)), this, SLOT(slotUpdateButtonState()));
+  postDate->setDate(QDate());
 
   kMyMoneyEdit* value = new kMyMoneyEdit;
   m_editWidgets["amount"] = value;
@@ -828,12 +893,10 @@ void StdTransactionEditor::createEditWidgets(void)
   m_editWidgets["cashflow"] = cashflow;
   cashflow->setObjectName(QLatin1String("Cashflow"));
   connect(cashflow, SIGNAL(directionSelected(KMyMoneyRegister::CashFlowDirection)), this, SLOT(slotUpdateCashFlow(KMyMoneyRegister::CashFlowDirection)));
-  connect(cashflow, SIGNAL(directionSelected(KMyMoneyRegister::CashFlowDirection)), this, SLOT(slotUpdateButtonState()));
 
   KMyMoneyReconcileCombo* reconcile = new KMyMoneyReconcileCombo;
   m_editWidgets["status"] = reconcile;
   reconcile->setObjectName(QLatin1String("Reconcile"));
-  connect(reconcile, SIGNAL(itemSelected(QString)), this, SLOT(slotUpdateButtonState()));
 
   KMyMoneyRegister::QWidgetContainer::iterator it_w;
   for (it_w = m_editWidgets.begin(); it_w != m_editWidgets.end(); ++it_w) {
@@ -853,6 +916,9 @@ void StdTransactionEditor::createEditWidgets(void)
   m_editWidgets["category-label"] = label = new QLabel(i18n("Category"));
   label->setAlignment(Qt::AlignVCenter);
 
+  m_editWidgets["tag-label"] = label = new QLabel(i18n("Tags"));
+  label->setAlignment(Qt::AlignVCenter);
+
   m_editWidgets["memo-label"] = label = new QLabel(i18n("Memo"));
   label->setAlignment(Qt::AlignVCenter);
 
@@ -931,6 +997,11 @@ void StdTransactionEditor::loadEditWidgets(KMyMoneyRegister::Action action)
   KMyMoneyCategory* category = dynamic_cast<KMyMoneyCategory*>(m_editWidgets["category"]);
   disconnect(category, SIGNAL(focusIn()), this, SLOT(slotEditSplits()));
 
+  // load the tag widget
+  //KMyMoneyTagCombo* tag = dynamic_cast<KMyMoneyTagCombo*>(m_editWidgets["tag"]);
+  KTagContainer* tag = dynamic_cast<KTagContainer*>(m_editWidgets["tag"]);
+  tag->loadTags(MyMoneyFile::instance()->tagList());
+
   // check if the current transaction has a reference to an equity account
   bool haveEquityAccount = false;
   QList<MyMoneySplit>::const_iterator it_s;
@@ -959,8 +1030,13 @@ void StdTransactionEditor::loadEditWidgets(KMyMoneyRegister::Action action)
   if (!m_account.id().isEmpty())
     category->selector()->removeItem(m_account.id());
 
+  //  also show memo text if isMultiSelection()
+  dynamic_cast<KTextEdit*>(m_editWidgets["memo"])->setText(m_split.memo());
+  // need to know if it changed
+  m_memoText = m_split.memo();
+  m_memoChanged = false;
+
   if (!isMultiSelection()) {
-    dynamic_cast<KTextEdit*>(m_editWidgets["memo"])->setText(m_split.memo());
     if (m_transaction.postDate().isValid())
       dynamic_cast<kMyMoneyDateInput*>(m_editWidgets["postdate"])->setDate(m_transaction.postDate());
     else if (m_lastPostDate.isValid())
@@ -984,6 +1060,11 @@ void StdTransactionEditor::loadEditWidgets(KMyMoneyRegister::Action action)
     if (!payeeId.isEmpty()) {
       payee->setSelectedItem(payeeId);
     }
+    QList<QString> t = m_split.tagIdList();
+    if (!t.isEmpty()) {
+      for (int i = 0; i < t.size(); i++)
+        tag->addTagWidget(t[i]);
+    }
 
     m_splits.clear();
     if (m_transaction.splitCount() < 2) {
@@ -1001,7 +1082,7 @@ void StdTransactionEditor::loadEditWidgets(KMyMoneyRegister::Action action)
 
     if ((w = haveWidget("cashflow")) != 0) {
       KMyMoneyCashFlowCombo* cashflow = dynamic_cast<KMyMoneyCashFlowCombo*>(w);
-      cashflow->setDirection(m_split.value().isNegative() ? KMyMoneyRegister::Payment : KMyMoneyRegister::Deposit);
+      cashflow->setDirection(!m_split.value().isPositive() ? KMyMoneyRegister::Payment : KMyMoneyRegister::Deposit);  //  include isZero case
     }
 
     if ((w = haveWidget("category-label")) != 0) {
@@ -1071,7 +1152,7 @@ void StdTransactionEditor::loadEditWidgets(KMyMoneyRegister::Action action)
       }
     }
 
-  } else {
+  } else {  //  isMultiSelection()
     dynamic_cast<kMyMoneyDateInput*>(m_editWidgets["postdate"])->loadDate(QDate());
     dynamic_cast<KMyMoneyReconcileCombo*>(m_editWidgets["status"])->setState(MyMoneySplit::Unknown);
     if (haveWidget("deposit")) {
@@ -1085,10 +1166,8 @@ void StdTransactionEditor::loadEditWidgets(KMyMoneyRegister::Action action)
       dynamic_cast<kMyMoneyEdit*>(w)->setAllowEmpty();
     }
 
-    if ((w = haveWidget("cashflow")) != 0) {
-      KMyMoneyCashFlowCombo* cashflow = dynamic_cast<KMyMoneyCashFlowCombo*>(w);
-      cashflow->setDirection(KMyMoneyRegister::Unknown);
-    }
+    slotUpdateAction(action);
+
     if ((w = haveWidget("tabbar")) != 0) {
       w->setEnabled(false);
     }
@@ -1142,6 +1221,17 @@ void StdTransactionEditor::slotReloadEditWidgets(void)
   if (!payeeId.isEmpty()) {
     payee->setSelectedItem(payeeId);
   }
+
+  // reload tag widget
+  KTagContainer* tag = dynamic_cast<KTagContainer*>(m_editWidgets["tag"]);
+  QString tagId = tag->tagCombo()->selectedItem();
+
+  tag->loadTags(MyMoneyFile::instance()->tagList());
+
+  if (!tagId.isEmpty()) {
+    tag->RemoveAllTagWidgets();
+    tag->addTagWidget(tagId);
+  }
 }
 
 void StdTransactionEditor::slotUpdatePayee(const QString& payeeId)
@@ -1355,12 +1445,13 @@ void StdTransactionEditor::autoFill(const QString& payeeId)
           s.setNumber(KMyMoneyUtils::nextCheckNumber(m_account));
         }
 
+        // if the memos should not be used with autofill or
         // if the transaction has exactly two splits, remove
         // the memo text of the split that does not reference
         // the current account. This allows the user to change
         // the autofilled memo text which will then also be used
         // in this split. See createTransaction() for this logic.
-        if (s.accountId() != m_account.id() && t.splitCount() == 2)
+        if ((s.accountId() != m_account.id() && t.splitCount() == 2) || !KMyMoneyGlobalSettings::autoFillUseMemos())
           s.setMemo(QString());
 
         m_transaction.addSplit(s);
@@ -1407,7 +1498,14 @@ void StdTransactionEditor::slotUpdateAction(int action)
         cashflow->setDirection(KMyMoneyRegister::Deposit);
         break;
       case KMyMoneyRegister::ActionTransfer:
-        categoryLabel->setText(i18n("Transfer from"));
+        if (m_split.shares().isNegative()) {
+          cashflow->setDirection(KMyMoneyRegister::Payment);
+          categoryLabel->setText(i18n("Transfer to"));
+        } else {
+          cashflow->setDirection(KMyMoneyRegister::Deposit);
+          categoryLabel->setText(i18n("Transfer from"));
+        }
+        tabbar->setCurrentIndex(KMyMoneyRegister::ActionTransfer);
         slotUpdateCashFlow(cashflow->direction());
         break;
       case KMyMoneyRegister::ActionWithdrawal:
@@ -1422,10 +1520,12 @@ void StdTransactionEditor::slotUpdateAction(int action)
 void StdTransactionEditor::slotUpdateCashFlow(KMyMoneyRegister::CashFlowDirection dir)
 {
   QLabel* categoryLabel = dynamic_cast<QLabel*>(haveWidget("category-label"));
-
+  KMyMoneyCashFlowCombo* cashflow = dynamic_cast<KMyMoneyCashFlowCombo*>(haveWidget("cashflow"));
+  cashflow->setDirection(dir);
   // qDebug("Update cashflow to %d", dir);
   if (categoryLabel) {
     TabBar* tabbar = dynamic_cast<TabBar*>(haveWidget("tabbar"));
+    if (!tabbar) return;  //  no transaction form
     if (categoryLabel->text() != i18n("Category")) {
       tabbar->setCurrentIndex(KMyMoneyRegister::ActionTransfer);
       if (dir == KMyMoneyRegister::Deposit) {
@@ -1474,24 +1574,25 @@ void StdTransactionEditor::slotUpdateCategory(const QString& id)
           tabbar->setTabEnabled(KMyMoneyRegister::ActionDeposit, false);
           tabbar->setTabEnabled(KMyMoneyRegister::ActionWithdrawal, false);
         }
+        KMyMoneyCashFlowCombo* cashflow = dynamic_cast<KMyMoneyCashFlowCombo*>(m_editWidgets["cashflow"]);
         if (val.isZero()) {
-          KMyMoneyCashFlowCombo* cashflow = dynamic_cast<KMyMoneyCashFlowCombo*>(m_editWidgets["cashflow"]);
           if (cashflow && (cashflow->direction() == KMyMoneyRegister::Deposit)) {
             categoryLabel->setText(i18n("Transfer from"));
           } else {
             categoryLabel->setText(i18n("Transfer to"));
           }
-        } else if (val.isNegative())
+        } else if (val.isNegative()) {
           categoryLabel->setText(i18n("Transfer from"));
-        else
+          cashflow->setDirection(KMyMoneyRegister::Deposit);
+        } else
           categoryLabel->setText(i18n("Transfer to"));
       } else {
         if (tabbar)
           tabbar->setTabEnabled(KMyMoneyRegister::ActionTransfer, false);
         categoryLabel->setText(i18n("Category"));
       }
-      updateAmount(val.abs());
-    } else {
+      updateAmount(val);
+    } else {  //id.isEmpty()
       KMyMoneyCategory* category = dynamic_cast<KMyMoneyCategory*>(m_editWidgets["category"]);
       if (!category->currentText().isEmpty() && tabbar)
         tabbar->setTabEnabled(KMyMoneyRegister::ActionTransfer, false);
@@ -1544,23 +1645,25 @@ void StdTransactionEditor::updateAmount(const MyMoneyMoney& val)
   if (categoryLabel) {
     KMyMoneyCashFlowCombo* cashflow = dynamic_cast<KMyMoneyCashFlowCombo*>(m_editWidgets["cashflow"]);
 
-    if (val.isNegative()) {
+    if (!val.isPositive())  {  //   fixes BUG321317
       if (categoryLabel->text() != i18n("Category")) {
-        if (categoryLabel->text() == i18n("Transfer from")) {
+        if (cashflow->direction() == KMyMoneyRegister::Payment) {
           categoryLabel->setText(i18n("Transfer to"));
-          cashflow->setDirection(KMyMoneyRegister::Payment);
-        } else {
-          categoryLabel->setText(i18n("Transfer from"));
-          cashflow->setDirection(KMyMoneyRegister::Deposit);
         }
       } else {
-        if (cashflow->direction() == KMyMoneyRegister::Deposit)
-          cashflow->setDirection(KMyMoneyRegister::Payment);
-        else
-          cashflow->setDirection(KMyMoneyRegister::Deposit);
         slotUpdateCashFlow(cashflow->direction());
       }
       dynamic_cast<kMyMoneyEdit*>(m_editWidgets["amount"])->setValue(val.abs());
+    } else {
+      if (categoryLabel->text() != i18n("Category")) {
+        if (cashflow->direction() == KMyMoneyRegister::Payment) {
+          categoryLabel->setText(i18n("Transfer to"));
+        } else {
+          categoryLabel->setText(i18n("Transfer from"));
+          cashflow->setDirection(KMyMoneyRegister::Deposit);  //  editing with +ve shows 'from' not 'pay to'
+        }
+      }
+      dynamic_cast<kMyMoneyEdit*>(m_editWidgets["amount"])->setValue(val.abs());
     }
   }
 }
@@ -1711,40 +1814,68 @@ bool StdTransactionEditor::isComplete(QString& reason) const
     postDate->markAsBadDate();
     postDate->setToolTip("");
   }
+  bool payeeIsPresent = false;
+  bool categoryIsPresent = false;
+  bool amountIsPresent = false;
+  bool memoIsPresent = false;
+  bool dateIsPresent = false;
 
   for (it_w = m_editWidgets.begin(); it_w != m_editWidgets.end(); ++it_w) {
     KMyMoneyPayeeCombo* payee = dynamic_cast<KMyMoneyPayeeCombo*>(*it_w);
+    KTagContainer* tagContainer = dynamic_cast<KTagContainer*>(*it_w);
     KMyMoneyCategory* category = dynamic_cast<KMyMoneyCategory*>(*it_w);
     kMyMoneyEdit* amount = dynamic_cast<kMyMoneyEdit*>(*it_w);
     KMyMoneyReconcileCombo* reconcile = dynamic_cast<KMyMoneyReconcileCombo*>(*it_w);
     KMyMoneyCashFlowCombo* cashflow = dynamic_cast<KMyMoneyCashFlowCombo*>(*it_w);
     KTextEdit* memo = dynamic_cast<KTextEdit*>(*it_w);
 
-    if (payee && !(payee->currentText().isEmpty()))
-      break;
+    if (payee && !(payee->currentText().isEmpty())) {
+      payeeIsPresent = true;
+    }
 
-    if (category && !category->lineEdit()->text().isEmpty())
-      break;
+//    if (tagContainer && !(tagContainer->selectedTags().isEmpty()))  //  Tag is not a mandatory field
+//      break;
 
-    if (amount && !(amount->value().isZero()))
-      break;
+    //  Mandatory fields
+
+    if (category && !category->lineEdit()->text().isEmpty()) {
+      categoryIsPresent = true;
+    }
+
+    if (amount && !(amount->value().isZero())) {
+      amountIsPresent = true;
+    }
 
     // the following widgets are only checked if we are editing multiple transactions
     if (isMultiSelection()) {
+      TabBar* tabbar = dynamic_cast<TabBar*>(haveWidget("tabbar"));
+      if (tabbar) {
+        tabbar->setEnabled(true);
+      }
       if (reconcile && reconcile->state() != MyMoneySplit::Unknown)
         break;
 
       if (cashflow && cashflow->direction() != KMyMoneyRegister::Unknown)
         break;
 
-      if (postDate->date().isValid() && (postDate->date() >= m_account.openingDate()))
+      if (postDate->date().isValid() && (postDate->date() >= m_account.openingDate())) {
+        dateIsPresent = true;
         break;
+      }
 
-      if (memo && !memo->toPlainText().isEmpty())
+      if (memo && m_memoChanged) {
+        memoIsPresent = true;
+        break;
+      }
+
+      if (tagContainer && !(tagContainer->selectedTags().isEmpty()))  //  Tag is optional field
         break;
     }
   }
-  return it_w != m_editWidgets.end();
+  bool rc = (categoryIsPresent && amountIsPresent);  //   usual mandatory fields
+  bool rc1 = (payeeIsPresent || categoryIsPresent || amountIsPresent || memoIsPresent) || dateIsPresent;  //  if isMultiSelection(), these are optional
+
+  return (rc || (isMultiSelection() && (it_w != m_editWidgets.end() || rc1)));
 }
 
 void StdTransactionEditor::slotCreateCategory(const QString& name, QString& id)
@@ -1855,8 +1986,8 @@ int StdTransactionEditor::slotEditSplits(void)
       delete dlg;
     }
 
-    // focus jumps into the memo field
-    if ((w = haveWidget("memo")) != 0) {
+    // focus jumps into the tag field
+    if ((w = haveWidget("tag")) != 0) {
       w->setFocus();
     }
 
@@ -1961,7 +2092,7 @@ bool StdTransactionEditor::createTransaction(MyMoneyTransaction& t, const MyMone
   //       by the user
   KTextEdit* memo = dynamic_cast<KTextEdit*>(m_editWidgets["memo"]);
   if (memo) {
-    if (!isMultiSelection() || (isMultiSelection() && !memo->toPlainText().isEmpty()))
+    if (!isMultiSelection() || (isMultiSelection() && m_memoChanged))
       s0.setMemo(memo->toPlainText());
   }
 
@@ -1978,6 +2109,12 @@ bool StdTransactionEditor::createTransaction(MyMoneyTransaction& t, const MyMone
     s0.setPayeeId(payeeId);
   }
 
+  //KMyMoneyTagCombo* tag = dynamic_cast<KMyMoneyTagCombo*>(m_editWidgets["tag"]);
+  KTagContainer* tag = dynamic_cast<KTagContainer*>(m_editWidgets["tag"]);
+  if (!isMultiSelection() || (isMultiSelection() && !tag->selectedTags().isEmpty())) {
+    s0.setTagIdList(tag->selectedTags());
+  }
+
   bool updateValue;
   MyMoneyMoney value = amountFromWidget(&updateValue);
 
@@ -2116,6 +2253,3 @@ void StdTransactionEditor::slotUpdateAccount(const QString& id)
     category->splitButton()->setDisabled(id.isEmpty());
   }
 }
-
-#include "transactioneditor.moc"
-
