@@ -41,6 +41,7 @@
 class AccountsModel::Private
 {
 public:
+
   /**
     * The pimpl.
     */
@@ -64,6 +65,17 @@ public:
     for (QStringList::ConstIterator it_l = list.constBegin(); it_l != list.constEnd(); ++it_l) {
       const MyMoneyAccount& acc = m_file->account(*it_l);
 
+      MyMoneyAccount::accountTypeE acctype = acc.accountType();
+      if ((acctype == MyMoneyAccount::Expense) || (acctype == MyMoneyAccount::Income)) {
+        if (m_addedCategoriesList.contains(acc.id())) {
+          //  Has been added already
+          return;
+        } else {
+          //  Add it to list
+          m_addedCategoriesList << acc.id();
+        }
+      }
+
       QStandardItem *item = new QStandardItem(acc.name());
       accountsItem->appendRow(item);
       item->setColumnCount(model->columnCount());
@@ -236,9 +248,8 @@ public:
         prices += m_file->price(account.currencyId(), m_file->baseCurrency().id());
       }
 
-    } catch (MyMoneyException *e) {
-      kDebug(2) << Q_FUNC_INFO << " caught exception while adding " << account.name() << "[" << account.id() << "]: " << e->what();
-      delete e;
+    } catch (const MyMoneyException &e) {
+      kDebug(2) << Q_FUNC_INFO << " caught exception while adding " << account.name() << "[" << account.id() << "]: " << e.what();
     }
 
     MyMoneyMoney value = balance;
@@ -318,7 +329,7 @@ public:
     QModelIndexList list = model->match(model->index(0, 0), AccountsModel::AccountIdRole, QVariant(accountId), -1, Qt::MatchFlags(Qt::MatchExactly | Qt::MatchCaseSensitive | Qt::MatchRecursive));
     foreach (const QModelIndex &index, list) {
       // always return the account which is not the child of the favorite accounts item
-      if (model->data(index.parent(), AccountsModel::AccountIdRole).toString() != AccountsModel::favoritesAccountId) {
+      if (index.parent().data(AccountsModel::AccountIdRole).toString() != AccountsModel::favoritesAccountId) {
         return model->itemFromIndex(index);
       }
     }
@@ -341,6 +352,12 @@ public:
     * Used to set the reconciliation flag.
     */
   MyMoneyAccount m_reconciledAccount;
+  /**
+    * Used to maintain a list of categories added
+    * during recursion through a category tree,
+    * before its completion.
+    */
+  QStringList m_addedCategoriesList;
 };
 
 const QString AccountsModel::favoritesAccountId("Favorites");
@@ -402,8 +419,8 @@ void AccountsModel::init()
   */
 void AccountsModel::load()
 {
-  //::timetrace("Start AccountsModel::load");
   QStandardItem *rootItem = invisibleRootItem();
+  d->m_addedCategoriesList.clear();
 
   QFont font;
   font.setBold(true);
@@ -414,7 +431,7 @@ void AccountsModel::load()
   setData(favoriteAccountsItem->index(), favoritesAccountId, AccountIdRole);
   setData(favoriteAccountsItem->index(), 0, DisplayOrderRole);
   favoriteAccountsItem->setColumnCount(columnCount());
-  favoriteAccountsItem->setIcon(QIcon(DesktopIcon("account"))); //krazy:exclude=iconnames
+  favoriteAccountsItem->setIcon(QIcon(DesktopIcon("view-bank-account")));
   favoriteAccountsItem->setEditable(false);
   favoriteAccountsItem->setFont(font);
 
@@ -493,7 +510,6 @@ void AccountsModel::load()
 
   checkNetWorth();
   checkProfit();
-  //::timetrace("Done AccountsModel::load");
 }
 
 /**
@@ -1247,5 +1263,3 @@ bool AccountsFilterProxyModel::hideUnusedIncomeExpenseAccounts(void) const
 {
   return d->m_hideUnusedIncomeExpenseAccounts;
 }
-
-#include "accountsmodel.moc"
