@@ -39,12 +39,14 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 #include "storage/mymoneyseqaccessmgr.h"
+#include "mymoneyaccount.h"
 #include "mymoneyreport.h"
 #include "mymoneybalancecache.h"
 #include "mymoneybudget.h"
 #include "mymoneyprice.h"
 #include "mymoneyobjectcontainer.h"
 #include "mymoneypayee.h"
+//#include "mymoneytag.h"
 
 // include the following line to get a 'cout' for debug purposes
 // #include <iostream>
@@ -78,6 +80,12 @@ public:
       m_id(payee.id()) {
   }
 
+  MyMoneyNotification(MyMoneyFile::notificationModeT mode, const MyMoneyTag& tag) :
+      m_objType(MyMoneyFile::notifyTag),
+      m_notificationMode(mode),
+      m_id(tag.id()) {
+  }
+
   MyMoneyNotification(MyMoneyFile::notificationModeT mode, const MyMoneySchedule& schedule) :
       m_objType(MyMoneyFile::notifySchedule),
       m_notificationMode(mode),
@@ -189,7 +197,7 @@ public:
     */
   inline void checkStorage(void) const {
     if (m_storage == 0)
-      throw new MYMONEYEXCEPTION("No storage object attached to MyMoneyFile");
+      throw MYMONEYEXCEPTION("No storage object attached to MyMoneyFile");
   }
 
   /**
@@ -200,7 +208,7 @@ public:
   void checkTransaction(const char* txt) const {
     checkStorage();
     if (!m_inTransaction) {
-      throw new MYMONEYEXCEPTION(QString("No transaction started for %1").arg(txt));
+      throw MYMONEYEXCEPTION(QString("No transaction started for %1").arg(txt));
     }
   }
 
@@ -310,10 +318,10 @@ MyMoneyFile::MyMoneyFile(IMyMoneyStorage *storage) :
 void MyMoneyFile::attachStorage(IMyMoneyStorage* const storage)
 {
   if (d->m_storage != 0)
-    throw new MYMONEYEXCEPTION("Storage already attached");
+    throw MYMONEYEXCEPTION("Storage already attached");
 
   if (storage == 0)
-    throw new MYMONEYEXCEPTION("Storage must not be 0");
+    throw MYMONEYEXCEPTION("Storage must not be 0");
 
   d->m_storage = storage;
 
@@ -352,7 +360,7 @@ void MyMoneyFile::startTransaction(void)
 {
   d->checkStorage();
   if (d->m_inTransaction) {
-    throw new MYMONEYEXCEPTION("Already started a transaction!");
+    throw MYMONEYEXCEPTION("Already started a transaction!");
   }
 
   d->m_storage->startTransaction();
@@ -445,7 +453,7 @@ void MyMoneyFile::addInstitution(MyMoneyInstitution& institution)
 
   if (institution.name().length() == 0
       || institution.id().length() != 0)
-    throw new MYMONEYEXCEPTION("Not a new institution");
+    throw MYMONEYEXCEPTION("Not a new institution");
 
   d->checkTransaction(Q_FUNC_INFO);
 
@@ -489,9 +497,9 @@ void MyMoneyFile::modifyTransaction(const MyMoneyTransaction& transaction)
     // account does not exist
     MyMoneyAccount acc = MyMoneyFile::account((*it_s).accountId());
     if (acc.id().isEmpty())
-      throw new MYMONEYEXCEPTION("Cannot store split with no account assigned");
+      throw MYMONEYEXCEPTION("Cannot store split with no account assigned");
     if (isStandardAccount((*it_s).accountId()))
-      throw new MYMONEYEXCEPTION("Cannot store split referencing standard account");
+      throw MYMONEYEXCEPTION("Cannot store split referencing standard account");
     if (acc.isLoan() && ((*it_s).action() == MyMoneySplit::ActionTransfer))
       loanAccountAffected = true;
   }
@@ -526,6 +534,7 @@ void MyMoneyFile::modifyTransaction(const MyMoneyTransaction& transaction)
   for (it_s = tr.splits().constBegin(); it_s != tr.splits().constEnd(); ++it_s) {
     d->addCacheNotification((*it_s).accountId(), tr.postDate());
     d->addCacheNotification((*it_s).payeeId());
+    //FIXME-ALEX Do I need to add d->addCacheNotification((*it_s).tagList()); ??
   }
 
   // perform modification
@@ -535,6 +544,7 @@ void MyMoneyFile::modifyTransaction(const MyMoneyTransaction& transaction)
   for (it_s = t->splits().constBegin(); it_s != t->splits().constEnd(); ++it_s) {
     d->addCacheNotification((*it_s).accountId(), t->postDate());
     d->addCacheNotification((*it_s).payeeId());
+    //FIXME-ALEX Do I need to add d->addCacheNotification((*it_s).tagList()); ??
   }
 }
 
@@ -557,12 +567,12 @@ void MyMoneyFile::modifyAccount(const MyMoneyAccount& _account)
 
     // now check that it is the same
     if (!(account == _account))
-      throw new MYMONEYEXCEPTION("Unable to modify the standard account groups");
+      throw MYMONEYEXCEPTION("Unable to modify the standard account groups");
   }
 
   if (account.accountType() != acc.accountType() &&
       !account.isLiquidAsset() && !acc.isLiquidAsset())
-    throw new MYMONEYEXCEPTION("Unable to change account type");
+    throw MYMONEYEXCEPTION("Unable to change account type");
 
   // clear all changed objects from cache
   MyMoneyNotifier notifier(d);
@@ -599,17 +609,17 @@ void MyMoneyFile::reparentAccount(MyMoneyAccount &acc, MyMoneyAccount& parent)
 
   // check that it's not one of the standard account groups
   if (isStandardAccount(acc.id()))
-    throw new MYMONEYEXCEPTION("Unable to reparent the standard account groups");
+    throw MYMONEYEXCEPTION("Unable to reparent the standard account groups");
 
   if (acc.accountGroup() == parent.accountGroup()
       || (acc.accountType() == MyMoneyAccount::Income && parent.accountType() == MyMoneyAccount::Expense)
       || (acc.accountType() == MyMoneyAccount::Expense && parent.accountType() == MyMoneyAccount::Income)) {
 
     if (acc.isInvest() && parent.accountType() != MyMoneyAccount::Investment)
-      throw new MYMONEYEXCEPTION("Unable to reparent Stock to non-investment account");
+      throw MYMONEYEXCEPTION("Unable to reparent Stock to non-investment account");
 
     if (parent.accountType() == MyMoneyAccount::Investment && !acc.isInvest())
-      throw new MYMONEYEXCEPTION("Unable to reparent non-stock to investment account");
+      throw MYMONEYEXCEPTION("Unable to reparent non-stock to investment account");
 
     // clear all changed objects from cache
     MyMoneyNotifier notifier(d);
@@ -630,7 +640,7 @@ void MyMoneyFile::reparentAccount(MyMoneyAccount &acc, MyMoneyAccount& parent)
     d->m_changeSet += MyMoneyNotification(notifyModify, acc);
 
   } else
-    throw new MYMONEYEXCEPTION("Unable to reparent to different account type");
+    throw MYMONEYEXCEPTION("Unable to reparent to different account type");
 }
 
 const MyMoneyInstitution& MyMoneyFile::institution(const QString& id) const
@@ -676,9 +686,10 @@ void MyMoneyFile::removeTransaction(const MyMoneyTransaction& transaction)
   for (it_s = tr.splits().constBegin(); it_s != tr.splits().constEnd(); ++it_s) {
     MyMoneyAccount acc = account((*it_s).accountId());
     if (acc.isClosed())
-      throw new MYMONEYEXCEPTION(i18n("Cannot remove transaction that references a closed account."));
+      throw MYMONEYEXCEPTION(i18n("Cannot remove transaction that references a closed account."));
     d->addCacheNotification((*it_s).accountId(), tr.postDate());
     d->addCacheNotification((*it_s).payeeId());
+    //FIXME-ALEX Do I need to add d->addCacheNotification((*it_s).tagList()); ??
   }
 
   d->m_storage->removeTransaction(transaction);
@@ -729,10 +740,10 @@ void MyMoneyFile::removeAccount(const MyMoneyAccount& account)
 
   // check that it's not one of the standard account groups
   if (isStandardAccount(account.id()))
-    throw new MYMONEYEXCEPTION("Unable to remove the standard account groups");
+    throw MYMONEYEXCEPTION("Unable to remove the standard account groups");
 
   if (hasActiveSplits(account.id())) {
-    throw new MYMONEYEXCEPTION("Unable to remove account with active splits");
+    throw MYMONEYEXCEPTION("Unable to remove account with active splits");
   }
 
   // clear all changed objects from cache
@@ -768,14 +779,14 @@ void MyMoneyFile::removeAccount(const MyMoneyAccount& account)
 void MyMoneyFile::removeAccountList(const QStringList& account_list, unsigned int level)
 {
   if (level > 100)
-    throw new MYMONEYEXCEPTION("Too deep recursion in [MyMoneyFile::removeAccountList]!");
+    throw MYMONEYEXCEPTION("Too deep recursion in [MyMoneyFile::removeAccountList]!");
 
   d->checkTransaction(Q_FUNC_INFO);
 
   // upon entry, we check that we could proceed with the operation
   if (!level) {
     if (!hasOnlyUnusedAccounts(account_list, 0)) {
-      throw new MYMONEYEXCEPTION("One or more accounts cannot be removed");
+      throw MYMONEYEXCEPTION("One or more accounts cannot be removed");
     }
   }
 
@@ -803,7 +814,7 @@ void MyMoneyFile::removeAccountList(const QStringList& account_list, unsigned in
 bool MyMoneyFile::hasOnlyUnusedAccounts(const QStringList& account_list, unsigned int level)
 {
   if (level > 100)
-    throw new MYMONEYEXCEPTION("Too deep recursion in [MyMoneyFile::hasOnlyUnusedAccounts]!");
+    throw MYMONEYEXCEPTION("Too deep recursion in [MyMoneyFile::hasOnlyUnusedAccounts]!");
   // process all accounts in the list and test if they have transactions assigned
   for (QStringList::ConstIterator it = account_list.constBegin(); it != account_list.constEnd(); ++it) {
     if (transactionCount(*it) != 0)
@@ -854,19 +865,19 @@ void MyMoneyFile::addAccount(MyMoneyAccount& account, MyMoneyAccount& parent)
   // it's own ID is not set and it does not have a pointer to (MyMoneyFile)
 
   if (account.name().length() == 0)
-    throw new MYMONEYEXCEPTION("Account has no name");
+    throw MYMONEYEXCEPTION("Account has no name");
 
   if (account.id().length() != 0)
-    throw new MYMONEYEXCEPTION("New account must have no id");
+    throw MYMONEYEXCEPTION("New account must have no id");
 
   if (account.accountList().count() != 0)
-    throw new MYMONEYEXCEPTION("New account must have no sub-accounts");
+    throw MYMONEYEXCEPTION("New account must have no sub-accounts");
 
   if (!account.parentAccountId().isEmpty())
-    throw new MYMONEYEXCEPTION("New account must have no parent-id");
+    throw MYMONEYEXCEPTION("New account must have no parent-id");
 
   if (account.accountType() == MyMoneyAccount::UnknownAccountType)
-    throw new MYMONEYEXCEPTION("Account has invalid type");
+    throw MYMONEYEXCEPTION("Account has invalid type");
 
   // make sure, that the parent account exists
   // if not, an exception is thrown. If it exists,
@@ -899,13 +910,13 @@ void MyMoneyFile::addAccount(MyMoneyAccount& account, MyMoneyAccount& parent)
   // that the parent for a stock account must be an investment. Also,
   // an investment cannot have another investment account as it's parent
   if (parent.isInvest())
-    throw new MYMONEYEXCEPTION("Stock account cannot be parent account");
+    throw MYMONEYEXCEPTION("Stock account cannot be parent account");
 
   if (account.isInvest() && parent.accountType() != MyMoneyAccount::Investment)
-    throw new MYMONEYEXCEPTION("Stock account must have investment account as parent ");
+    throw MYMONEYEXCEPTION("Stock account must have investment account as parent ");
 
   if (!account.isInvest() && parent.accountType() == MyMoneyAccount::Investment)
-    throw new MYMONEYEXCEPTION("Investment account can only have stock accounts as children");
+    throw MYMONEYEXCEPTION("Investment account can only have stock accounts as children");
 
   // clear all changed objects from cache
   MyMoneyNotifier notifier(d);
@@ -996,8 +1007,7 @@ QString MyMoneyFile::openingBalanceTransaction(const MyMoneyAccount& acc) const
 
   try {
     openAcc = openingBalanceAccount(currency);
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     return result;
   }
 
@@ -1015,10 +1025,9 @@ QString MyMoneyFile::openingBalanceTransaction(const MyMoneyAccount& acc) const
       // If so, we have a winner!
       result = (*it_t).id();
       break;
-    } catch (MyMoneyException *e) {
+    } catch (const MyMoneyException &) {
       // If not, keep searching
       ++it_t;
-      delete e;
     }
   }
 
@@ -1028,12 +1037,11 @@ QString MyMoneyFile::openingBalanceTransaction(const MyMoneyAccount& acc) const
 const MyMoneyAccount MyMoneyFile::openingBalanceAccount(const MyMoneySecurity& security)
 {
   if (!security.isCurrency())
-    throw new MYMONEYEXCEPTION("Opening balance for non currencies not supported");
+    throw MYMONEYEXCEPTION("Opening balance for non currencies not supported");
 
   try {
     return openingBalanceAccount_internal(security);
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     MyMoneyFileTransaction ft;
     MyMoneyAccount acc;
 
@@ -1041,9 +1049,8 @@ const MyMoneyAccount MyMoneyFile::openingBalanceAccount(const MyMoneySecurity& s
       acc = createOpeningBalanceAccount(security);
       ft.commit();
 
-    } catch (MyMoneyException* e) {
+    } catch (const MyMoneyException &) {
       qDebug("Unable to create opening balance account for security %s", qPrintable(security.id()));
-      delete e;
     }
     return acc;
   }
@@ -1057,7 +1064,7 @@ const MyMoneyAccount MyMoneyFile::openingBalanceAccount(const MyMoneySecurity& s
 const MyMoneyAccount MyMoneyFile::openingBalanceAccount_internal(const MyMoneySecurity& security) const
 {
   if (!security.isCurrency())
-    throw new MYMONEYEXCEPTION("Opening balance for non currencies not supported");
+    throw MYMONEYEXCEPTION("Opening balance for non currencies not supported");
 
   MyMoneyAccount acc;
   QRegExp match(QString("^%1").arg((MyMoneyFile::OpeningBalancesPrefix)));
@@ -1077,7 +1084,7 @@ const MyMoneyAccount MyMoneyFile::openingBalanceAccount_internal(const MyMoneySe
   }
 
   if (acc.id().isEmpty()) {
-    throw new MYMONEYEXCEPTION(QString("No opening balance account for %1").arg(security.tradingSymbol()));
+    throw MYMONEYEXCEPTION(QString("No opening balance account for %1").arg(security.tradingSymbol()));
   }
 
   return acc;
@@ -1116,9 +1123,9 @@ void MyMoneyFile::addTransaction(MyMoneyTransaction& transaction)
 
   // first perform all the checks
   if (!transaction.id().isEmpty())
-    throw new MYMONEYEXCEPTION("Unable to add transaction with id set");
+    throw MYMONEYEXCEPTION("Unable to add transaction with id set");
   if (!transaction.postDate().isValid())
-    throw new MYMONEYEXCEPTION("Unable to add transaction with invalid postdate");
+    throw MYMONEYEXCEPTION("Unable to add transaction with invalid postdate");
 
   // now check the splits
   bool loanAccountAffected = false;
@@ -1128,11 +1135,11 @@ void MyMoneyFile::addTransaction(MyMoneyTransaction& transaction)
     // account does not exist or is one of the standard accounts
     MyMoneyAccount acc = MyMoneyFile::account((*it_s).accountId());
     if (acc.id().isEmpty())
-      throw new MYMONEYEXCEPTION("Cannot add split with no account assigned");
+      throw MYMONEYEXCEPTION("Cannot add split with no account assigned");
     if (acc.isLoan())
       loanAccountAffected = true;
     if (isStandardAccount((*it_s).accountId()))
-      throw new MYMONEYEXCEPTION("Cannot add split referencing standard account");
+      throw MYMONEYEXCEPTION("Cannot add split referencing standard account");
   }
 
   // change transfer splits between asset/liability and loan accounts
@@ -1164,6 +1171,7 @@ void MyMoneyFile::addTransaction(MyMoneyTransaction& transaction)
   for (it_s = transaction.splits().constBegin(); it_s != transaction.splits().constEnd(); ++it_s) {
     d->addCacheNotification((*it_s).accountId(), transaction.postDate());
     d->addCacheNotification((*it_s).payeeId());
+    //FIXME-ALEX Do I need to add d->addCacheNotification((*it_s).tagList()); ??
   }
 }
 
@@ -1239,6 +1247,64 @@ void MyMoneyFile::removePayee(const MyMoneyPayee& payee)
   d->m_changeSet += MyMoneyNotification(notifyRemove, payee);
 }
 
+void MyMoneyFile::addTag(MyMoneyTag& tag)
+{
+  d->checkTransaction(Q_FUNC_INFO);
+
+  // clear all changed objects from cache
+  MyMoneyNotifier notifier(d);
+
+  d->m_storage->addTag(tag);
+
+  // The notifier mechanism only refreshes the cache but does not
+  // load new objects. So we simply force loading of the new one here
+  d->m_cache.preloadTag(tag);
+
+  d->m_changeSet += MyMoneyNotification(notifyAdd, tag);
+}
+
+const MyMoneyTag& MyMoneyFile::tag(const QString& id) const
+{
+  return d->m_cache.tag(id);
+}
+
+const MyMoneyTag& MyMoneyFile::tagByName(const QString& name) const
+{
+  d->checkStorage();
+
+  return d->m_cache.tag(d->m_storage->tagByName(name).id());
+}
+
+void MyMoneyFile::modifyTag(const MyMoneyTag& tag)
+{
+  d->checkTransaction(Q_FUNC_INFO);
+
+  // clear all changed objects from cache
+  MyMoneyNotifier notifier(d);
+
+  d->addCacheNotification(tag.id());
+
+  d->m_storage->modifyTag(tag);
+
+  d->m_changeSet += MyMoneyNotification(notifyModify, tag);
+}
+
+void MyMoneyFile::removeTag(const MyMoneyTag& tag)
+{
+  d->checkTransaction(Q_FUNC_INFO);
+
+  // FIXME we need to make sure, that the tag is not referenced anymore
+
+  // clear all changed objects from cache
+  MyMoneyNotifier notifier(d);
+
+  d->m_storage->removeTag(tag);
+
+  d->addCacheNotification(tag.id(), false);
+
+  d->m_changeSet += MyMoneyNotification(notifyRemove, tag);
+}
+
 void MyMoneyFile::accountList(QList<MyMoneyAccount>& list, const QStringList& idlist, const bool recursive) const
 {
   if (idlist.isEmpty()) {
@@ -1414,6 +1480,37 @@ const MyMoneyMoney MyMoneyFile::balance(const QString& id, const QDate& date) co
   return returnValue;
 }
 
+const MyMoneyMoney MyMoneyFile::clearedBalance(const QString &id, const QDate& date) const
+{
+  MyMoneyMoney cleared;
+  QList<MyMoneyTransaction> list;
+
+  cleared = balance(id, date);
+
+  MyMoneyAccount account = this->account(id);
+  MyMoneyMoney factor(1, 1);
+  if (account.accountGroup() == MyMoneyAccount::Liability || account.accountGroup() == MyMoneyAccount::Equity)
+    factor = -factor;
+
+  MyMoneyTransactionFilter filter;
+  filter.addAccount(id);
+  filter.setDateFilter(QDate(), date);
+  filter.setReportAllSplits(false);
+  filter.addState(MyMoneyTransactionFilter::notReconciled);
+  transactionList(list, filter);
+
+  for (QList<MyMoneyTransaction>::const_iterator it_t = list.constBegin(); it_t != list.constEnd(); ++it_t) {
+    const QList<MyMoneySplit>& splits = (*it_t).splits();
+    for (QList<MyMoneySplit>::const_iterator it_s = splits.constBegin(); it_s != splits.constEnd(); ++it_s) {
+      const MyMoneySplit &split = (*it_s);
+      if (split.accountId() != id)
+        continue;
+      cleared -= split.shares();
+    }
+  }
+  return cleared * factor;
+}
+
 const MyMoneyMoney MyMoneyFile::totalBalance(const QString& id, const QDate& date) const
 {
   d->checkStorage();
@@ -1429,9 +1526,8 @@ void MyMoneyFile::warningMissingRate(const QString& fromId, const QString& toId)
     to = security(toId);
     qWarning("Missing price info for conversion from %s to %s", qPrintable(from.name()), qPrintable(to.name()));
 
-  } catch (MyMoneyException *e) {
-    qWarning("Missing security caught in MyMoneyFile::warningMissingRate(): %s(%ld) %s", qPrintable(e->file()), e->line(), qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    qWarning("Missing security caught in MyMoneyFile::warningMissingRate(): %s(%ld) %s", qPrintable(e.file()), e.line(), qPrintable(e.what()));
   }
 }
 
@@ -1461,6 +1557,13 @@ const QList<MyMoneyPayee> MyMoneyFile::payeeList(void) const
   return list;
 }
 
+const QList<MyMoneyTag> MyMoneyFile::tagList(void) const
+{
+  QList<MyMoneyTag> list;
+  d->m_cache.tag(list);
+  return list;
+}
+
 QString MyMoneyFile::accountToCategory(const QString& accountId, bool includeStandardAccounts) const
 {
   MyMoneyAccount acc;
@@ -1575,9 +1678,9 @@ void MyMoneyFile::addSchedule(MyMoneySchedule& sched)
     // account does not exist or is one of the standard accounts
     MyMoneyAccount acc = MyMoneyFile::account((*it_s).accountId());
     if (acc.id().isEmpty())
-      throw new MYMONEYEXCEPTION("Cannot add split with no account assigned");
+      throw MYMONEYEXCEPTION("Cannot add split with no account assigned");
     if (isStandardAccount((*it_s).accountId()))
-      throw new MYMONEYEXCEPTION("Cannot add split referencing standard account");
+      throw MYMONEYEXCEPTION("Cannot add split referencing standard account");
   }
 
   // clear all changed objects from cache
@@ -1603,9 +1706,9 @@ void MyMoneyFile::modifySchedule(const MyMoneySchedule& sched)
     // account does not exist or is one of the standard accounts
     MyMoneyAccount acc = MyMoneyFile::account((*it_s).accountId());
     if (acc.id().isEmpty())
-      throw new MYMONEYEXCEPTION("Cannot store split with no account assigned");
+      throw MYMONEYEXCEPTION("Cannot store split with no account assigned");
     if (isStandardAccount((*it_s).accountId()))
-      throw new MYMONEYEXCEPTION("Cannot store split referencing standard account");
+      throw MYMONEYEXCEPTION("Cannot store split referencing standard account");
   }
 
   // clear all changed objects from cache
@@ -1747,9 +1850,8 @@ const QStringList MyMoneyFile::consistencyCheck(void)
         parentId = parent.parentAccountId();
       }
 
-    } catch (MyMoneyException *e) {
+    } catch (const MyMoneyException &) {
       // if we don't know about a parent, we catch it later
-      delete e;
     }
 
     // check that the parent exists
@@ -1786,8 +1888,7 @@ const QStringList MyMoneyFile::consistencyCheck(void)
         if (accountRebuild.contains(parent.id()) == 0)
           accountRebuild << parent.id();
       }
-    } catch (MyMoneyException *e) {
-      delete e;
+    } catch (const MyMoneyException &) {
       // apparently, the parent does not exist anymore. we reconnect to the
       // master group account (asset, liability, etc) to which this account
       // should belong and update it in the engine.
@@ -1813,9 +1914,9 @@ const QStringList MyMoneyFile::consistencyCheck(void)
       try {
         child = account(*it_c);
         if (child.parentAccountId() != (*it_a).id()) {
-          throw new MYMONEYEXCEPTION("Child account has a different parent");
+          throw MYMONEYEXCEPTION("Child account has a different parent");
         }
-      } catch (MyMoneyException *e) {
+      } catch (const MyMoneyException &) {
         problemCount++;
         if (problemAccount != (*it_a).name()) {
           problemAccount = (*it_a).name();
@@ -1836,7 +1937,7 @@ const QStringList MyMoneyFile::consistencyCheck(void)
       }
       try {
         payee(loan.payee());
-      } catch (MyMoneyException *e) {
+      } catch (const MyMoneyException &) {
         problemCount++;
         if (problemAccount != (*it_a).name()) {
           problemAccount = (*it_a).name();
@@ -1866,8 +1967,7 @@ const QStringList MyMoneyFile::consistencyCheck(void)
       try {
         d->m_storage->modifyAccount(*it_a, true);
         d->addCacheNotification((*it_a).id());
-      } catch (MyMoneyException *e) {
-        delete e;
+      } catch (const MyMoneyException &) {
         rc << i18n("  * Unable to update account data in engine.");
         return rc;
       }
@@ -1910,8 +2010,7 @@ const QStringList MyMoneyFile::consistencyCheck(void)
       try {
         d->m_storage->modifyAccount(*it_a, true);
         d->addCacheNotification((*it_a).id());
-      } catch (MyMoneyException *e) {
-        delete e;
+      } catch (const MyMoneyException &) {
         rc << i18n("  * Unable to update account data for account %1 in engine", (*it_a).name());
       }
     }
@@ -1981,11 +2080,10 @@ const QStringList MyMoneyFile::consistencyCheck(void)
           sChanged = true;
           ++problemCount;
         }
-      } catch (MyMoneyException *e) {
+      } catch (const MyMoneyException &) {
         rc << i18n("  * Split %2 in transaction '%1' contains a reference to invalid account %3. Please fix manually.", t.id(), (*it_s).id(), (*it_s).accountId());
         ++problemCount;
         ++unfixedCount;
-        delete e;
       }
 
       // make sure the interest splits are marked correct as such
@@ -2067,11 +2165,10 @@ const QStringList MyMoneyFile::consistencyCheck(void)
           sChanged = true;
           ++problemCount;
         }
-      } catch (MyMoneyException *e) {
+      } catch (const MyMoneyException &) {
         rc << i18n("  * Split %2 in schedule '%1' contains a reference to invalid account %3. Please fix manually.", (*it_sch).name(), (*it_s).id(), (*it_s).accountId());
         ++problemCount;
         ++unfixedCount;
-        delete e;
       }
       if (sChanged) {
         t.modifySplit(s);
@@ -2205,7 +2302,7 @@ const QStringList MyMoneyFile::consistencyCheck(void)
     for (QList<MyMoneyBudget::AccountGroup>::const_iterator it_bacc = baccounts.constBegin(); it_bacc != baccounts.constEnd(); ++it_bacc) {
       try {
         account((*it_bacc).id());
-      } catch (MyMoneyException *e) {
+      } catch (const MyMoneyException &) {
         problemCount++;
         if (problemBudget != b.name()) {
           problemBudget = b.name();
@@ -2245,7 +2342,7 @@ QString MyMoneyFile::createCategory(const MyMoneyAccount& base, const QString& n
   QString categoryText;
 
   if (base.id() != expense().id() && base.id() != income().id())
-    throw new MYMONEYEXCEPTION("Invalid base category");
+    throw MYMONEYEXCEPTION("Invalid base category");
 
   QStringList subAccounts = name.split(AccountSeperator);
   QStringList::Iterator it;
@@ -2268,13 +2365,12 @@ QString MyMoneyFile::createCategory(const MyMoneyAccount& base, const QString& n
       else {
         categoryAccount = account(categoryId);
       }
-    } catch (MyMoneyException *e) {
+    } catch (const MyMoneyException &e) {
       qDebug("Unable to add account %s, %s, %s: %s",
              qPrintable(categoryAccount.name()),
              qPrintable(parent.name()),
              qPrintable(categoryText),
-             qPrintable(e->what()));
-      delete e;
+             qPrintable(e.what()));
     }
 
     parent = categoryAccount;
@@ -2394,7 +2490,7 @@ void MyMoneyFile::removeCurrency(const MyMoneySecurity& currency)
   d->checkTransaction(Q_FUNC_INFO);
 
   if (currency.id() == d->m_baseCurrency.id()) {
-    throw new MYMONEYEXCEPTION("Cannot delete base currency.");
+    throw MYMONEYEXCEPTION("Cannot delete base currency.");
   }
 
   // FIXME check that security is not referenced by other object
@@ -2418,7 +2514,7 @@ const MyMoneySecurity& MyMoneyFile::currency(const QString& id) const
   if (curr.id().isEmpty()) {
     QString msg;
     msg = QString("Currency '%1' not found.").arg(id);
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
   return curr;
 }
@@ -2508,11 +2604,35 @@ const MyMoneyPrice MyMoneyFile::price(const QString& fromId, const QString& toId
     return MyMoneyPrice(fromId, toId, date, MyMoneyMoney(1, 1), "KMyMoney");
   }
 
-  // search 'from-to' rate
-  MyMoneyPrice rc = d->m_storage->price(fromId, to, date, exactDate);
+  // if not asking for exact date, try to find the exact date match first,
+  // either the requested price or its reciprocal value. If unsuccessful, it will move
+  // on and look for prices of previous dates
+  MyMoneyPrice rc = d->m_storage->price(fromId, to, date, true);
   if (!rc.isValid()) {
-    // not found, search 'to-fron' rate and use reciprocal value
-    rc = d->m_storage->price(to, fromId, date, exactDate);
+    // not found, search 'to-from' rate and use reciprocal value
+    rc = d->m_storage->price(to, fromId, date, true);
+
+    // not found, search previous dates, if exact date is not needed
+    if (!exactDate && !rc.isValid()) {
+      // search 'from-to' and 'to-from', select the most recent one
+      MyMoneyPrice fromPrice = d->m_storage->price(fromId, to, date, exactDate);
+      MyMoneyPrice toPrice = d->m_storage->price(to, fromId, date, exactDate);
+
+      // check first whether both prices are valid
+      if (fromPrice.isValid() && toPrice.isValid()) {
+        if (fromPrice.date() >= toPrice.date()) {
+          // if 'from-to' is newer or the same date, prefer that one
+          rc = fromPrice;
+        } else {
+          // otherwise, use the reciprocal price
+          rc = toPrice;
+        }
+      } else if (fromPrice.isValid()) { // check if any of the prices is valid, return that one
+        rc = fromPrice;
+      } else if (toPrice.isValid()) {
+        rc = toPrice;
+      }
+    }
   }
   return rc;
 }
@@ -2654,7 +2774,62 @@ void MyMoneyFile::removeBudget(const MyMoneyBudget& budget)
   d->addCacheNotification(budget.id(), false);
 }
 
+bool MyMoneyFile::addVATSplit(MyMoneyTransaction& transaction, const MyMoneyAccount& account, const MyMoneyAccount& category, const MyMoneyMoney& amount)
+{
+  bool rc = false;
 
+  try {
+    MyMoneySplit cat;  // category
+    MyMoneySplit tax;  // tax
+
+    if (category.value("VatAccount").isEmpty())
+      return false;
+    MyMoneyAccount vatAcc = this->account(category.value("VatAccount").toLatin1());
+    const MyMoneySecurity& asec = security(account.currencyId());
+    const MyMoneySecurity& csec = security(category.currencyId());
+    const MyMoneySecurity& vsec = security(vatAcc.currencyId());
+    if (asec.id() != csec.id() || asec.id() != vsec.id()) {
+      qDebug("Auto VAT assignment only works if all three accounts use the same currency.");
+      return false;
+    }
+
+    MyMoneyMoney vatRate(vatAcc.value("VatRate"));
+    MyMoneyMoney gv, nv;    // gross value, net value
+    int fract = account.fraction();
+
+    if (!vatRate.isZero()) {
+
+      tax.setAccountId(vatAcc.id());
+
+      // qDebug("vat amount is '%s'", category.value("VatAmount").toLatin1());
+      if (category.value("VatAmount").toLower() != QString("net")) {
+        // split value is the gross value
+        gv = amount;
+        nv = gv / (MyMoneyMoney(1, 1) + vatRate);
+        MyMoneySplit catSplit = transaction.splitByAccount(account.id(), false);
+        catSplit.setShares(-nv.convert(fract));
+        catSplit.setValue(catSplit.shares());
+        transaction.modifySplit(catSplit);
+
+      } else {
+        // split value is the net value
+        nv = amount;
+        gv = nv * (MyMoneyMoney(1, 1) + vatRate);
+        MyMoneySplit accSplit = transaction.splitByAccount(account.id());
+        accSplit.setValue(gv.convert(fract));
+        accSplit.setShares(accSplit.value());
+        transaction.modifySplit(accSplit);
+      }
+
+      tax.setValue(-(gv - nv).convert(fract));
+      tax.setShares(tax.value());
+      transaction.addSplit(tax);
+      rc = true;
+    }
+  } catch (const MyMoneyException &) {
+  }
+  return rc;
+}
 
 bool MyMoneyFile::isReferenced(const MyMoneyObject& obj, const MyMoneyFileBitArray& skipChecks) const
 {
@@ -2681,8 +2856,7 @@ bool MyMoneyFile::checkNoUsed(const QString& accId, const QString& no) const
       split = (*it_t).splitByAccount(accId, true /*match*/);
       if (!split.number().isEmpty() && split.number() == no)
         return true;
-    } catch (MyMoneyException *e) {
-      delete e;
+    } catch (const MyMoneyException &) {
     }
     ++it_t;
   }
@@ -2710,14 +2884,22 @@ QString MyMoneyFile::highestCheckNo(const QString& accId) const
           no = split.number();
         }
       }
-    } catch (MyMoneyException *e) {
-      delete e;
+    } catch (const MyMoneyException &) {
     }
     ++it_t;
   }
   return no;
 }
 
+bool MyMoneyFile::hasNewerTransaction(const QString& accId, const QDate& date) const
+{
+  MyMoneyTransactionFilter filter;
+  filter.addAccount(accId);
+  filter.setDateFilter(date.addDays(+1), QDate());
+  QList<MyMoneyTransaction> transactions = transactionList(filter);
+  return transactions.count() > 0;
+}
+
 void MyMoneyFile::clearCache(void)
 {
   d->checkStorage();
@@ -2734,6 +2916,7 @@ void MyMoneyFile::preloadCache(void)
   d->m_storage->accountList(a_list);
   d->m_cache.preloadAccount(a_list);
   d->m_cache.preloadPayee(d->m_storage->payeeList());
+  d->m_cache.preloadTag(d->m_storage->tagList());
   d->m_cache.preloadInstitution(d->m_storage->institutionList());
   d->m_cache.preloadSecurity(d->m_storage->securityList() +
                              d->m_storage->currencyList());
@@ -2774,8 +2957,7 @@ bool MyMoneyFile::referencesClosedAccount(const MyMoneySplit& s) const
 
   try {
     return account(s.accountId()).isClosed();
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
   }
   return false;
 }
@@ -2790,9 +2972,8 @@ QString MyMoneyFile::storageId(void)
       setValue("kmm-id", uid.toString());
       ft.commit();
       id = uid.toString();
-    } catch (MyMoneyException *e) {
+    } catch (const MyMoneyException &) {
       qDebug("Unable to setup UID for new storage object");
-      delete e;
     }
   }
   return id;
@@ -2804,7 +2985,7 @@ bool MyMoneyFile::hasMatchingOnlineBalance(const MyMoneyAccount& _acc) const
   MyMoneyAccount acc = account(_acc.id());
 
   // if there's no last transaction import data we are done
-  if( acc.value("lastImportedTransactionDate").isEmpty()
+  if (acc.value("lastImportedTransactionDate").isEmpty()
       || acc.value("lastStatementBalance").isEmpty())
     return false;
 
@@ -2815,6 +2996,14 @@ bool MyMoneyFile::hasMatchingOnlineBalance(const MyMoneyAccount& _acc) const
   return balance == accBalance;
 }
 
+int MyMoneyFile::countTransactionsWithSpecificReconciliationState(const QString& accId, enum MyMoneyTransactionFilter::stateOptionE state) const
+{
+  MyMoneyTransactionFilter filter;
+  filter.addAccount(accId);
+  filter.addState(state);
+  return transactionList(filter).count();
+}
+
 MyMoneyFileTransaction::MyMoneyFileTransaction() :
     m_isNested(MyMoneyFile::instance()->hasTransaction()),
     m_needRollback(!m_isNested)
@@ -2850,6 +3039,3 @@ void MyMoneyFileTransaction::rollback(void)
     MyMoneyFile::instance()->rollbackTransaction();
   m_needRollback = false;
 }
-
-
-#include "mymoneyfile.moc"
