@@ -15,8 +15,6 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <config-kmymoney.h>
-
 #include "mymoneyreport.h"
 
 // ----------------------------------------------------------------------------
@@ -33,21 +31,21 @@
 
 #include "mymoneyfile.h"
 
-const QStringList MyMoneyReport::kRowTypeText = QString("none,assetliability,expenseincome,category,topcategory,account,payee,month,week,topaccount,topaccount-account,equitytype,accounttype,institution,budget,budgetactual,schedule,accountinfo,accountloaninfo,accountreconcile,cashflow").split(',');
+const QStringList MyMoneyReport::kRowTypeText = QString("none,assetliability,expenseincome,category,topcategory,account,tag,payee,month,week,topaccount,topaccount-account,equitytype,accounttype,institution,budget,budgetactual,schedule,accountinfo,accountloaninfo,accountreconcile,cashflow").split(',');
 const QStringList MyMoneyReport::kColumnTypeText = QString("none,months,bimonths,quarters,4,5,6,weeks,8,9,10,11,years").split(',');
 
 // if you add names here, don't forget to update the bitmap for EQueryColumns
 // and shift the bit for eQCend one position to the left
-const QStringList MyMoneyReport::kQueryColumnsText = QString("none,number,payee,category,memo,account,reconcileflag,action,shares,price,performance,loan,balance").split(',');
+const QStringList MyMoneyReport::kQueryColumnsText = QString("none,number,payee,category,tag,memo,account,reconcileflag,action,shares,price,performance,loan,balance").split(',');
 
-const MyMoneyReport::EReportType MyMoneyReport::kTypeArray[] = { eNoReport, ePivotTable, ePivotTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, ePivotTable, ePivotTable, eInfoTable, eInfoTable, eInfoTable, eQueryTable, eQueryTable, eNoReport };
+const MyMoneyReport::EReportType MyMoneyReport::kTypeArray[] = { eNoReport, ePivotTable, ePivotTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, eQueryTable, ePivotTable, ePivotTable, eInfoTable, eInfoTable, eInfoTable, eQueryTable, eQueryTable, eNoReport };
 const QStringList MyMoneyReport::kDetailLevelText = QString("none,all,top,group,total,invalid").split(',');
 const QStringList MyMoneyReport::kChartTypeText = QString("none,line,bar,pie,ring,stackedbar,invalid").split(',');
 
 // This should live in mymoney/mymoneytransactionfilter.h
 const QStringList kTypeText = QString("all,payments,deposits,transfers,none").split(',');
 const QStringList kStateText = QString("all,notreconciled,cleared,reconciled,frozen,none").split(',');
-const QStringList kDateLockText = QString("alldates,untiltoday,currentmonth,currentyear,monthtodate,yeartodate,yeartomonth,lastmonth,lastyear,last7days,last30days,last3months,last6months,last12months,next7days,next30days,next3months,next6months,next12months,userdefined,last3tonext3months,last11Months,currentQuarter,lastQuarter,nextQuarter,currentFiscalYear,lastFiscalYear,today").split(',');
+const QStringList kDateLockText = QString("alldates,untiltoday,currentmonth,currentyear,monthtodate,yeartodate,yeartomonth,lastmonth,lastyear,last7days,last30days,last3months,last6months,last12months,next7days,next30days,next3months,next6months,next12months,userdefined,last3tonext3months,last11Months,currentQuarter,lastQuarter,nextQuarter,currentFiscalYear,lastFiscalYear,today,next18months").split(',');
 const QStringList kAccountTypeText = QString("unknown,checkings,savings,cash,creditcard,loan,certificatedep,investment,moneymarket,asset,liability,currency,income,expense,assetloan,stock,equity,invalid").split(',');
 
 MyMoneyReport::MyMoneyReport() :
@@ -137,7 +135,7 @@ MyMoneyReport::MyMoneyReport(ERowType _rt, unsigned _ct, dateOptionE _dl, EDetai
   //throw exception if the type is inconsistent
   if ((_rt > static_cast<ERowType>(sizeof(kTypeArray) / sizeof(kTypeArray[0])))
       || (m_reportType == eNoReport))
-    throw new MYMONEYEXCEPTION("Invalid report type");
+    throw MYMONEYEXCEPTION("Invalid report type");
 
   //add the corresponding account groups
   if (_rt == MyMoneyReport::eAssetLiability) {
@@ -172,6 +170,8 @@ MyMoneyReport::MyMoneyReport(const QDomElement& node) :
     MyMoneyObject(node),
     m_currentDateColumn(0)
 {
+  // properly initialize the object before reading it
+  *this = MyMoneyReport();
   if (!read(node))
     clearId();
 }
@@ -478,6 +478,28 @@ void MyMoneyReport::write(QDomElement& e, QDomDocument *doc, bool anonymous) con
     }
   }
 
+  //
+  // Tags Filter
+  //
+
+  QStringList taglist;
+  if (tags(taglist)) {
+    if (taglist.empty()) {
+      QDomElement p = doc->createElement("TAG");
+      e.appendChild(p);
+    } else {
+      // iterate over tags, and add each one
+      QStringList::const_iterator it_tag = taglist.constBegin();
+      while (it_tag != taglist.constEnd()) {
+        QDomElement p = doc->createElement("TAG");
+        p.setAttribute("id", *it_tag);
+        e.appendChild(p);
+
+        ++it_tag;
+      }
+    }
+  }
+
   //
   // Account Groups Filter
   //
@@ -726,6 +748,9 @@ bool MyMoneyReport::read(const QDomElement& e)
       if ("PAYEE" == c.tagName()) {
         addPayee(c.attribute("id"));
       }
+      if ("TAG" == c.tagName()) {
+        addTag(c.attribute("id"));
+      }
       if ("CATEGORY" == c.tagName() && c.hasAttribute("id")) {
         addCategory(c.attribute("id"));
       }
@@ -758,6 +783,7 @@ bool MyMoneyReport::hasReferenceTo(const QString& id) const
   accounts(list);
   categories(list);
   payees(list);
+  tags(list);
 
   return (list.contains(id) > 0);
 }
@@ -768,5 +794,3 @@ void MyMoneyReport::setLineWidth(int width)
 {
   m_lineWidth = width;
 }
-
-// vim:cin:si:ai:et:ts=2:sw=2:
