@@ -177,8 +177,8 @@ void PivotTable::init(void)
   m_config_f.setConsiderCategory(true);
   try {
     transactions = file->transactionList(m_config_f);
-  } catch (MyMoneyException *e) {
-    qDebug("ERR: %s thrown in %s(%ld)", qPrintable(e->what()), qPrintable(e->file()), e->line());
+  } catch (const MyMoneyException &e) {
+    qDebug("ERR: %s thrown in %s(%ld)", qPrintable(e.what()), qPrintable(e.file()), e.line());
     throw e;
   }
   DEBUG_OUTPUT(QString("Found %1 matching transactions").arg(transactions.count()));
@@ -465,9 +465,9 @@ void PivotTable::accumulateColumn(int destcolumn, int sourcecolumn)
       PivotInnerGroup::iterator it_row = (*it_innergroup).begin();
       while (it_row != (*it_innergroup).end()) {
         if ((*it_row)[eActual].count() <= sourcecolumn)
-          throw new MYMONEYEXCEPTION(QString("Sourcecolumn %1 out of grid range (%2) in PivotTable::accumulateColumn").arg(sourcecolumn).arg((*it_row)[eActual].count()));
+          throw MYMONEYEXCEPTION(QString("Sourcecolumn %1 out of grid range (%2) in PivotTable::accumulateColumn").arg(sourcecolumn).arg((*it_row)[eActual].count()));
         if ((*it_row)[eActual].count() <= destcolumn)
-          throw new MYMONEYEXCEPTION(QString("Destcolumn %1 out of grid range (%2) in PivotTable::accumulateColumn").arg(sourcecolumn).arg((*it_row)[eActual].count()));
+          throw MYMONEYEXCEPTION(QString("Destcolumn %1 out of grid range (%2) in PivotTable::accumulateColumn").arg(sourcecolumn).arg((*it_row)[eActual].count()));
 
         (*it_row)[eActual][destcolumn] += (*it_row)[eActual][sourcecolumn];
         ++it_row;
@@ -494,7 +494,7 @@ void PivotTable::clearColumn(int column)
       PivotInnerGroup::iterator it_row = (*it_innergroup).begin();
       while (it_row != (*it_innergroup).end()) {
         if ((*it_row)[eActual].count() <= column)
-          throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::accumulateColumn").arg(column).arg((*it_row)[eActual].count()));
+          throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::accumulateColumn").arg(column).arg((*it_row)[eActual].count()));
 
         (*it_row++)[eActual][column] = PivotCell();
       }
@@ -678,7 +678,7 @@ void PivotTable::calculateRunningSums(PivotInnerGroup::iterator& it_row)
   int column = 1;
   while (column < m_numColumns) {
     if (it_row.value()[eActual].count() <= column)
-      throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateRunningSums").arg(column).arg(it_row.value()[eActual].count()));
+      throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateRunningSums").arg(column).arg(it_row.value()[eActual].count()));
 
     runningsum = it_row.value()[eActual][column].calculateRunningSum(runningsum);
 
@@ -703,7 +703,7 @@ void PivotTable::calculateRunningSums(void)
         int column = 1;
         while (column < m_numColumns) {
           if (it_row.value()[eActual].count() <= column)
-            throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateRunningSums").arg(column).arg(it_row.value()[eActual].count()));
+            throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateRunningSums").arg(column).arg(it_row.value()[eActual].count()));
 
           runningsum = (it_row.value()[eActual][column] += runningsum);
 
@@ -723,7 +723,7 @@ MyMoneyMoney PivotTable::cellBalance(const QString& outergroup, const ReportAcco
 {
   if (m_runningSumsCalculated) {
     qDebug("You must not call PivotTable::cellBalance() after calling PivotTable::calculateRunningSums()");
-    throw new MYMONEYEXCEPTION(QString("You must not call PivotTable::cellBalance() after calling PivotTable::calculateRunningSums()"));
+    throw MYMONEYEXCEPTION(QString("You must not call PivotTable::cellBalance() after calling PivotTable::calculateRunningSums()"));
   }
 
   // for budget reports, if this is the actual value, map it to the account which
@@ -747,9 +747,9 @@ MyMoneyMoney PivotTable::cellBalance(const QString& outergroup, const ReportAcco
   QString innergroup(row.topParentName());
 
   if (m_numColumns <= _column)
-    throw new MYMONEYEXCEPTION(QString("Column %1 out of m_numColumns range (%2) in PivotTable::cellBalance").arg(_column).arg(m_numColumns));
+    throw MYMONEYEXCEPTION(QString("Column %1 out of m_numColumns range (%2) in PivotTable::cellBalance").arg(_column).arg(m_numColumns));
   if (m_grid[outergroup][innergroup][row][eActual].count() <= _column)
-    throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::cellBalance").arg(_column).arg(m_grid[outergroup][innergroup][row][eActual].count()));
+    throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::cellBalance").arg(_column).arg(m_grid[outergroup][innergroup][row][eActual].count()));
 
   MyMoneyMoney balance;
   if (budget)
@@ -760,7 +760,7 @@ MyMoneyMoney PivotTable::cellBalance(const QString& outergroup, const ReportAcco
   int column = 1;
   while (column < _column) {
     if (m_grid[outergroup][innergroup][row][eActual].count() <= column)
-      throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::cellBalance").arg(column).arg(m_grid[outergroup][innergroup][row][eActual].count()));
+      throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::cellBalance").arg(column).arg(m_grid[outergroup][innergroup][row][eActual].count()));
 
     balance = m_grid[outergroup][innergroup][row][eActual][column].cellBalance(balance);
 
@@ -884,8 +884,8 @@ void PivotTable::calculateBudgetMapping(void)
 
         // skip the account if it has no periods
         if (periods.count() < 1) {
-            ++it_bacc;
-            continue;
+          ++it_bacc;
+          continue;
         }
 
         MyMoneyMoney value = (*periods.begin()).amount() * reverse;
@@ -975,7 +975,7 @@ void PivotTable::convertToBaseCurrency(void)
         int column = 1;
         while (column < m_numColumns) {
           if (it_row.value()[eActual].count() <= column)
-            throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::convertToBaseCurrency").arg(column).arg(it_row.value()[eActual].count()));
+            throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::convertToBaseCurrency").arg(column).arg(it_row.value()[eActual].count()));
 
           QDate valuedate = columnDate(column);
 
@@ -1020,7 +1020,7 @@ void PivotTable::convertToDeepCurrency(void)
         int column = 1;
         while (column < m_numColumns) {
           if (it_row.value()[eActual].count() <= column)
-            throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::convertToDeepCurrency").arg(column).arg(it_row.value()[eActual].count()));
+            throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::convertToDeepCurrency").arg(column).arg(it_row.value()[eActual].count()));
 
           QDate valuedate = columnDate(column);
 
@@ -1105,9 +1105,9 @@ void PivotTable::calculateTotals(void)
         while (column < m_numColumns) {
           for (int i = 0; i < m_rowTypeList.size(); ++i) {
             if (it_row.value()[ m_rowTypeList[i] ].count() <= column)
-              throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateTotals, row columns").arg(column).arg(it_row.value()[ m_rowTypeList[i] ].count()));
+              throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateTotals, row columns").arg(column).arg(it_row.value()[ m_rowTypeList[i] ].count()));
             if ((*it_innergroup).m_total[ m_rowTypeList[i] ].count() <= column)
-              throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateTotals, inner group totals").arg(column).arg((*it_innergroup).m_total[ m_rowTypeList[i] ].count()));
+              throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateTotals, inner group totals").arg(column).arg((*it_innergroup).m_total[ m_rowTypeList[i] ].count()));
 
             //calculate total
             MyMoneyMoney value = it_row.value()[ m_rowTypeList[i] ][column];
@@ -1127,9 +1127,9 @@ void PivotTable::calculateTotals(void)
       while (column < m_numColumns) {
         for (int i = 0; i < m_rowTypeList.size(); ++i) {
           if ((*it_innergroup).m_total[ m_rowTypeList[i] ].count() <= column)
-            throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateTotals, inner group totals").arg(column).arg((*it_innergroup).m_total[ m_rowTypeList[i] ].count()));
+            throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateTotals, inner group totals").arg(column).arg((*it_innergroup).m_total[ m_rowTypeList[i] ].count()));
           if ((*it_outergroup).m_total[ m_rowTypeList[i] ].count() <= column)
-            throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateTotals, outer group totals").arg(column).arg((*it_innergroup).m_total[ m_rowTypeList[i] ].count()));
+            throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateTotals, outer group totals").arg(column).arg((*it_innergroup).m_total[ m_rowTypeList[i] ].count()));
 
           //calculate totals
           MyMoneyMoney value = (*it_innergroup).m_total[ m_rowTypeList[i] ][column];
@@ -1146,21 +1146,23 @@ void PivotTable::calculateTotals(void)
     // Outer Row Group Totals
     //
 
-    bool invert_total = (*it_outergroup).m_inverted;
+    const bool isIncomeExpense = (m_config_f.rowType() == MyMoneyReport::eExpenseIncome);
+    const bool invert_total = (*it_outergroup).m_inverted;
     int column = 1;
     while (column < m_numColumns) {
       for (int i = 0; i < m_rowTypeList.size(); ++i) {
         if (m_grid.m_total[ m_rowTypeList[i] ].count() <= column)
-          throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateTotals, grid totals").arg(column).arg((*it_innergroup).m_total[ m_rowTypeList[i] ].count()));
+          throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::calculateTotals, grid totals").arg(column).arg((*it_innergroup).m_total[ m_rowTypeList[i] ].count()));
 
         //calculate actual totals
         MyMoneyMoney value = (*it_outergroup).m_total[ m_rowTypeList[i] ][column];
         (*it_outergroup).m_total[ m_rowTypeList[i] ].m_total += value;
 
         //so far the invert only applies to actual and budget
-        if (invert_total
-            && m_rowTypeList[i] != eBudgetDiff
-            &&  m_rowTypeList[i] != eForecast)
+        if (invert_total && m_rowTypeList[i] != eBudgetDiff && m_rowTypeList[i] != eForecast)
+          value = -value;
+        // forecast income expense reports should be inverted as oposed to asset/liability reports
+        if (invert_total && isIncomeExpense && m_rowTypeList[i] == eForecast)
           value = -value;
 
         m_grid.m_total[ m_rowTypeList[i] ][column] += value;
@@ -1178,7 +1180,7 @@ void PivotTable::calculateTotals(void)
   while (totalcolumn < m_numColumns) {
     for (int i = 0; i < m_rowTypeList.size(); ++i) {
       if (m_grid.m_total[ m_rowTypeList[i] ].count() <= totalcolumn)
-        throw new MYMONEYEXCEPTION(QString("Total column %1 out of grid range (%2) in PivotTable::calculateTotals, grid totals").arg(totalcolumn).arg(m_grid.m_total[ m_rowTypeList[i] ].count()));
+        throw MYMONEYEXCEPTION(QString("Total column %1 out of grid range (%2) in PivotTable::calculateTotals, grid totals").arg(totalcolumn).arg(m_grid.m_total[ m_rowTypeList[i] ].count()));
 
       //calculate actual totals
       MyMoneyMoney value = m_grid.m_total[ m_rowTypeList[i] ][totalcolumn];
@@ -1214,9 +1216,11 @@ void PivotTable::assignCell(const QString& outergroup, const ReportAccount& _row
   QString innergroup(row.topParentName());
 
   if (m_numColumns <= column)
-    throw new MYMONEYEXCEPTION(QString("Column %1 out of m_numColumns range (%2) in PivotTable::assignCell").arg(column).arg(m_numColumns));
+    throw MYMONEYEXCEPTION(QString("Column %1 out of m_numColumns range (%2) in PivotTable::assignCell").arg(column).arg(m_numColumns));
   if (m_grid[outergroup][innergroup][row][eActual].count() <= column)
-    throw new MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::assignCell").arg(column).arg(m_grid[outergroup][innergroup][row][eActual].count()));
+    throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::assignCell").arg(column).arg(m_grid[outergroup][innergroup][row][eActual].count()));
+  if (m_grid[outergroup][innergroup][row][eBudget].count() <= column)
+    throw MYMONEYEXCEPTION(QString("Column %1 out of grid range (%2) in PivotTable::assignCell").arg(column).arg(m_grid[outergroup][innergroup][row][eBudget].count()));
 
   if (!stockSplit) {
     // Determine whether the value should be inverted before being placed in the row
@@ -1224,10 +1228,22 @@ void PivotTable::assignCell(const QString& outergroup, const ReportAccount& _row
       value = -value;
 
     // Add the value to the grid cell
-    if (budget)
+    if (budget) {
       m_grid[outergroup][innergroup][row][eBudget][column] += value;
-    else
-      m_grid[outergroup][innergroup][row][eActual][column] += value;
+    } else {
+      // If it is loading an actual value for a budget report
+      // check whether it is a subaccount of a budget account (include subaccounts)
+      // If so, check if is the same currency and convert otherwise
+      if (m_config_f.hasBudget() &&
+          row.id() != _row.id() &&
+          row.currencyId() != _row.currencyId()) {
+        ReportAccount origAcc = _row;
+        MyMoneyMoney rate = origAcc.foreignCurrencyPrice(row.currencyId(), columnDate(column), false);
+        m_grid[outergroup][innergroup][row][eActual][column] += (value * rate).reduce();
+      } else {
+        m_grid[outergroup][innergroup][row][eActual][column] += value;
+      }
+    }
   } else {
     m_grid[outergroup][innergroup][row][eActual][column] += PivotCell::stockSplit(value);
   }
@@ -1263,7 +1279,7 @@ void PivotTable::createRow(const QString& outergroup, const ReportAccount& row,
 int PivotTable::columnValue(const QDate& _date) const
 {
   if (m_config_f.isColumnsAreDays())
-    return (QDate().daysTo(_date));
+    return (m_beginDate.daysTo(_date));
   else
     return (_date.year() * 12 + _date.month());
 }
@@ -1983,9 +1999,8 @@ void PivotTable::calculateForecast(void)
         } else {
           //if columns are months
           while (column < m_numColumns) {
-            //set forecastDate to last day of each month
-            //TODO we really need a date manipulation util
-            forecastDate = QDate(forecastDate.year(), forecastDate.month(), forecastDate.daysInMonth());
+            // the forecast balance is on the first day of the month see MyMoneyForecast::calculateScheduledMonthlyBalances()
+            forecastDate = QDate(forecastDate.year(), forecastDate.month(), 1);
             //check that forecastDate is not over ending date
             if (forecastDate > m_endDate)
               forecastDate = m_endDate;
@@ -1993,7 +2008,7 @@ void PivotTable::calculateForecast(void)
             //get forecast balance and set the corresponding column
             it_row.value()[eForecast][column] = forecast.forecastBalance(it_row.key(), forecastDate);
 
-            forecastDate = forecastDate.addDays(1);
+            forecastDate = forecastDate.addMonths(1);
             ++column;
           }
         }
@@ -2277,4 +2292,3 @@ int PivotTable::currentDateColumn(void)
 }
 
 } // namespace
-// vim:cin:si:ai:et:ts=2:sw=2:
