@@ -76,6 +76,10 @@
 #define VIEW_HOME           "home"
 #define VIEW_REPORTS        "reports"
 
+bool accountNameLess(const MyMoneyAccount &acc1, const MyMoneyAccount &acc2)
+{
+  return acc1.name().localeAwareCompare(acc2.name()) < 0;
+}
 
 using namespace reports;
 
@@ -88,8 +92,6 @@ public:
       m_netWorthGraphLastValidSize(400, 300) {
   }
 
-  void addNameIndex(QMap<QString, MyMoneyAccount> &idx, const MyMoneyAccount& account);
-
   /**
    * daily balances of an account
    */
@@ -100,6 +102,7 @@ public:
   bool            m_showAllSchedules;
   bool            m_needReload;
   MyMoneyForecast m_forecast;
+  MyMoneyMoney    m_total;
   /**
     * Hold the last valid size of the net worth graph
     * for the times when the needed size can't be computed.
@@ -112,23 +115,6 @@ public:
   QMap<QString, dailyBalances> m_accountList;
 };
 
-void KHomeView::Private::addNameIndex(QMap<QString, MyMoneyAccount> &idx, const MyMoneyAccount& account)
-{
-  QString key = account.name();
-
-  if (idx[key].id().isEmpty()) {
-    idx[key] = account;
-    //take care of accounts with duplicate names
-  } else if (idx[key].id() != account.id()) {
-    key = account.name() + "[%1]";
-    int dup = 2;
-    while (!idx[key.arg(dup)].id().isEmpty()
-           && idx[key.arg(dup)].id() != account.id())
-      ++dup;
-    idx[key.arg(dup)] = account;
-  }
-}
-
 KHomeView::KHomeView(QWidget *parent, const char *name) :
     KMyMoneyViewBase(parent, name, i18n("Home")),
     d(new Private)
@@ -204,7 +190,6 @@ void KHomeView::loadView(void)
 {
   d->m_part->setFontScaleFactor(KMyMoneyGlobalSettings::fontSizePercentage());
 
-  ::timetrace("Start KHomeView::loadView");
   QList<MyMoneyAccount> list;
   MyMoneyFile::instance()->accountList(list);
   if (list.count() == 0) {
@@ -233,19 +218,16 @@ void KHomeView::loadView(void)
 
     QStringList::ConstIterator it;
 
-    ::timetrace("Done preparation");
     for (it = settings.constBegin(); it != settings.constEnd(); ++it) {
       int option = (*it).toInt();
       if (option > 0) {
         switch (option) {
           case 1:         // payments
             showPayments();
-            ::timetrace("Done KHomeView::showPayments");
             break;
 
           case 2:         // preferred accounts
             showAccounts(Preferred, i18n("Preferred Accounts"));
-            ::timetrace("Done KHomeView::showPreferredAccounts");
             break;
 
           case 3:         // payment accounts
@@ -256,31 +238,24 @@ void KHomeView::loadView(void)
             } else {
               showAccounts(Payment, i18n("Payment Accounts"));
             }
-            ::timetrace("Done KHomeView::showPaymentAccounts");
             break;
           case 4:         // favorite reports
             showFavoriteReports();
-            ::timetrace("Done KHomeView::showFavoriteReports");
             break;
           case 5:         // forecast
             showForecast();
-            ::timetrace("Done KHomeView::showForecast");
             break;
           case 6:         // net worth graph over all accounts
             showNetWorthGraph();
-            ::timetrace("Done KHomeView::showNetWorthGraph");
             break;
           case 8:         // assets and liabilities
             showAssetsLiabilities();
-            ::timetrace("Done KHomeView::showAssetsLiabilities");
             break;
           case 9:         // budget
             showBudget();
-            ::timetrace("Done KHomeView::showBudget");
             break;
           case 10:         // cash flow summary
             showCashFlowSummary();
-            ::timetrace("Done KHomeView::showCashFlowSummary");
             break;
 
 
@@ -299,7 +274,6 @@ void KHomeView::loadView(void)
     d->m_part->write(d->m_html);
     d->m_part->end();
   }
-  ::timetrace("Done KHomeView::loadView");
 }
 
 void KHomeView::showNetWorthGraph(void)
@@ -656,24 +630,22 @@ void KHomeView::showPaymentEntry(const MyMoneySchedule& sched, int cnt)
         d->m_html += tmp;
       }
     }
-  } catch (MyMoneyException* e) {
-    qDebug("Unable to display schedule entry: %s", qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    qDebug("Unable to display schedule entry: %s", qPrintable(e.what()));
   }
 }
 
 void KHomeView::showAccounts(KHomeView::paymentTypeE type, const QString& header)
 {
   MyMoneyFile* file = MyMoneyFile::instance();
+  int prec = MyMoneyMoney::denomToPrec(file->baseCurrency().smallestAccountFraction());
   QList<MyMoneyAccount> accounts;
-  QList<MyMoneyAccount>::Iterator it;
-  QMap<QString, MyMoneyAccount> nameIdx;
 
   bool showClosedAccounts = kmymoney->toggleAction("view_show_all_accounts")->isChecked();
 
   // get list of all accounts
   file->accountList(accounts);
-  for (it = accounts.begin(); it != accounts.end();) {
+  for (QList<MyMoneyAccount>::Iterator it = accounts.begin(); it != accounts.end();) {
     bool removeAccount = false;
     if (!(*it).isClosed() || showClosedAccounts) {
       switch ((*it).accountType()) {
@@ -737,23 +709,43 @@ void KHomeView::showAccounts(KHomeView::paymentTypeE type, const QString& header
 
     if (removeAccount)
       it = accounts.erase(it);
-    else {
-      d->addNameIndex(nameIdx, *it);
+    else
       ++it;
-    }
   }
 
   if (!accounts.isEmpty()) {
+    // sort the accounts by name
+    qStableSort(accounts.begin(), accounts.end(), accountNameLess);
     QString tmp;
     int i = 0;
     tmp = "<div class=\"shadow\"><div class=\"displayblock\"><div class=\"summaryheader\">" + header + "</div>\n<div class=\"gap\">&nbsp;</div>\n";
     d->m_html += tmp;
     d->m_html += "<table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
-    d->m_html += "<tr class=\"item\"><td class=\"left\" width=\"35%\">";
+    d->m_html += "<tr class=\"item\">";
+
+    if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) {
+      QString pathStatusHeader;
+      KIconLoader::global()->loadIcon(QString("download"), KIconLoader::Small, KIconLoader::SizeSmall, KIconLoader::DefaultState, QStringList(), &pathStatusHeader);
+      d->m_html += QString("<td class=\"center\"><img src=\"%1\" border=\"0\"></td>").arg(pathStatusHeader);
+    }
+
+    d->m_html += "<td class=\"left\" width=\"35%\">";
     d->m_html += i18n("Account");
-    d->m_html += "</td><td width=\"25%\" class=\"right\">";
+    d->m_html += "</td>";
+
+    if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions())
+      d->m_html += QString("<td class=\"center\">!M</td>");
+
+    if (KMyMoneyGlobalSettings::showCountOfClearedTransactions())
+      d->m_html += QString("<td class=\"center\">C</td>");
+
+    if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
+      d->m_html += QString("<td class=\"center\">!R</td>");
+
+    d->m_html += "<td width=\"25%\" class=\"right\">";
     d->m_html += i18n("Current Balance");
     d->m_html += "</td>";
+
     //only show limit info if user chose to do so
     if (KMyMoneyGlobalSettings::showLimitInfo()) {
       d->m_html += "<td width=\"40%\" class=\"right\">";
@@ -762,13 +754,21 @@ void KHomeView::showAccounts(KHomeView::paymentTypeE type, const QString& header
     }
     d->m_html += "</tr>";
 
-
-    QMap<QString, MyMoneyAccount>::const_iterator it_m;
-    for (it_m = nameIdx.constBegin(); it_m != nameIdx.constEnd(); ++it_m) {
+    d->m_total = 0;
+    QList<MyMoneyAccount>::const_iterator it_m;
+    for (it_m = accounts.constBegin(); it_m != accounts.constEnd(); ++it_m) {
       d->m_html += QString("<tr class=\"row-%1\">").arg(i++ & 0x01 ? "even" : "odd");
       showAccountEntry(*it_m);
       d->m_html += "</tr>";
     }
+    d->m_html += QString("<tr class=\"row-%1\">").arg(i++ & 0x01 ? "even" : "odd");
+    QString amount = d->m_total.formatMoney(file->baseCurrency().tradingSymbol(), prec);
+    if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) d->m_html += "<td></td>";
+    d->m_html += QString("<td class=\"right\"><b>%1</b></td>").arg(i18n("Total"));
+    if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions()) d->m_html += "<td></td>";
+    if (KMyMoneyGlobalSettings::showCountOfClearedTransactions()) d->m_html += "<td></td>";
+    if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions()) d->m_html += "<td></td>";
+    d->m_html += QString("<td class=\"right\"><b>%1</b></td></tr>").arg(showColoredAmount(amount, d->m_total.isNegative()));
     d->m_html += "</table></div></div>";
   }
 }
@@ -790,7 +790,15 @@ void KHomeView::showAccountEntry(const MyMoneyAccount& acc)
   } else {
     //get balance for normal accounts
     value = file->balance(acc.id(), QDate::currentDate());
-
+    if (acc.currencyId() != file->baseCurrency().id()) {
+      ReportAccount repAcc = ReportAccount(acc.id());
+      MyMoneyMoney curPrice = repAcc.baseCurrencyPrice(QDate::currentDate());
+      MyMoneyMoney baseValue = value * curPrice;
+      baseValue = baseValue.convert(file->baseCurrency().smallestAccountFraction());
+      d->m_total += baseValue;
+    } else {
+      d->m_total += value;
+    }
     //if credit card or checkings account, show maximum credit
     if (acc.accountType() == MyMoneyAccount::CreditCard ||
         acc.accountType() == MyMoneyAccount::Checkings) {
@@ -825,8 +833,65 @@ void KHomeView::showAccountEntry(const MyMoneyAccount& acc, const MyMoneyMoney&
     amountToMinBal.replace(QChar(' '), "&nbsp;");
   }
 
-  tmp = QString("<td>") +
-        link(VIEW_LEDGER, QString("?id=%1").arg(acc.id())) + acc.name() + linkend() + "</td>";
+  QString cellStatus, cellCounts, pathOK, pathTODO, pathNotOK;
+
+  if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) {
+    //show account's online-status
+    KIconLoader::global()->loadIcon(QString("dialog-ok-apply"), KIconLoader::Small, KIconLoader::SizeSmall, KIconLoader::DefaultState, QStringList(), &pathOK);
+    KIconLoader::global()->loadIcon(QString("mail-receive"), KIconLoader::Small, KIconLoader::SizeSmall, KIconLoader::DefaultState, QStringList(), &pathTODO);
+    KIconLoader::global()->loadIcon(QString("dialog-cancel"), KIconLoader::Small, KIconLoader::SizeSmall, KIconLoader::DefaultState, QStringList(), &pathNotOK);
+
+    if (acc.value("lastImportedTransactionDate").isEmpty() || acc.value("lastStatementBalance").isEmpty())
+      cellStatus = '-';
+    else if (file->hasMatchingOnlineBalance(acc)) {
+      if (file->hasNewerTransaction(acc.id(), QDate::fromString(acc.value("lastImportedTransactionDate"), Qt::ISODate)))
+        cellStatus = QString("<img src=\"%1\" border=\"0\">").arg(pathTODO);
+      else
+        cellStatus = QString("<img src=\"%1\" border=\"0\">").arg(pathOK);
+    } else
+      cellStatus = QString("<img src=\"%1\" border=\"0\">").arg(pathNotOK);
+
+    tmp = QString("<td class=\"center\">%1</td>").arg(cellStatus);
+  }
+
+  tmp += QString("<td>") +
+         link(VIEW_LEDGER, QString("?id=%1").arg(acc.id())) + acc.name() + linkend() + "</td>";
+
+  int countNotMarked = 0, countCleared = 0, countNotReconciled = 0;
+  QString countStr;
+
+  if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions() || KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
+    countNotMarked = file->countTransactionsWithSpecificReconciliationState(acc.id(), MyMoneyTransactionFilter::notReconciled);
+
+  if (KMyMoneyGlobalSettings::showCountOfClearedTransactions() || KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
+    countCleared = file->countTransactionsWithSpecificReconciliationState(acc.id(), MyMoneyTransactionFilter::cleared);
+
+  if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
+    countNotReconciled = countNotMarked + countCleared;
+
+  if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions()) {
+    if (countNotMarked)
+      countStr = QString("%1").arg(countNotMarked);
+    else
+      countStr = '-';
+    tmp += QString("<td class=\"center\">%1</td>").arg(countStr);
+  }
+
+  if (KMyMoneyGlobalSettings::showCountOfClearedTransactions()) {
+    if (countCleared)
+      countStr = QString("%1").arg(countCleared);
+    else
+      countStr = '-';
+    tmp += QString("<td class=\"center\">%1</td>").arg(countStr);
+  }
+
+  if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions()) {
+    if (countNotReconciled)
+      countStr = QString("%1").arg(countNotReconciled);
+    else
+      countStr = '-';
+    tmp += QString("<td class=\"center\">%1</td>").arg(countStr);
+  }
 
   //show account balance
   tmp += QString("<td class=\"right\">%1</td>").arg(showColoredAmount(amount, value.isNegative()));
@@ -865,9 +930,8 @@ MyMoneyMoney KHomeView::investmentBalance(const MyMoneyAccount& acc)
         val = val * file->price(security.tradingCurrency(), accountCurrency.id()).rate(accountCurrency.id());
         val = val.convert(acc.fraction());
         value += val;
-      } catch (MyMoneyException* e) {
-        qWarning("%s", qPrintable(QString("cannot convert stock balance of %1 to base currency: %2").arg(stock.name(), e->what())));
-        delete e;
+      } catch (const MyMoneyException &e) {
+        qWarning("%s", qPrintable(QString("cannot convert stock balance of %1 to base currency: %2").arg(stock.name(), e.what())));
       }
     }
   }
@@ -912,7 +976,6 @@ void KHomeView::showFavoriteReports(void)
 
 void KHomeView::showForecast(void)
 {
-  QMap<QString, MyMoneyAccount> nameIdx;
   MyMoneyFile* file = MyMoneyFile::instance();
   QList<MyMoneyAccount> accList;
 
@@ -922,13 +985,9 @@ void KHomeView::showForecast(void)
 
   accList = d->m_forecast.accountList();
 
-  //add it to a map to have it ordered by name
-  QList<MyMoneyAccount>::const_iterator accList_t = accList.constBegin();
-  for (; accList_t != accList.constEnd(); ++accList_t) {
-    d->addNameIndex(nameIdx, *accList_t);
-  }
-
-  if (nameIdx.count() > 0) {
+  if (accList.count() > 0) {
+    // sort the accounts by name
+    qStableSort(accList.begin(), accList.end(), accountNameLess);
     int i = 0;
 
     int colspan = 1;
@@ -957,8 +1016,8 @@ void KHomeView::showForecast(void)
     // Now output entries
     i = 0;
 
-    QMap<QString, MyMoneyAccount>::ConstIterator it_account;
-    for (it_account = nameIdx.constBegin(); it_account != nameIdx.constEnd(); ++it_account) {
+    QList<MyMoneyAccount>::ConstIterator it_account;
+    for (it_account = accList.constBegin(); it_account != accList.constEnd(); ++it_account) {
       //MyMoneyAccount acc = (*it_n);
 
       d->m_html += QString("<tr class=\"row-%1\">").arg(i++ & 0x01 ? "even" : "odd");
@@ -1154,8 +1213,8 @@ void KHomeView::showAssetsLiabilities(void)
 {
   QList<MyMoneyAccount> accounts;
   QList<MyMoneyAccount>::ConstIterator it;
-  QMap<QString, MyMoneyAccount> nameAssetsIdx;
-  QMap<QString, MyMoneyAccount> nameLiabilitiesIdx;
+  QList<MyMoneyAccount> assets;
+  QList<MyMoneyAccount> liabilities;
   MyMoneyMoney netAssets;
   MyMoneyMoney netLiabilities;
   QString fontStart, fontEnd;
@@ -1172,7 +1231,7 @@ void KHomeView::showAssetsLiabilities(void)
       switch ((*it).accountType()) {
           // group all assets into one list but make sure that investment accounts always show up
         case MyMoneyAccount::Investment:
-          d->addNameIndex(nameAssetsIdx, *it);
+          assets << *it;
           break;
 
         case MyMoneyAccount::Checkings:
@@ -1182,7 +1241,7 @@ void KHomeView::showAssetsLiabilities(void)
         case MyMoneyAccount::AssetLoan:
           // list account if it's the last in the hierarchy or has transactions in it
           if ((*it).accountList().isEmpty() || (file->transactionCount((*it).id()) > 0)) {
-            d->addNameIndex(nameAssetsIdx, *it);
+            assets << *it;
           }
           break;
 
@@ -1192,7 +1251,7 @@ void KHomeView::showAssetsLiabilities(void)
         case MyMoneyAccount::Loan:
           // list account if it's the last in the hierarchy or has transactions in it
           if ((*it).accountList().isEmpty() || (file->transactionCount((*it).id()) > 0)) {
-            d->addNameIndex(nameLiabilitiesIdx, *it);
+            liabilities << *it;
           }
           break;
 
@@ -1204,33 +1263,87 @@ void KHomeView::showAssetsLiabilities(void)
   }
 
   //only do it if we have assets or liabilities account
-  if (nameAssetsIdx.count() > 0 || nameLiabilitiesIdx.count() > 0) {
+  if (assets.count() > 0 || liabilities.count() > 0) {
+    // sort the accounts by name
+    qStableSort(assets.begin(), assets.end(), accountNameLess);
+    qStableSort(liabilities.begin(), liabilities.end(), accountNameLess);
+    QString statusHeader;
+    if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) {
+      QString pathStatusHeader;
+      KIconLoader::global()->loadIcon(QString("online-banking"), KIconLoader::Small, KIconLoader::SizeSmall, KIconLoader::DefaultState, QStringList(), &pathStatusHeader);
+      statusHeader = QString("<img src=\"%1\" border=\"0\">").arg(pathStatusHeader);
+    }
+
     //print header
     d->m_html += "<div class=\"shadow\"><div class=\"displayblock\"><div class=\"summaryheader\">" + i18n("Assets and Liabilities Summary") + "</div>\n<div class=\"gap\">&nbsp;</div>\n";
     d->m_html += "<table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
+
     //column titles
-    d->m_html += "<tr class=\"item\"><td class=\"left\" width=\"30%\">";
+
+    d->m_html += "<tr class=\"item\">";
+
+    if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) {
+      d->m_html += "<td class=\"setcolor\">";
+      d->m_html += statusHeader;
+      d->m_html += "</td>";
+    }
+
+    d->m_html += "<td class=\"left\" width=\"30%\">";
     d->m_html += i18n("Asset Accounts");
     d->m_html += "</td>";
+
+    if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions())
+      d->m_html += "<td class=\"setcolor\">!M</td>";
+
+    if (KMyMoneyGlobalSettings::showCountOfClearedTransactions())
+      d->m_html += "<td class=\"setcolor\">C</td>";
+
+    if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
+      d->m_html += "<td class=\"setcolor\">!R</td>";
+
     d->m_html += "<td width=\"15%\" class=\"right\">";
     d->m_html += i18n("Current Balance");
     d->m_html += "</td>";
+
     //intermediate row to separate both columns
     d->m_html += "<td width=\"10%\" class=\"setcolor\"></td>";
+
+    if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) {
+      d->m_html += "<td class=\"setcolor\">";
+      d->m_html += statusHeader;
+      d->m_html += "</td>";
+    }
+
     d->m_html += "<td class=\"left\" width=\"30%\">";
     d->m_html += i18n("Liability Accounts");
     d->m_html += "</td>";
+
+    if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions())
+      d->m_html += "<td class=\"setcolor\">!M</td>";
+
+    if (KMyMoneyGlobalSettings::showCountOfClearedTransactions())
+      d->m_html += "<td class=\"setcolor\">C</td>";
+
+    if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
+      d->m_html += "<td class=\"setcolor\">!R</td>";
+
     d->m_html += "<td width=\"15%\" class=\"right\">";
     d->m_html += i18n("Current Balance");
     d->m_html += "</td></tr>";
 
+    QString placeHolder_Status, placeHolder_Counts;
+    if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) placeHolder_Status = "<td></td>";
+    if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions()) placeHolder_Counts = "<td></td>";
+    if (KMyMoneyGlobalSettings::showCountOfClearedTransactions()) placeHolder_Counts += "<td></td>";
+    if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions()) placeHolder_Counts += "<td></td>";
+
     //get asset and liability accounts
-    QMap<QString, MyMoneyAccount>::const_iterator asset_it = nameAssetsIdx.constBegin();
-    QMap<QString, MyMoneyAccount>::const_iterator liabilities_it = nameLiabilitiesIdx.constBegin();
-    for (; asset_it != nameAssetsIdx.constEnd() || liabilities_it != nameLiabilitiesIdx.constEnd();) {
+    QList<MyMoneyAccount>::const_iterator asset_it = assets.constBegin();
+    QList<MyMoneyAccount>::const_iterator liabilities_it = liabilities.constBegin();
+    for (; asset_it != assets.constEnd() || liabilities_it != liabilities.constEnd();) {
       d->m_html += QString("<tr class=\"row-%1\">").arg(i++ & 0x01 ? "even" : "odd");
       //write an asset account if we still have any
-      if (asset_it != nameAssetsIdx.constEnd()) {
+      if (asset_it != assets.constEnd()) {
         MyMoneyMoney value;
         //investment accounts consolidate the balance of its subaccounts
         if ((*asset_it).accountType() == MyMoneyAccount::Investment) {
@@ -1254,14 +1367,14 @@ void KHomeView::showAssetsLiabilities(void)
         ++asset_it;
       } else {
         //write a white space if we don't
-        d->m_html += "<td></td><td></td>";
+        d->m_html += QString("%1<td></td>%2<td></td>").arg(placeHolder_Status).arg(placeHolder_Counts);
       }
 
       //leave the intermediate column empty
       d->m_html += "<td class=\"setcolor\"></td>";
 
       //write a liability account
-      if (liabilities_it != nameLiabilitiesIdx.constEnd()) {
+      if (liabilities_it != liabilities.constEnd()) {
         MyMoneyMoney value;
         value = MyMoneyFile::instance()->balance((*liabilities_it).id(), QDate::currentDate());
         //calculate balance if foreign currency
@@ -1279,7 +1392,7 @@ void KHomeView::showAssetsLiabilities(void)
         ++liabilities_it;
       } else {
         //leave the space empty if we run out of liabilities
-        d->m_html += "<td></td><td></td>";
+        d->m_html += QString("%1<td></td>%2<td></td>").arg(placeHolder_Status).arg(placeHolder_Counts);
       }
       d->m_html += "</tr>";
     }
@@ -1298,20 +1411,20 @@ void KHomeView::showAssetsLiabilities(void)
     d->m_html += QString("<tr class=\"row-%1\" style=\"font-weight:bold;\">").arg(i++ & 0x01 ? "even" : "odd");
 
     //print total for assets
-    d->m_html += QString("<td class=\"left\">%1</td><td align=\"right\">%2</td>").arg(i18n("Total Assets")).arg(showColoredAmount(amountAssets, netAssets.isNegative()));
+    d->m_html += QString("%1<td class=\"left\">%2</td>%3<td align=\"right\">%4</td>").arg(placeHolder_Status).arg(i18n("Total Assets")).arg(placeHolder_Counts).arg(showColoredAmount(amountAssets, netAssets.isNegative()));
 
     //leave the intermediate column empty
     d->m_html += "<td class=\"setcolor\"></td>";
 
     //print total liabilities
-    d->m_html += QString("<td class=\"left\">%1</td><td align=\"right\">%2</td>").arg(i18n("Total Liabilities")).arg(showColoredAmount(amountLiabilities, netLiabilities.isNegative()));
+    d->m_html += QString("%1<td class=\"left\">%2</td>%3<td align=\"right\">%4</td>").arg(placeHolder_Status).arg(i18n("Total Liabilities")).arg(placeHolder_Counts).arg(showColoredAmount(amountLiabilities, netLiabilities.isNegative()));
     d->m_html += "</tr>";
 
     //print net worth
     d->m_html += QString("<tr class=\"row-%1\" style=\"font-weight:bold;\">").arg(i++ & 0x01 ? "even" : "odd");
 
-    d->m_html += "<td></td><td></td><td class=\"setcolor\"></td>";
-    d->m_html += QString("<td class=\"left\">%1</td><td align=\"right\">%2</td>").arg(i18n("Net Worth")).arg(showColoredAmount(amountNetWorth, netWorth.isNegative()));
+    d->m_html += QString("%1<td></td><td></td>%2<td class=\"setcolor\"></td>").arg(placeHolder_Status).arg(placeHolder_Counts);
+    d->m_html += QString("%1<td class=\"left\">%2</td>%3<td align=\"right\">%4</td>").arg(placeHolder_Status).arg(i18n("Net Worth")).arg(placeHolder_Counts).arg(showColoredAmount(amountNetWorth, netWorth.isNegative()));
 
     d->m_html += "</tr>";
     d->m_html += "</table>";
@@ -1900,5 +2013,3 @@ void KHomeView::showCashFlowSummary()
 #undef VIEW_WELCOME
 #undef VIEW_HOME
 #undef VIEW_REPORTS
-
-#include "khomeview.moc"
