@@ -17,8 +17,6 @@
 
 #include "kmymoneyview.h"
 
-#include <unistd.h>
-
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -56,6 +54,10 @@
 #include <kdeversion.h>
 #include <ktitlewidget.h>
 
+#ifdef KActivities_FOUND
+#include <KActivities/ResourceInstance>
+#endif
+
 // ----------------------------------------------------------------------------
 // Project Includes
 
@@ -87,6 +89,7 @@
 #include "kcategoriesview.h"
 #include "kinstitutionsview.h"
 #include "kpayeesview.h"
+#include "ktagsview.h"
 #include "kscheduledview.h"
 #include "kgloballedgerview.h"
 #include "kinvestmentview.h"
@@ -108,6 +111,9 @@ KMyMoneyView::KMyMoneyView(QWidget *parent)
     m_fileOpen(false),
     m_fmode(0600),
     m_lastViewSelected(0)
+#ifdef KActivities_FOUND
+    , m_activityResourceInstance(0)
+#endif
 {
   // this is a workaround for the bug in KPageWidget that causes the header to be shown
   // for a short while during page switch which causes a kind of bouncing of the page's
@@ -157,7 +163,7 @@ KMyMoneyView::KMyMoneyView(QWidget *parent)
   // Page 1
   m_institutionsView = new KInstitutionsView();
   m_institutionsViewFrame = m_model->addPage(m_institutionsView, i18n("Institutions"));
-  m_institutionsViewFrame->setIcon(KIcon("institution"));
+  m_institutionsViewFrame->setIcon(KIcon("view-bank"));
 
   connect(m_institutionsView, SIGNAL(selectObject(MyMoneyObject)), kmymoney, SLOT(slotSelectAccount(MyMoneyObject)));
   connect(m_institutionsView, SIGNAL(selectObject(MyMoneyObject)), kmymoney, SLOT(slotSelectInstitution(MyMoneyObject)));
@@ -170,7 +176,7 @@ KMyMoneyView::KMyMoneyView(QWidget *parent)
   // Page 2
   m_accountsView = new KAccountsView();
   m_accountsViewFrame = m_model->addPage(m_accountsView, i18n("Accounts"));
-  m_accountsViewFrame->setIcon(KIcon("account")); //krazy:exclude=iconnames
+  m_accountsViewFrame->setIcon(KIcon("view-bank-account"));
 
   connect(m_accountsView, SIGNAL(selectObject(MyMoneyObject)), kmymoney, SLOT(slotSelectAccount(MyMoneyObject)));
   connect(m_accountsView, SIGNAL(selectObject(MyMoneyObject)), kmymoney, SLOT(slotSelectInstitution(MyMoneyObject)));
@@ -183,11 +189,7 @@ KMyMoneyView::KMyMoneyView(QWidget *parent)
   // Page 3
   m_scheduledView = new KScheduledView();
 //this is to solve the way long strings are handled differently among versions of KPageWidget
-#if KDE_IS_VERSION(4,4,0)
   m_scheduleViewFrame = m_model->addPage(m_scheduledView, i18n("Scheduled transactions"));
-#else
-  m_scheduleViewFrame = m_model->addPage(m_scheduledView, i18n("Scheduled\ntransactions"));
-#endif
   m_scheduleViewFrame->setIcon(KIcon("view-pim-calendar"));
 
   connect(m_scheduledView, SIGNAL(scheduleSelected(MyMoneySchedule)), kmymoney, SLOT(slotSelectSchedule(MyMoneySchedule)));
@@ -200,7 +202,7 @@ KMyMoneyView::KMyMoneyView(QWidget *parent)
   // Page 4
   m_categoriesView = new KCategoriesView();
   m_categoriesViewFrame = m_model->addPage(m_categoriesView, i18n("Categories"));
-  m_categoriesViewFrame->setIcon(KIcon("categories"));
+  m_categoriesViewFrame->setIcon(KIcon("view-financial-categories"));
 
   connect(m_categoriesView, SIGNAL(selectObject(MyMoneyObject)), kmymoney, SLOT(slotSelectAccount(MyMoneyObject)));
   connect(m_categoriesView, SIGNAL(selectObject(MyMoneyObject)), kmymoney, SLOT(slotSelectInstitution(MyMoneyObject)));
@@ -209,7 +211,20 @@ KMyMoneyView::KMyMoneyView(QWidget *parent)
   connect(m_categoriesView, SIGNAL(reparent(MyMoneyAccount,MyMoneyAccount)), kmymoney, SLOT(slotReparentAccount(MyMoneyAccount,MyMoneyAccount)));
   connect(m_categoriesView, SIGNAL(aboutToShow()), this, SIGNAL(aboutToChangeView()));
 
-  // Page 5
+// Page 5
+  m_tagsView = new KTagsView();
+  m_tagsViewFrame = m_model->addPage(m_tagsView, i18n("Tags"));
+  m_tagsViewFrame->setIcon(KIcon("mail-tagged"));
+
+  connect(kmymoney, SIGNAL(tagCreated(QString)), m_tagsView, SLOT(slotSelectTagAndTransaction(QString)));
+  connect(kmymoney, SIGNAL(tagRename()), m_tagsView, SLOT(slotRenameButtonCliked()));
+  connect(m_tagsView, SIGNAL(openContextMenu(MyMoneyObject)), kmymoney, SLOT(slotShowTagContextMenu()));
+  connect(m_tagsView, SIGNAL(selectObjects(QList<MyMoneyTag>)), kmymoney, SLOT(slotSelectTags(QList<MyMoneyTag>)));
+  connect(m_tagsView, SIGNAL(transactionSelected(QString,QString)),
+          this, SLOT(slotLedgerSelected(QString,QString)));
+  connect(m_tagsView, SIGNAL(aboutToShow()), this, SIGNAL(aboutToChangeView()));
+
+  // Page 6
   m_payeesView = new KPayeesView();
   m_payeesViewFrame = m_model->addPage(m_payeesView, i18n("Payees"));
   m_payeesViewFrame->setIcon(KIcon("system-users"));
@@ -222,10 +237,10 @@ KMyMoneyView::KMyMoneyView(QWidget *parent)
           this, SLOT(slotLedgerSelected(QString,QString)));
   connect(m_payeesView, SIGNAL(aboutToShow()), this, SIGNAL(aboutToChangeView()));
 
-  // Page 6
+  // Page 7
   m_ledgerView = new KGlobalLedgerView();
   m_ledgerViewFrame = m_model->addPage(m_ledgerView, i18n("Ledgers"));
-  m_ledgerViewFrame->setIcon(KIcon("ledger"));  //krazy:exclude=iconnames
+  m_ledgerViewFrame->setIcon(KIcon("view-financial-list"));
 
   connect(m_ledgerView, SIGNAL(accountSelected(MyMoneyObject)), kmymoney, SLOT(slotSelectAccount(MyMoneyObject)));
   connect(m_ledgerView, SIGNAL(openContextMenu()), kmymoney, SLOT(slotShowTransactionContextMenu()));
@@ -240,10 +255,10 @@ KMyMoneyView::KMyMoneyView(QWidget *parent)
   connect(m_ledgerView, SIGNAL(aboutToShow()), this, SIGNAL(aboutToChangeView()));
 
 
-  // Page 7
+  // Page 8
   m_investmentView = new KInvestmentView();
   m_investmentViewFrame = m_model->addPage(m_investmentView, i18n("Investments"));
-  m_investmentViewFrame->setIcon(KIcon("investment"));
+  m_investmentViewFrame->setIcon(KIcon("view-investment"));
 
   connect(m_investmentView, SIGNAL(accountSelected(QString,QString)),
           this, SLOT(slotLedgerSelected(QString,QString)));
@@ -251,28 +266,28 @@ KMyMoneyView::KMyMoneyView(QWidget *parent)
   connect(m_investmentView, SIGNAL(investmentRightMouseClick()), kmymoney, SLOT(slotShowInvestmentContextMenu()));
   connect(m_investmentView, SIGNAL(aboutToShow()), this, SIGNAL(aboutToChangeView()));
 
-  // Page 8
+  // Page 9
   m_reportsView = new KReportsView();
   m_reportsViewFrame = m_model->addPage(m_reportsView, i18n("Reports"));
-  m_reportsViewFrame->setIcon(KIcon("office-chart-tall-pie"));
+  m_reportsViewFrame->setIcon(KIcon("office-chart-bar"));
   connect(m_reportsView, SIGNAL(ledgerSelected(QString,QString)),
           this, SLOT(slotLedgerSelected(QString,QString)));
   connect(m_reportsView, SIGNAL(aboutToShow()), this, SIGNAL(aboutToChangeView()));
 
-  // Page 9
+  // Page 10
   m_budgetView = new KBudgetView();
   m_budgetViewFrame = m_model->addPage(m_budgetView, i18n("Budgets"));
-  m_budgetViewFrame->setIcon(KIcon("budget"));
+  m_budgetViewFrame->setIcon(KIcon("view-time-schedule-calculus"));
 
   connect(m_budgetView, SIGNAL(openContextMenu(MyMoneyObject)), kmymoney, SLOT(slotShowBudgetContextMenu()));
   connect(m_budgetView, SIGNAL(selectObjects(QList<MyMoneyBudget>)), kmymoney, SLOT(slotSelectBudget(QList<MyMoneyBudget>)));
   connect(kmymoney, SIGNAL(budgetRename()), m_budgetView, SLOT(slotStartRename()));
   connect(m_budgetView, SIGNAL(aboutToShow()), this, SIGNAL(aboutToChangeView()));
 
-  // Page 10
+  // Page 11
   m_forecastView = new KForecastView();
   m_forecastViewFrame = m_model->addPage(m_forecastView, i18n("Forecast"));
-  m_forecastViewFrame->setIcon(KIcon("forecast"));
+  m_forecastViewFrame->setIcon(KIcon("view-financial-forecast"));
   connect(m_forecastView, SIGNAL(aboutToShow()), this, SIGNAL(aboutToChangeView()));
 
   //set the model
@@ -284,6 +299,14 @@ KMyMoneyView::KMyMoneyView(QWidget *parent)
   updateViewType();
 
   m_inConstructor = false;
+
+  // Initialize kactivities resource instance
+
+#ifdef KActivities_FOUND
+  m_activityResourceInstance = new KActivities::ResourceInstance(window()->winId());
+  m_activityResourceInstance->setParent(this);
+  connect(kmymoney, SIGNAL(fileLoaded(KUrl)), m_activityResourceInstance, SLOT(setUri(KUrl)));
+#endif
 }
 
 KMyMoneyView::~KMyMoneyView()
@@ -465,35 +488,33 @@ void KMyMoneyView::removeStorage(void)
   }
 }
 
-void KMyMoneyView::enableViews(int state)
+void KMyMoneyView::enableViewsIfFileOpen()
 {
-  if (state == -1)
-    state = m_fileOpen;
-
   // call set enabled only if the state differs to avoid widgets 'bouncing on the screen' while doing this
-  if (m_accountsViewFrame->isEnabled() != state)
-    m_accountsViewFrame->setEnabled(state);
-  if (m_institutionsViewFrame->isEnabled() != state)
-    m_institutionsViewFrame->setEnabled(state);
-  if (m_scheduleViewFrame->isEnabled() != state)
-    m_scheduleViewFrame->setEnabled(state);
-  if (m_categoriesViewFrame->isEnabled() != state)
-    m_categoriesViewFrame->setEnabled(state);
-  if (m_payeesViewFrame->isEnabled() != state)
-    m_payeesViewFrame->setEnabled(state);
-  if (m_budgetViewFrame->isEnabled() != state)
-    m_budgetViewFrame->setEnabled(state);
-  if (m_ledgerViewFrame->isEnabled() != state)
-    m_ledgerViewFrame->setEnabled(state);
-  if (m_investmentViewFrame->isEnabled() != state)
-    m_investmentViewFrame->setEnabled(state);
-  if (m_reportsViewFrame->isEnabled() != state)
-    m_reportsViewFrame->setEnabled(state);
-  if (m_forecastViewFrame->isEnabled() != state)
-    m_forecastViewFrame->setEnabled(state);
-
-  emit viewStateChanged(state != 0);
+  if (m_accountsViewFrame->isEnabled() != m_fileOpen)
+    m_accountsViewFrame->setEnabled(m_fileOpen);
+  if (m_institutionsViewFrame->isEnabled() != m_fileOpen)
+    m_institutionsViewFrame->setEnabled(m_fileOpen);
+  if (m_scheduleViewFrame->isEnabled() != m_fileOpen)
+    m_scheduleViewFrame->setEnabled(m_fileOpen);
+  if (m_categoriesViewFrame->isEnabled() != m_fileOpen)
+    m_categoriesViewFrame->setEnabled(m_fileOpen);
+  if (m_payeesViewFrame->isEnabled() != m_fileOpen)
+    m_payeesViewFrame->setEnabled(m_fileOpen);
+  if (m_tagsViewFrame->isEnabled() != m_fileOpen)
+    m_tagsViewFrame->setEnabled(m_fileOpen);
+  if (m_budgetViewFrame->isEnabled() != m_fileOpen)
+    m_budgetViewFrame->setEnabled(m_fileOpen);
+  if (m_ledgerViewFrame->isEnabled() != m_fileOpen)
+    m_ledgerViewFrame->setEnabled(m_fileOpen);
+  if (m_investmentViewFrame->isEnabled() != m_fileOpen)
+    m_investmentViewFrame->setEnabled(m_fileOpen);
+  if (m_reportsViewFrame->isEnabled() != m_fileOpen)
+    m_reportsViewFrame->setEnabled(m_fileOpen);
+  if (m_forecastViewFrame->isEnabled() != m_fileOpen)
+    m_forecastViewFrame->setEnabled(m_fileOpen);
 
+  emit viewStateChanged(m_fileOpen);
 }
 
 void KMyMoneyView::slotLedgerSelected(const QString& _accId, const QString& transaction)
@@ -543,6 +564,12 @@ void KMyMoneyView::slotPayeeSelected(const QString& payee, const QString& accoun
   m_payeesView->slotSelectPayeeAndTransaction(payee, account, transaction);
 }
 
+void KMyMoneyView::slotTagSelected(const QString& tag, const QString& account, const QString& transaction)
+{
+  showPage(m_tagsViewFrame);
+  m_tagsView->slotSelectTagAndTransaction(tag, account, transaction);
+}
+
 void KMyMoneyView::slotScheduleSelected(const QString& scheduleId)
 {
   MyMoneySchedule sched = MyMoneyFile::instance()->schedule(scheduleId);
@@ -684,7 +711,6 @@ bool KMyMoneyView::readFile(const KUrl& url)
   bool haveAt = true;
 
   emit kmmFilePlugin(preOpen);
-  ::timetrace("start reading file");
   if (file.open(QIODevice::ReadOnly)) {
     QByteArray hdr(2, '\0');
     int cnt;
@@ -693,14 +719,11 @@ bool KMyMoneyView::readFile(const KUrl& url)
 
     if (cnt == 2) {
       if (QString(hdr) == QString("\037\213")) {        // gzipped?
-        ::timetrace("detected GZIP");
         qfile = KFilterDev::deviceForFile(filename, COMPRESSION_MIME_TYPE);
       } else if (QString(hdr) == QString("--")          // PGP ASCII armored?
                  || QString(hdr) == QString("\205\001")     // PGP binary?
                  || QString(hdr) == QString("\205\002")) {  // PGP binary?
-        ::timetrace("detected GPG");
         if (KGPGFile::GPGAvailable()) {
-          ::timetrace("have GPG");
           qfile = new KGPGFile(filename);
           haveAt = false;
           isEncrypted = true;
@@ -713,7 +736,6 @@ bool KMyMoneyView::readFile(const KUrl& url)
         qfile = new QFile(file.fileName());
       }
 
-      ::timetrace("open file");
       if (qfile->open(QIODevice::ReadOnly)) {
         try {
           hdr.resize(8);
@@ -747,7 +769,6 @@ bool KMyMoneyView::readFile(const KUrl& url)
               pReader = 0;
               m_fileType = KmmBinary;
             } else {
-              ::timetrace("is not binary format");
               // Scan the first 70 bytes to see if we find something
               // we know. For now, we support our own XML format and
               // GNUCash XML format. If the file is smaller, then it
@@ -762,11 +783,9 @@ bool KMyMoneyView::readFile(const KUrl& url)
                 QRegExp gncexp("<gnc-v(\\d+)");
                 QByteArray txt(hdr, 70);
                 if (kmyexp.indexIn(txt) != -1) {
-                  ::timetrace("is XML format");
                   pReader = new MyMoneyStorageXML;
                   m_fileType = KmmXML;
                 } else if (gncexp.indexIn(txt) != -1) {
-                  ::timetrace("is GNC format");
                   MyMoneyFile::instance()->attachStorage(storage);
                   MyMoneyFileTransaction ft;
                   loadDefaultCurrencies(); // currency list required for gnc
@@ -781,9 +800,7 @@ bool KMyMoneyView::readFile(const KUrl& url)
             }
             if (pReader) {
               pReader->setProgressCallback(&KMyMoneyView::progressCallback);
-              ::timetrace("read data to memory");
               pReader->readFile(qfile, dynamic_cast<IMyMoneySerialize*>(storage));
-              ::timetrace("done reading to memory");
             } else {
               if (m_fileType == KmmBinary) {
                 KMessageBox::sorry(this, QString("<qt>%1</qt>"). arg(i18n("File <b>%1</b> contains the old binary format used by KMyMoney. Please use an older version of KMyMoney (0.8.x) that still supports this format to convert it to the new XML based format.", filename)));
@@ -796,9 +813,8 @@ bool KMyMoneyView::readFile(const KUrl& url)
             KMessageBox::sorry(this, QString("<qt>%1</qt>"). arg(i18n("Cannot read from file <b>%1</b>.", filename)));
             rc = false;
           }
-        } catch (MyMoneyException *e) {
-          KMessageBox::sorry(this, QString("<qt>%1</qt>"). arg(i18n("Cannot load file <b>%1</b>. Reason: %2", filename, e->what())));
-          delete e;
+        } catch (const MyMoneyException &e) {
+          KMessageBox::sorry(this, QString("<qt>%1</qt>"). arg(i18n("Cannot load file <b>%1</b>. Reason: %2", filename, e.what())));
           rc = false;
         }
         if (pReader) {
@@ -827,7 +843,6 @@ bool KMyMoneyView::readFile(const KUrl& url)
   // objects that are cached
   MyMoneyFile::instance()->attachStorage(storage);
 
-  ::timetrace("done reading file");
   if (rc == false)
     return rc;
 
@@ -848,8 +863,7 @@ bool KMyMoneyView::readFile(const KUrl& url)
     checkAccountName(file->expense(), i18n("Expense"));
     checkAccountName(file->equity(), i18n("Equity"));
     ft.commit();
-  } catch (MyMoneyException* e) {
-    delete e;
+  } catch (const MyMoneyException &) {
   }
 
   // if a temporary file was constructed by NetAccess::download,
@@ -871,7 +885,6 @@ void KMyMoneyView::checkAccountName(const MyMoneyAccount& _acc, const QString& n
 
 bool KMyMoneyView::openDatabase(const KUrl& url)
 {
-  ::timetrace("start opening database");
   m_fileOpen = false;
 
   // open the database
@@ -929,7 +942,6 @@ bool KMyMoneyView::openDatabase(const KUrl& url)
   }
   m_fileOpen = true;
   reader->setProgressCallback(0);
-  ::timetrace("done opening database");
   return initializeStorage();
 }
 
@@ -945,8 +957,7 @@ bool KMyMoneyView::initializeStorage()
     loadDefaultCurrencies();
     loadAncientCurrencies();
     ft.commit();
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     MyMoneyFile::instance()->blockSignals(blocked);
     return false;
   }
@@ -956,9 +967,8 @@ bool KMyMoneyView::initializeStorage()
   QString baseId;
   try {
     baseId = MyMoneyFile::instance()->baseCurrency().id();
-  } catch (MyMoneyException *e) {
-    qDebug("%s", qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    qDebug("%s", qPrintable(e.what()));
   }
 
   if (baseId.isEmpty()) {
@@ -968,9 +978,8 @@ bool KMyMoneyView::initializeStorage()
       selectBaseCurrency();
       try {
         baseId = MyMoneyFile::instance()->baseCurrency().id();
-      } catch (MyMoneyException *e) {
-        qDebug("%s", qPrintable(e->what()));
-        delete e;
+      } catch (const MyMoneyException &e) {
+        qDebug("%s", qPrintable(e.what()));
       }
     }
   } else {
@@ -998,7 +1007,6 @@ bool KMyMoneyView::initializeStorage()
   } else {
     page = m_homeViewFrame;
   }
-  ::timetrace("start fixing file");
 
   // For debugging purposes, we can turn off the automatic fix manually
   // by setting the entry in kmymoneyrc to true
@@ -1034,12 +1042,11 @@ bool KMyMoneyView::initializeStorage()
             // add new levels above. Don't forget to increase currentFixVersion() for all
             // the storage backends this fix applies to
           default:
-            throw new MYMONEYEXCEPTION(i18n("Unknown fix level in input file"));
+            throw MYMONEYEXCEPTION(i18n("Unknown fix level in input file"));
         }
       }
       ft.commit();
-    } catch (MyMoneyException *e) {
-      delete e;
+    } catch (const MyMoneyException &) {
       MyMoneyFile::instance()->blockSignals(blocked);
       return false;
     }
@@ -1054,7 +1061,6 @@ bool KMyMoneyView::initializeStorage()
   // new account wizard
   // kmymoney->createInitialAccount();
 
-  ::timetrace("file open");
   m_fileOpen = true;
   emit kmmFilePlugin(postOpen);
 
@@ -1113,7 +1119,7 @@ void KMyMoneyView::saveToLocalFile(const QString& localFile, IMyMoneyStorageForm
     if (KMyMoneyGlobalSettings::encryptRecover()) {
       encryptRecover = true;
       if (!KGPGFile::keyAvailable(QString(RECOVER_KEY_ID))) {
-        KMessageBox::sorry(this, i18n("<p>You have selected to encrypt your data also with the KMyMoney recover key, but the key with id</p><p><center><b>%1</b></center></p><p>has not been found in your keyring at this time. Please make sure to import this key into your keyring. You can find it on the <a href=\"http://kmymoney.org/\">KMyMoney web-site</a>. This time your data will not be encrypted with the KMyMoney recover key.</p>", QString(RECOVER_KEY_ID)), i18n("GPG-Key not found"));
+        KMessageBox::sorry(this, i18n("<p>You have selected to encrypt your data also with the KMyMoney recover key, but the key with id</p><p><center><b>%1</b></center></p><p>has not been found in your keyring at this time. Please make sure to import this key into your keyring. You can find it on the <a href=\"http://kmymoney.org/\">KMyMoney web-site</a>. This time your data will not be encrypted with the KMyMoney recover key.</p>", QString(RECOVER_KEY_ID)), i18n("GPG Key not found"));
         encryptRecover = false;
       }
     }
@@ -1122,7 +1128,7 @@ void KMyMoneyView::saveToLocalFile(const QString& localFile, IMyMoneyStorageForm
     QStringList::const_iterator it_s;
     for (it_s = keys.constBegin(); it_s != keys.constEnd(); ++it_s) {
       if (!KGPGFile::keyAvailable(*it_s)) {
-        KMessageBox::sorry(this, i18n("<p>You have specified to encrypt your data for the user-id</p><p><center><b>%1</b>.</center></p><p>Unfortunately, a valid key for this user-id was not found in your keyring. Please make sure to import a valid key for this user-id. This time, encryption is disabled.</p>", *it_s), i18n("GPG-Key not found"));
+        KMessageBox::sorry(this, i18n("<p>You have specified to encrypt your data for the user-id</p><p><center><b>%1</b>.</center></p><p>Unfortunately, a valid key for this user-id was not found in your keyring. Please make sure to import a valid key for this user-id. This time, encryption is disabled.</p>", *it_s), i18n("GPG Key not found"));
         encryptedOk = false;
       }
     }
@@ -1159,7 +1165,7 @@ void KMyMoneyView::saveToLocalFile(const QString& localFile, IMyMoneyStorageForm
     if (!dev || !dev->open(QIODevice::WriteOnly)) {
       MyMoneyFile::instance()->blockSignals(blocked);
       delete dev;
-      throw new MYMONEYEXCEPTION(i18n("Unable to open file '%1' for writing.", localFile));
+      throw MYMONEYEXCEPTION(i18n("Unable to open file '%1' for writing.", localFile));
     }
 
   } else if (!plaintext) {
@@ -1172,14 +1178,14 @@ void KMyMoneyView::saveToLocalFile(const QString& localFile, IMyMoneyStorageForm
       if (!dev || !dev->open(QIODevice::WriteOnly)) {
         MyMoneyFile::instance()->blockSignals(blocked);
         delete dev;
-        throw new MYMONEYEXCEPTION(i18n("Unable to open file '%1' for writing.", localFile));
+        throw MYMONEYEXCEPTION(i18n("Unable to open file '%1' for writing.", localFile));
       }
       statusDevice = base->device();
     }
   } else if (plaintext) {
     qfile.open();
     if (qfile.error() != QFile::NoError) {
-      throw new MYMONEYEXCEPTION(i18n("Unable to write changes to '%1'", localFile));
+      throw MYMONEYEXCEPTION(i18n("Unable to write changes to '%1'", localFile));
     }
   }
 
@@ -1191,7 +1197,7 @@ void KMyMoneyView::saveToLocalFile(const QString& localFile, IMyMoneyStorageForm
   MyMoneyFile::instance()->blockSignals(blocked);
   QFile *fileStatusDevice = qobject_cast<QFile*>(statusDevice);
   if (fileStatusDevice->error() != QFile::NoError) {
-    throw new MYMONEYEXCEPTION(i18n("Failure while writing to '%1'", localFile));
+    throw MYMONEYEXCEPTION(i18n("Failure while writing to '%1'", localFile));
   }
   pWriter->setProgressCallback(0);
 
@@ -1200,7 +1206,7 @@ void KMyMoneyView::saveToLocalFile(const QString& localFile, IMyMoneyStorageForm
     if (fileStatusDevice->error() != QFile::NoError) {
       delete dev;
       dev = 0;
-      throw new MYMONEYEXCEPTION(i18n("Failure while writing to '%1'", localFile));
+      throw MYMONEYEXCEPTION(i18n("Failure while writing to '%1'", localFile));
     }
     delete dev;
   } else
@@ -1234,53 +1240,31 @@ bool KMyMoneyView::saveFile(const KUrl& url, const QString& keyList)
   bool rc = true;
   try {
     if (! url.isValid()) {
-      throw new MYMONEYEXCEPTION(i18n("Malformed URL '%1'", url.url()));
+      throw MYMONEYEXCEPTION(i18n("Malformed URL '%1'", url.url()));
     }
 
     if (url.isLocalFile()) {
       filename = url.toLocalFile();
-      int fmode = 0600;
-      gid_t gid = static_cast<gid_t>(-1);      // don't change the group id (see "man 2 chown")
-      QFileInfo fi(filename);
-      if (fi.exists()) {
-        fmode |= fi.permission(QFile::ReadGroup) ? 040 : 0;
-        fmode |= fi.permission(QFile::WriteGroup) ? 020 : 0;
-        fmode |= fi.permission(QFile::ReadOther) ? 004 : 0;
-        fmode |= fi.permission(QFile::WriteOther) ? 002 : 0;
-        if (fi.groupId() != static_cast<uint>(-2))
-          gid = fi.groupId();
-      }
-
-      // create a new basic block here, so that the object qfile gets
-      // deleted, before we reach the chown() call
-      {
-        //FIXME: Port to KDE4 - set the mode later
-        int mask = umask((~fmode) & 0777);
-        umask(mask);
-        try {
-          unsigned int nbak = KMyMoneyGlobalSettings::autoBackupCopies();
-          if (nbak) {
-            KSaveFile::numberedBackupFile(filename, QString(), QString::fromLatin1("~"), nbak);
-          }
-          saveToLocalFile(filename, pWriter, plaintext, keyList);
-        } catch (MyMoneyException* e) {
-          delete e;
-          throw new MYMONEYEXCEPTION(i18n("Unable to write changes to '%1'", filename));
+      try {
+        const unsigned int nbak = KMyMoneyGlobalSettings::autoBackupCopies();
+        if (nbak) {
+          KSaveFile::numberedBackupFile(filename, QString(), QString::fromLatin1("~"), nbak);
         }
+        saveToLocalFile(filename, pWriter, plaintext, keyList);
+      } catch (const MyMoneyException &) {
+        throw MYMONEYEXCEPTION(i18n("Unable to write changes to '%1'", filename));
       }
-      chown(QFile::encodeName(filename), static_cast<uid_t>(-1), gid);
     } else {
       KTemporaryFile tmpfile;
       tmpfile.open(); // to obtain the name
       saveToLocalFile(tmpfile.fileName(), pWriter, plaintext, keyList);
       if (!KIO::NetAccess::upload(tmpfile.fileName(), url, 0))
-        throw new MYMONEYEXCEPTION(i18n("Unable to upload to '%1'", url.url()));
+        throw MYMONEYEXCEPTION(i18n("Unable to upload to '%1'", url.prettyUrl()));
       tmpfile.close();
     }
     m_fileType = KmmXML;
-  } catch (MyMoneyException *e) {
-    KMessageBox::error(this, e->what());
-    delete e;
+  } catch (const MyMoneyException &e) {
+    KMessageBox::error(this, e.what());
     MyMoneyFile::instance()->setDirty();
     rc = false;
   }
@@ -1384,9 +1368,8 @@ void KMyMoneyView::slotSetBaseCurrency(const MyMoneySecurity& baseCurrency)
     QString baseId;
     try {
       baseId = MyMoneyFile::instance()->baseCurrency().id();
-    } catch (MyMoneyException *e) {
-      qDebug("%s", qPrintable(e->what()));
-      delete e;
+    } catch (const MyMoneyException &e) {
+      qDebug("%s", qPrintable(e.what()));
     }
 
     if (baseCurrency.id() != baseId) {
@@ -1394,9 +1377,8 @@ void KMyMoneyView::slotSetBaseCurrency(const MyMoneySecurity& baseCurrency)
       try {
         MyMoneyFile::instance()->setBaseCurrency(baseCurrency);
         ft.commit();
-      } catch (MyMoneyException *e) {
-        KMessageBox::sorry(this, i18n("Cannot set %1 as base currency: %2", baseCurrency.name(), e->what()), i18n("Set base currency"));
-        delete e;
+      } catch (const MyMoneyException &e) {
+        KMessageBox::sorry(this, i18n("Cannot set %1 as base currency: %2", baseCurrency.name(), e.what()), i18n("Set base currency"));
       }
     }
   }
@@ -1410,9 +1392,8 @@ void KMyMoneyView::selectBaseCurrency(void)
   QString baseId;
   try {
     baseId = MyMoneyFile::instance()->baseCurrency().id();
-  } catch (MyMoneyException *e) {
-    qDebug("%s", qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    qDebug("%s", qPrintable(e.what()));
   }
 
   if (baseId.isEmpty()) {
@@ -1424,9 +1405,8 @@ void KMyMoneyView::selectBaseCurrency(void)
 
   try {
     baseId = MyMoneyFile::instance()->baseCurrency().id();
-  } catch (MyMoneyException *e) {
-    qDebug("%s", qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    qDebug("%s", qPrintable(e.what()));
   }
 
   if (!baseId.isEmpty()) {
@@ -1448,8 +1428,7 @@ void KMyMoneyView::selectBaseCurrency(void)
       try {
         if (!(*it).currencyId().isEmpty() || (*it).currencyId().length() != 0)
           cid = MyMoneyFile::instance()->currency((*it).currencyId()).id();
-      } catch (MyMoneyException *e) {
-        delete e;
+      } catch (const MyMoneyException &) {
       }
 
       if (cid.isEmpty()) {
@@ -1458,9 +1437,8 @@ void KMyMoneyView::selectBaseCurrency(void)
         try {
           file->modifyAccount(*it);
           ft.commit();
-        } catch (MyMoneyException *e) {
-          qDebug("Unable to setup base currency in account %s (%s): %s", qPrintable((*it).name()), qPrintable((*it).id()), qPrintable(e->what()));
-          delete e;
+        } catch (const MyMoneyException &e) {
+          qDebug("Unable to setup base currency in account %s (%s): %s", qPrintable((*it).name()), qPrintable((*it).id()), qPrintable(e.what()));
         }
       }
     }
@@ -1479,16 +1457,14 @@ void KMyMoneyView::loadDefaultCurrency(const MyMoneySecurity& currency, const bo
       file->modifyCurrency(sec);
     }
     ft.commit();
-  } catch (MyMoneyException* e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     try {
       if (create) {
         file->addCurrency(currency);
       }
       ft.commit();
-    } catch (MyMoneyException* e) {
-      qDebug("Error %s loading default currency", qPrintable(e->what()));
-      delete e;
+    } catch (const MyMoneyException &e) {
+      qDebug("Error %s loading default currency", qPrintable(e.what()));
     }
   }
 }
@@ -1513,7 +1489,8 @@ void KMyMoneyView::loadDefaultCurrencies(void)
   loadDefaultCurrency(MyMoneySecurity("BHD", i18n("Bahraini Dinar"),         "BHD", 1000, 1000), create);
   loadDefaultCurrency(MyMoneySecurity("BDT", i18n("Bangladeshi Taka")), create);
   loadDefaultCurrency(MyMoneySecurity("BBD", i18n("Barbados Dollar"),        "$"), create);
-  loadDefaultCurrency(MyMoneySecurity("BYR", i18n("Belarusian Ruble"),      "BYR", 1, 1), create);
+  loadDefaultCurrency(MyMoneySecurity("BTC", i18n("Bitcoin"),                "BTC"), create);
+  loadDefaultCurrency(MyMoneySecurity("BYR", i18n("Belarusian Ruble"),       "BYR", 1, 1), create);
   loadDefaultCurrency(MyMoneySecurity("BZD", i18n("Belize Dollar"),          "$"), create);
   loadDefaultCurrency(MyMoneySecurity("BMD", i18n("Bermudian Dollar"),       "$"), create);
   loadDefaultCurrency(MyMoneySecurity("BTN", i18n("Bhutan Ngultrum")), create);
@@ -1639,7 +1616,7 @@ void KMyMoneyView::loadDefaultCurrencies(void)
   loadDefaultCurrency(MyMoneySecurity("CHF", i18n("Swiss Franc"),            "SFr"), create);
   loadDefaultCurrency(MyMoneySecurity("SYP", i18n("Syrian Pound"),           QChar(0x00A3)), create);
   loadDefaultCurrency(MyMoneySecurity("TWD", i18n("Taiwan Dollar"),          "$"), create);
-  loadDefaultCurrency(MyMoneySecurity("TJS", i18n("Tajikistan Somani")), create);
+  loadDefaultCurrency(MyMoneySecurity("TJS", i18n("Tajikistan Somoni")), create);
   loadDefaultCurrency(MyMoneySecurity("TZS", i18n("Tanzanian Shilling")), create);
   loadDefaultCurrency(MyMoneySecurity("THB", i18n("Thai Baht"),              QChar(0x0E3F)), create);
   loadDefaultCurrency(MyMoneySecurity("TOP", i18n("Tongan Pa'anga")), create);
@@ -1679,17 +1656,15 @@ void KMyMoneyView::loadAncientCurrency(const QString& id, const QString& name, c
       file->addPrice(price);
     }
     ft.commit();
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     try {
       file->addCurrency(MyMoneySecurity(id, name, sym, partsPerUnit, smallestCashFraction, smallestAccountFraction));
       if (date.isValid()) {
         file->addPrice(price);
       }
       ft.commit();
-    } catch (MyMoneyException *e) {
-      qDebug("Error loading currency: %s", qPrintable(e->what()));
-      delete e;
+    } catch (const MyMoneyException &e) {
+      qDebug("Error loading currency: %s", qPrintable(e.what()));
     }
   }
 }
@@ -1735,12 +1710,6 @@ void KMyMoneyView::loadAncientCurrencies(void)
   loadAncientCurrency("AZM", i18n("Azerbaijani Manat"), "m.", QDate(2006, 1, 1), MyMoneyMoney(1, 5000), "AZN");
 }
 
-void KMyMoneyView::viewUp(void)
-{
-  if (!fileOpen())
-    return;
-}
-
 void KMyMoneyView::viewAccountList(const QString& /*selectAccount*/)
 {
   if (m_accountsViewFrame != currentPage())
@@ -1767,6 +1736,7 @@ void KMyMoneyView::slotRefreshViews()
   m_institutionsView->slotLoadAccounts();
   m_categoriesView->slotLoadAccounts();
   m_payeesView->slotLoadPayees();
+  m_tagsView->slotLoadTags();
   m_ledgerView->slotLoadView();
   m_budgetView->slotRefreshView();
   m_homeView->slotLoadView();
@@ -1882,8 +1852,7 @@ void KMyMoneyView::fixFile_1(void)
           MyMoneyFile::instance()->modifyReport(*it_r);
         }
       }
-    } catch (MyMoneyException* e) {
-      delete e;
+    } catch (const MyMoneyException &) {
     }
   }
 }
@@ -1934,17 +1903,14 @@ void KMyMoneyView::fixFile_0(void)
   MyMoneyFile* file = MyMoneyFile::instance();
   QList<MyMoneyAccount> accountList;
   file->accountList(accountList);
-  ::timetrace("Have account list");
   QList<MyMoneyAccount>::Iterator it_a;
   QList<MyMoneySchedule> scheduleList = file->scheduleList();
-  ::timetrace("Have schedule list");
   QList<MyMoneySchedule>::Iterator it_s;
 
   MyMoneyAccount equity = file->equity();
   MyMoneyAccount asset = file->asset();
   bool equityListEmpty = equity.accountList().count() == 0;
 
-  ::timetrace("Fix accounts start");
   for (it_a = accountList.begin(); it_a != accountList.end(); ++it_a) {
     if ((*it_a).accountType() == MyMoneyAccount::Loan
         || (*it_a).accountType() == MyMoneyAccount::AssetLoan) {
@@ -1967,14 +1933,11 @@ void KMyMoneyView::fixFile_0(void)
     }
   }
 
-  ::timetrace("Fix schedules start");
   for (it_s = scheduleList.begin(); it_s != scheduleList.end(); ++it_s) {
     fixSchedule_0(*it_s);
   }
 
-  ::timetrace("Fix transactions start");
   fixTransactions_0();
-  ::timetrace("Fix transactions done");
 }
 
 void KMyMoneyView::fixSchedule_0(MyMoneySchedule sched)
@@ -1996,18 +1959,16 @@ void KMyMoneyView::fixSchedule_0(MyMoneySchedule sched)
           MyMoneyAccount acc = MyMoneyFile::instance()->account((*it_s).accountId());
           t.setCommodity(acc.currencyId());
           updated = true;
-        } catch (MyMoneyException *e) {
-          delete e;
+        } catch (const MyMoneyException &) {
         }
       }
       // make sure the account exists. If not, remove the split
       try {
         MyMoneyFile::instance()->account((*it_s).accountId());
-      } catch (MyMoneyException *e) {
+      } catch (const MyMoneyException &) {
         kDebug(2) << Q_FUNC_INFO << " " << sched.id() << " " << (*it_s).id() << " removed, because account '" << (*it_s).accountId() << "' does not exist.";
         t.removeSplit(*it_s);
         updated = true;
-        delete e;
       }
       if ((*it_s).reconcileFlag() != MyMoneySplit::NotReconciled) {
         kDebug(2) << Q_FUNC_INFO << " " << sched.id() << " " << (*it_s).id() << " should be 'not reconciled'";
@@ -2033,9 +1994,8 @@ void KMyMoneyView::fixSchedule_0(MyMoneySchedule sched)
       sched.setTransaction(t);
       MyMoneyFile::instance()->modifySchedule(sched);
     }
-  } catch (MyMoneyException *e) {
-    qWarning("Unable to update broken schedule: %s", qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    qWarning("Unable to update broken schedule: %s", qPrintable(e.what()));
   }
 }
 
@@ -2053,7 +2013,7 @@ void KMyMoneyView::fixLoanAccount_0(MyMoneyAccount acc)
                                   , acc.name()),
                              i18n("Account problem"));
 
-    throw new MYMONEYEXCEPTION("Fix LoanAccount0 not supported anymore");
+    throw MYMONEYEXCEPTION("Fix LoanAccount0 not supported anymore");
   }
 }
 
@@ -2068,7 +2028,7 @@ void KMyMoneyView::createSchedule(MyMoneySchedule newSchedule, MyMoneyAccount& n
       // We assume at least 2 splits in the transaction
       MyMoneyTransaction t = newSchedule.transaction();
       if (t.splitCount() < 2) {
-        throw new MYMONEYEXCEPTION("Transaction for schedule has less than 2 splits!");
+        throw MYMONEYEXCEPTION("Transaction for schedule has less than 2 splits!");
       }
       // now search the split that does not have an account reference
       // and set it up to be the one of the account we just added
@@ -2095,9 +2055,8 @@ void KMyMoneyView::createSchedule(MyMoneySchedule newSchedule, MyMoneyAccount& n
         MyMoneyFile::instance()->modifyAccount(newAccount);
       }
       ft.commit();
-    } catch (MyMoneyException *e) {
-      KMessageBox::information(this, i18n("Unable to add schedule: "), e->what());
-      delete e;
+    } catch (const MyMoneyException &e) {
+      KMessageBox::information(this, i18n("Unable to add schedule: %1", e.what()));
     }
   }
 }
@@ -2106,21 +2065,11 @@ void KMyMoneyView::fixTransactions_0(void)
 {
   MyMoneyFile* file = MyMoneyFile::instance();
 
-#if 0
-  ::timetrace("Start alloc memory");
-  int * p = new int [10000];
-  delete p;
-  ::timetrace("Done alloc memory");
-#endif
-
-  ::timetrace("fixTransactions: get schedule list");
   QList<MyMoneySchedule> scheduleList = file->scheduleList();
-  ::timetrace("fixTransactions: get transaction list");
   MyMoneyTransactionFilter filter;
   filter.setReportAllSplits(false);
   QList<MyMoneyTransaction> transactionList;
   file->transactionList(transactionList, filter);
-  ::timetrace("fixTransactions: have list");
 
   QList<MyMoneySchedule>::Iterator it_x;
   QStringList interestAccounts;
@@ -2158,7 +2107,6 @@ void KMyMoneyView::fixTransactions_0(void)
       kmymoney->slotStatusProgressBar(cnt);
   }
 
-  ::timetrace("fixTransactions: start loop");
   // scan the transactions and modify loan transactions
   QList<MyMoneyTransaction>::Iterator it_t;
   for (it_t = transactionList.begin(); it_t != transactionList.end(); ++it_t) {
@@ -2299,9 +2247,8 @@ void KMyMoneyView::fixTransactions_0(void)
             (*it_t).modifySplit(*it_s);
             file->modifyTransaction(*it_t);
           }
-        } catch (MyMoneyException* e) {
+        } catch (const MyMoneyException &) {
           qDebug("Missing security '%s', split not altered", qPrintable(splitAccount.currencyId()));
-          delete e;
         }
       }
     }
@@ -2383,5 +2330,3 @@ QVBoxLayout* KMyMoneyViewBase::layout(void) const
 {
   return d->m_viewLayout;
 }
-
-#include "kmymoneyview.moc"
