@@ -80,7 +80,7 @@ KScheduledView::KScheduledView(QWidget *parent) :
   // attach popup to 'Filter...' button
   m_kaccPopup = new KMenu(this);
   m_accountsCombo->setMenu(m_kaccPopup);
-  connect(m_kaccPopup, SIGNAL(activated(int)), this, SLOT(slotAccountActivated(int)));
+  connect(m_kaccPopup, SIGNAL(triggered(QAction*)), this, SLOT(slotAccountActivated()));
 
   m_qbuttonNew->setGuiItem(KMyMoneyUtils::scheduleNewGuiItem());
   m_accountsCombo->setGuiItem(KMyMoneyUtils::accountsFilterGuiItem());
@@ -146,9 +146,8 @@ void KScheduledView::refresh(bool full, const QString& schedId)
           }
         }
 
-      } catch (MyMoneyException *e) {
-        KMessageBox::detailedError(this, i18n("Unable to load accounts: "), e->what());
-        delete e;
+      } catch (const MyMoneyException &e) {
+        KMessageBox::detailedError(this, i18n("Unable to load accounts: "), e.what());
       }
     }
 
@@ -165,7 +164,7 @@ void KScheduledView::refresh(bool full, const QString& schedId)
     m_scheduleTree->setSortingEnabled(false);
 
     KScheduleTreeItem *itemBills = new KScheduleTreeItem(m_scheduleTree);
-    itemBills->setIcon(0, KIcon("account-types-expense"));
+    itemBills->setIcon(0, KIcon("view-expenses-categories"));
     itemBills->setText(0, i18n("Bills"));
     itemBills->setData(0, KScheduleTreeItem::OrderRole, QVariant("0"));
     itemBills->setFirstColumnSpanned(true);
@@ -174,21 +173,21 @@ void KScheduledView::refresh(bool full, const QString& schedId)
     bold.setBold(true);
     itemBills->setFont(0, bold);
     KScheduleTreeItem *itemDeposits = new KScheduleTreeItem(m_scheduleTree);
-    itemDeposits->setIcon(0, KIcon("account-types-income"));
+    itemDeposits->setIcon(0, KIcon("view-income-categories"));
     itemDeposits->setText(0, i18n("Deposits"));
     itemDeposits->setData(0, KScheduleTreeItem::OrderRole, QVariant("1"));
     itemDeposits->setFirstColumnSpanned(true);
     itemDeposits->setFlags(Qt::ItemIsEnabled);
     itemDeposits->setFont(0, bold);
     KScheduleTreeItem *itemLoans = new KScheduleTreeItem(m_scheduleTree);
-    itemLoans->setIcon(0, KIcon("account-types-loan"));
+    itemLoans->setIcon(0, KIcon("view-loan"));
     itemLoans->setText(0, i18n("Loans"));
     itemLoans->setData(0, KScheduleTreeItem::OrderRole, QVariant("2"));
     itemLoans->setFirstColumnSpanned(true);
     itemLoans->setFlags(Qt::ItemIsEnabled);
     itemLoans->setFont(0, bold);
     KScheduleTreeItem *itemTransfers = new KScheduleTreeItem(m_scheduleTree);
-    itemTransfers->setIcon(0, KIcon("transaction"));
+    itemTransfers->setIcon(0, KIcon("view-financial-transfer"));
     itemTransfers->setText(0, i18n("Transfers"));
     itemTransfers->setData(0, KScheduleTreeItem::OrderRole, QVariant("3"));
     itemTransfers->setFirstColumnSpanned(true);
@@ -272,9 +271,8 @@ void KScheduledView::refresh(bool full, const QString& schedId)
     if (m_openLoans)
       itemLoans->setExpanded(true);
 
-  } catch (MyMoneyException *e) {
-    KMessageBox::error(this, e->what());
-    delete e;
+  } catch (const MyMoneyException &e) {
+    KMessageBox::error(this, e.what());
   }
 
   for (int i = 0; i < m_scheduleTree->columnCount(); ++i) {
@@ -396,10 +394,9 @@ QTreeWidgetItem* KScheduledView::addScheduleItem(QTreeWidgetItem* parent, MyMone
     item->setData(4, KScheduleTreeItem::OrderRole, QVariant(nextDueDate));
     item->setText(5, i18nc("Frequency of schedule", schedule.occurrenceToString().toLatin1()));
     item->setText(6, KMyMoneyUtils::paymentMethodToString(schedule.paymentType()));
-  } catch (MyMoneyException *e) {
+  } catch (const MyMoneyException &e) {
     item->setText(0, "Error:");
-    item->setText(1, e->what());
-    delete e;
+    item->setText(1, e.what());
   }
   return item;
 }
@@ -488,9 +485,8 @@ void KScheduledView::slotListViewContextMenu(const QPoint& pos)
       emit scheduleSelected(schedule);
       m_selectedSchedule = schedule.id();
       emit openContextMenu();
-    } catch (MyMoneyException *e) {
-      KMessageBox::detailedSorry(this, i18n("Error activating context menu"), e->what());
-      delete e;
+    } catch (const MyMoneyException &e) {
+      KMessageBox::detailedSorry(this, i18n("Error activating context menu"), e.what());
     }
   } else {
     emit openContextMenu();
@@ -506,13 +502,12 @@ void KScheduledView::slotListItemExecuted(QTreeWidgetItem* item, int)
     MyMoneySchedule schedule = item->data(0, Qt::UserRole).value<MyMoneySchedule>();
     m_selectedSchedule = schedule.id();
     emit editSchedule();
-  } catch (MyMoneyException *e) {
-    KMessageBox::detailedSorry(this, i18n("Error executing item"), e->what());
-    delete e;
+  } catch (const MyMoneyException &e) {
+    KMessageBox::detailedSorry(this, i18n("Error executing item"), e.what());
   }
 }
 
-void KScheduledView::slotAccountActivated(int /*id*/)
+void KScheduledView::slotAccountActivated()
 {
   m_filterAccounts.clear();
 
@@ -542,9 +537,8 @@ void KScheduledView::slotAccountActivated(int /*id*/)
     m_calendar->setFilterAccounts(m_filterAccounts);
 
     refresh(false, m_selectedSchedule);
-  } catch (MyMoneyException *e) {
-    KMessageBox::detailedError(this, i18n("Unable to filter account"), e->what());
-    delete e;
+  } catch (const MyMoneyException &e) {
+    KMessageBox::detailedError(this, i18n("Unable to filter account"), e.what());
   }
 }
 
@@ -606,12 +600,8 @@ void KScheduledView::slotSetSelectedItem()
       MyMoneySchedule schedule = item->data(0, Qt::UserRole).value<MyMoneySchedule>();
       emit scheduleSelected(schedule);
       m_selectedSchedule = schedule.id();
-    } catch (MyMoneyException* e) {
-      qDebug("KScheduledView::slotSetSelectedItem: %s", qPrintable(e->what()));
-      delete e;
+    } catch (const MyMoneyException &e) {
+      qDebug("KScheduledView::slotSetSelectedItem: %s", qPrintable(e.what()));
     }
   }
 }
-
-
-#include "kscheduledview.moc"
