@@ -41,7 +41,7 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "mymoneyscheduled.h"
+#include "mymoneyschedule.h"
 #include "kmymoneyutils.h"
 
 KMyMoneyBriefSchedule::KMyMoneyBriefSchedule(QWidget *parent)
@@ -150,8 +150,7 @@ void KMyMoneyBriefSchedule::loadSchedule()
       if (m_index == (m_scheduleList.count() - 1))
         m_nextButton->setEnabled(false);
     }
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
   }
 }
 
