@@ -47,6 +47,7 @@
 #include <klocale.h>
 #include <kglobal.h>
 #include <kdebug.h>
+#include <KPushButton>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -62,7 +63,7 @@
 
 const int LinesPerMemo = 3;
 
-static QString sortOrderText[] = {
+static const char * sortOrderText[] = {
   I18N_NOOP2("Unknown sort order", "Unknown"),
   I18N_NOOP("Post date"),
   I18N_NOOP("Date entered"),
@@ -142,8 +143,6 @@ bool ItemPtrVector::item_cmp(RegisterItem* i1, RegisterItem* i2)
   bool ok1, ok2;
   qulonglong n1, n2;
 
-  MyMoneyMoney tmp;
-
   for (it = sortOrder.begin(); it != sortOrder.end(); ++it) {
     TransactionSortField sortField = static_cast<TransactionSortField>(abs(*it));
     switch (sortField) {
@@ -163,10 +162,9 @@ bool ItemPtrVector::item_cmp(RegisterItem* i1, RegisterItem* i2)
         break;
 
       case ValueSort:
-        tmp = i1->sortValue() - i2->sortValue();
-        if (tmp.isZero())
+        if (i1->sortValue() == i2->sortValue())
           rc = 0;
-        else if (tmp.isNegative())
+        else if (i1->sortValue() < i2->sortValue())
           rc = -1;
         else
           rc = 1;
@@ -286,7 +284,7 @@ void GroupMarker::paintRegisterCell(QPainter *painter, QStyleOptionViewItemV4 &o
   cellRect.setWidth(m_parent->viewport()->width());
   cellRect.setHeight(m_parent->rowHeight(index.row()));
 
-  option.palette.setColor(QPalette::Base, isErronous() ? KMyMoneyGlobalSettings::listErronousTransactionColor() : KMyMoneyGlobalSettings::groupMarkerColor());
+  option.palette.setColor(QPalette::Base, isErroneous() ? KMyMoneyGlobalSettings::listErroneousTransactionColor() : KMyMoneyGlobalSettings::groupMarkerColor());
 
   QBrush backgroundBrush(option.palette.color(QPalette::Base));
   painter->fillRect(cellRect, backgroundBrush);
@@ -294,7 +292,7 @@ void GroupMarker::paintRegisterCell(QPainter *painter, QStyleOptionViewItemV4 &o
   painter->drawLine(cellRect.x(), cellRect.height() - 1, cellRect.width(), cellRect.height() - 1);
 
   // now write the text
-  painter->setPen(option.palette.color( isErronous() ? QPalette::HighlightedText : QPalette::Text ));
+  painter->setPen(option.palette.color(isErroneous() ? QPalette::HighlightedText : QPalette::Text));
   QFont font = painter->font();
   font.setBold(true);
   painter->setFont(font);
@@ -464,8 +462,8 @@ Register::Register(QWidget *parent) :
     m_focusItem(0),
     m_firstItem(0),
     m_lastItem(0),
-    m_firstErronous(0),
-    m_lastErronous(0),
+    m_firstErroneous(0),
+    m_lastErroneous(0),
     m_rowHeightHint(0),
     m_ledgerLensForced(false),
     m_selectionMode(QTableWidget::MultiSelection),
@@ -480,7 +478,7 @@ Register::Register(QWidget *parent) :
 {
   // used for custom coloring with the help of the application's stylesheet
   setObjectName(QLatin1String("register"));
-  m_itemDelegate = new RegisterItemDelegate(this);
+  setItemDelegate(new RegisterItemDelegate(this));
 
   setEditTriggers(QAbstractItemView::NoEditTriggers);
   setColumnCount(MaxColumns);
@@ -676,11 +674,7 @@ void Register::setupRegister(const MyMoneyAccount& account, bool showAccountColu
       break;
   }
 
-  switch (account.accountType()) {
-    default:
-      m_lastCol = BalanceColumn;
-      break;
-  }
+  m_lastCol = BalanceColumn;
 
   setUpdatesEnabled(true);
 }
@@ -755,14 +749,15 @@ TransactionSortField Register::primarySortKey(void) const
 
 void Register::clear(void)
 {
-  m_firstErronous = m_lastErronous = 0;
+  m_firstErroneous = m_lastErroneous = 0;
   m_ensureVisibleItem = 0;
 
+  m_items.clear();
+
   RegisterItem* p;
   while ((p = firstItem()) != 0) {
     delete p;
   }
-  m_items.clear();
 
   m_firstItem = m_lastItem = 0;
 
@@ -874,11 +869,6 @@ void Register::setupItemIndex(int rowCount)
   m_itemIndex.clear();
   m_itemIndex.reserve(rowCount);
 
-  // setup the item delegate for all the rows
-  for (int i = 0; i < rowCount; ++i) {
-    setItemDelegateForRow(i, m_itemDelegate);
-  }
-
   // fill index array
   rowCount = 0;
   RegisterItem* prev = 0;
@@ -940,16 +930,13 @@ void Register::suppressAdjacentMarkers(void)
 
 void Register::updateRegister(bool forceUpdateRowHeight)
 {
-#ifndef KMM_DESIGNER
-  ::timetrace("Update register");
-#endif
   if (m_listsDirty || forceUpdateRowHeight) {
     // don't get in here recursively
     m_listsDirty = false;
 
     int rowCount = 0;
     // determine the number of rows we need to display all items
-    // while going through the list, check for erronous transactions
+    // while going through the list, check for erroneous transactions
     for (QVector<RegisterItem*>::size_type i = 0; i < m_items.size(); ++i) {
       RegisterItem* item = m_items[i];
       if (!item)
@@ -958,10 +945,10 @@ void Register::updateRegister(bool forceUpdateRowHeight)
       item->setNeedResize();
       rowCount += item->numRowsRegister();
 
-      if (item->isErronous()) {
-        if (!m_firstErronous)
-          m_firstErronous = item;
-        m_lastErronous = item;
+      if (item->isErroneous()) {
+        if (!m_firstErroneous)
+          m_firstErroneous = item;
+        m_lastErroneous = item;
       }
     }
 
@@ -1003,9 +990,6 @@ void Register::updateRegister(bool forceUpdateRowHeight)
         QTimer::singleShot(0, this, SLOT(resize()));
     }
   }
-#ifndef KMM_DESIGNER
-  ::timetrace("Done updateing register");
-#endif
 }
 
 int Register::rowHeightHint(void) const
@@ -1113,6 +1097,8 @@ void Register::resize(int col, bool force)
       adjustColumn(PaymentColumn);
     if (columnWidth(DepositColumn))
       adjustColumn(DepositColumn);
+    if (columnWidth(QuantityColumn))
+      adjustColumn(QuantityColumn);
     if (columnWidth(BalanceColumn))
       adjustColumn(BalanceColumn);
     if (columnWidth(PriceColumn))
@@ -1129,39 +1115,37 @@ void Register::resize(int col, bool force)
       ewidth = columnWidth(PaymentColumn);
     if (ewidth < columnWidth(DepositColumn))
       ewidth = columnWidth(DepositColumn);
+    if (ewidth < columnWidth(QuantityColumn))
+      ewidth = columnWidth(QuantityColumn);
     if (dwidth < columnWidth(BalanceColumn))
       dwidth = columnWidth(BalanceColumn);
     if (ewidth < columnWidth(PriceColumn))
       ewidth = columnWidth(PriceColumn);
     if (dwidth < columnWidth(ValueColumn))
       dwidth = columnWidth(ValueColumn);
-
     int swidth = columnWidth(SecurityColumn);
     if (swidth > 0) {
       adjustColumn(SecurityColumn);
       swidth = columnWidth(SecurityColumn);
     }
 
+    adjustColumn(DateColumn);
+
 #ifndef KMM_DESIGNER
     // Resize the date and money fields to either
     // a) the size required by the input widget if no transaction form is shown and the register is used with an editor
     // b) the adjusted value for the input widget if the transaction form is visible or an editor is not used
     if (m_usedWithEditor && !KMyMoneyGlobalSettings::transactionForm()) {
-      kMyMoneyDateInput* dateField = new kMyMoneyDateInput;
-      kMyMoneyEdit* valField = new kMyMoneyEdit;
-
-      dateField->setFont(KMyMoneyGlobalSettings::listCellFont());
-      setColumnWidth(DateColumn, dateField->minimumSizeHint().width());
-      valField->setMinimumWidth(ewidth);
-      ewidth = valField->minimumSizeHint().width();
+      KPushButton *pushButton = new KPushButton;
+      const int pushButtonSpacing = pushButton->sizeHint().width() + 5;
+      setColumnWidth(DateColumn, columnWidth(DateColumn) + pushButtonSpacing + 4/* space for the spinbox arrows */);
+      ewidth += pushButtonSpacing;
 
       if (swidth > 0) {
+        // extend the security width to make space for the selector arrow
         swidth = columnWidth(SecurityColumn) + 40;
       }
-      delete valField;
-      delete dateField;
-    } else {
-      adjustColumn(DateColumn);
+      delete pushButton;
     }
 #endif
 
@@ -1169,6 +1153,8 @@ void Register::resize(int col, bool force)
       setColumnWidth(PaymentColumn, ewidth);
     if (columnWidth(DepositColumn))
       setColumnWidth(DepositColumn, ewidth);
+    if (columnWidth(QuantityColumn))
+      setColumnWidth(QuantityColumn, ewidth);
     if (columnWidth(BalanceColumn))
       setColumnWidth(BalanceColumn, dwidth);
     if (columnWidth(PriceColumn))
@@ -1203,26 +1189,18 @@ void Register::resize(int col, bool force)
   setColumnWidth(col, w);
 }
 
-
-void Register::adjustColumn(int col)
+int Register::minimumColumnWidth(int col)
 {
-#ifdef KMM_DESIGNER
-  Q_UNUSED(col)
-#else
-  QString msg = "%1 adjusting column %2";
-  ::timetrace(qPrintable(msg.arg("Start").arg(col)));
   QHeaderView *topHeader = horizontalHeader();
-  QFontMetrics cellFontMetrics(KMyMoneyGlobalSettings::listCellFont());
-
   int w = topHeader->fontMetrics().width(horizontalHeaderItem(col) ? horizontalHeaderItem(col)->text() : QString()) + 10;
   w = qMax(w, 20);
-
+#ifdef KMM_DESIGNER
+  return w;
+#else
   int maxWidth = 0;
   int minWidth = 0;
+  QFontMetrics cellFontMetrics(KMyMoneyGlobalSettings::listCellFont());
   switch (col) {
-    case NumberColumn:
-      maxWidth = cellFontMetrics.width("0123456789");
-      break;
     case DateColumn:
       minWidth = cellFontMetrics.width(KGlobal::locale()->formatDate(QDate(6999, 12, 29), KLocale::ShortDate) + "  ");
       break;
@@ -1240,14 +1218,13 @@ void Register::adjustColumn(int col)
       int nw = 0;
       try {
         nw = t->registerColWidth(col, cellFontMetrics);
-      } catch (MyMoneyException* e) {
+      } catch (const MyMoneyException &) {
         // This should only be reached if the data in the file disappeared
         // from under us, such as when the account was deleted from a
         // different view, then this view is restored. In this case, new
         // data is about to be loaded into the view anyway, so just remove
         // the item from the register and swallow the exception.
-        //qDebug("%s", qPrintable(e->what()));
-        delete e;
+        //qDebug("%s", qPrintable(e.what()));
         removeItem(t);
       }
       w = qMax(w, nw);
@@ -1264,10 +1241,15 @@ void Register::adjustColumn(int col)
     }
   }
 
-  setColumnWidth(col, w);
+  return w;
 #endif
 }
 
+void Register::adjustColumn(int col)
+{
+  setColumnWidth(col, minimumColumnWidth(col));
+}
+
 void Register::clearSelection(void)
 {
   unselectItems();
@@ -1418,9 +1400,10 @@ void Register::selectItem(int row, int col)
 
     // don't support selecting when the item has an editor
     // or the item itself is not selectable
-    if (item->hasEditorOpen() || !item->isSelectable())
+    if (item->hasEditorOpen() || !item->isSelectable()) {
+      m_mouseButton = Qt::NoButton;
       return;
-
+    }
     QString id = item->id();
     selectItem(item);
     // selectItem() might have changed the pointers, so we
@@ -1493,12 +1476,7 @@ void Register::selectItem(RegisterItem* item, bool dontChangeSelections)
     QList<RegisterItem*> itemList = selectedItems();
     bool okToSelect = true;
     int cnt = itemList.count();
-    bool sameEntryType = true;
-    if (cnt > 0) {
-      if (typeid(*itemList.begin()) != typeid(item))
-        sameEntryType = false;
-    }
-
+    const bool scheduledTransactionSelected = (cnt > 0 && itemList.front() && typeid(*itemList.front()) == typeid(StdTransactionScheduled));
     if (buttonState & Qt::LeftButton) {
       if (!(modifiers & (Qt::ShiftModifier | Qt::ControlModifier))
           || (m_selectAnchor == 0)) {
@@ -1519,8 +1497,7 @@ void Register::selectItem(RegisterItem* item, bool dontChangeSelections)
       if (m_selectionMode == MultiSelection) {
         switch (modifiers & (Qt::ShiftModifier | Qt::ControlModifier)) {
           case Qt::ControlModifier:
-            okToSelect = sameEntryType;
-            if (typeid(*item) == typeid(StdTransactionScheduled))
+            if (scheduledTransactionSelected || typeid(*item) == typeid(StdTransactionScheduled))
               okToSelect = false;
             // toggle selection state of current item
             emit aboutToSelectItem(item, okToSelect);
@@ -1533,8 +1510,7 @@ void Register::selectItem(RegisterItem* item, bool dontChangeSelections)
             break;
 
           case Qt::ShiftModifier:
-            okToSelect = sameEntryType;
-            if (typeid(*item) == typeid(StdTransactionScheduled))
+            if (scheduledTransactionSelected || typeid(*item) == typeid(StdTransactionScheduled))
               okToSelect = false;
             emit aboutToSelectItem(item, okToSelect);
             if (okToSelect) {
@@ -1618,7 +1594,7 @@ void Register::slotEnsureItemVisible(void)
 TransactionSortField KMyMoneyRegister::textToSortOrder(const QString& text)
 {
   for (int idx = 1; idx < static_cast<int>(MaxSortFields); ++idx) {
-    if (text == i18n(qPrintable(sortOrderText[idx]))) {
+    if (text == i18n(sortOrderText[idx])) {
       return static_cast<TransactionSortField>(idx);
     }
   }
@@ -1629,7 +1605,7 @@ const QString KMyMoneyRegister::sortOrderToText(TransactionSortField idx)
 {
   if (idx < PostDateSort || idx >= MaxSortFields)
     idx = UnknownSort;
-  return i18n(qPrintable(sortOrderText[idx]));
+  return i18n(sortOrderText[idx]);
 }
 
 QString Register::text(int /*row*/, int /*col*/) const
@@ -1765,10 +1741,8 @@ void Register::selectRange(RegisterItem* from, RegisterItem* to, bool invert, bo
         }
       } else {
         bool sel = !item->isSelected();
-        if ((item->isSelected() != sel) && (sel || !sel)) {
-          if (item->isVisible()) {
-            item->setSelected(sel);
-          }
+        if ((item->isSelected() != sel) && item->isVisible()) {
+          item->setSelected(sel);
         }
       }
     }
@@ -1999,7 +1973,7 @@ void Register::addGroupMarkers(void)
             balance = -balance;
           QString txt = i18n("Online Statement Balance: %1", balance.formatMoney(m_account.fraction()));
 
-          KMyMoneyRegister::StatementGroupMarker *p=new KMyMoneyRegister::StatementGroupMarker(this, KMyMoneyRegister::Deposit, QDate::fromString(m_account.value("lastImportedTransactionDate"), Qt::ISODate), txt);
+          KMyMoneyRegister::StatementGroupMarker *p = new KMyMoneyRegister::StatementGroupMarker(this, KMyMoneyRegister::Deposit, QDate::fromString(m_account.value("lastImportedTransactionDate"), Qt::ISODate), txt);
 
           p->setErroneous(!MyMoneyFile::instance()->hasMatchingOnlineBalance(m_account));
         }
@@ -2020,7 +1994,6 @@ void Register::addGroupMarkers(void)
       }
       if (KMyMoneyGlobalSettings::showFiscalMarker()) {
         QDate currentFiscalYear = KMyMoneyGlobalSettings::firstFiscalDate();
-        QDate previousFiscalYear = currentFiscalYear.addYears(-1);
         new KMyMoneyRegister::FiscalYearGroupMarker(this, currentFiscalYear, i18n("Current fiscal year"));
         new KMyMoneyRegister::FiscalYearGroupMarker(this, currentFiscalYear.addYears(-1), i18n("Previous fiscal year"));
         new KMyMoneyRegister::FiscalYearGroupMarker(this, currentFiscalYear.addYears(1), i18n("Next fiscal year"));
@@ -2148,7 +2121,3 @@ void Register::setDetailsColumnType(DetailsColumnType detailsColumnType)
 {
   m_detailsColumnType = detailsColumnType;
 }
-
-#include "register.moc"
-
-// vim:cin:si:ai:et:ts=2:sw=2:
