@@ -25,6 +25,7 @@
 #include <QToolTip>
 #include <QLabel>
 #include <QList>
+#include <qmath.h>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
@@ -142,7 +143,7 @@ const MyMoneyAccount& Wizard::account(void)
   m_account.setAccountType(m_accountTypePage->accountType());
   m_account.setInstitutionId(m_institutionPage->institution().id());
   m_account.setNumber(m_institutionPage->m_accountNumber->text());
-  m_account.setValue("IBAN", m_institutionPage->m_iban->text());
+  m_account.setValue("iban", m_institutionPage->m_iban->text());
   if (m_accountTypePage->m_preferredAccount->isChecked())
     m_account.setValue("PreferredAccount", "Yes");
   else
@@ -220,7 +221,7 @@ MyMoneyAccount Wizard::brokerageAccount(void) const
     if (m_brokeragepage->m_accountNumber->isEnabled() && !m_brokeragepage->m_accountNumber->text().isEmpty())
       account.setNumber(m_brokeragepage->m_accountNumber->text());
     if (m_brokeragepage->m_iban->isEnabled() && !m_brokeragepage->m_iban->text().isEmpty())
-      account.setValue("IBAN", m_brokeragepage->m_iban->text());
+      account.setValue("iban", m_brokeragepage->m_iban->text());
   }
   return account;
 }
@@ -430,8 +431,6 @@ void InstitutionPage::slotSelectInstitution(const int index)
 
 void InstitutionPage::selectExistingInstitution(const QString& id)
 {
-  QList<MyMoneyInstitution>::const_iterator it = d->m_list.constBegin();
-
   for (int i = 0; i < d->m_list.length(); ++i) {
     if (d->m_list[i].id() == id) {
       m_institutionComboBox->setCurrentIndex(i + 1);
@@ -918,7 +917,7 @@ void LoanDetailsPage::slotValuesChanged(void)
 void LoanDetailsPage::slotCalculate(void)
 {
   MyMoneyFinancialCalculator calc;
-  long double val;
+  double val;
   int PF, CF;
   QString result;
   bool moneyBorrowed = m_wizard->moneyBorrowed();
@@ -940,33 +939,33 @@ void LoanDetailsPage::slotCalculate(void)
 
 
   if (!m_loanAmount->lineedit()->text().isEmpty()) {
-    val = static_cast<long double>(m_loanAmount->value().abs().toDouble());
+    val = m_loanAmount->value().abs().toDouble();
     if (moneyBorrowed)
       val = -val;
     calc.setPv(val);
   }
 
   if (!m_interestRate->lineedit()->text().isEmpty()) {
-    val = static_cast<long double>(m_interestRate->value().abs().toDouble());
+    val = m_interestRate->value().abs().toDouble();
     calc.setIr(val);
   }
 
   if (!m_paymentAmount->lineedit()->text().isEmpty()) {
-    val = static_cast<long double>(m_paymentAmount->value().abs().toDouble());
+    val = m_paymentAmount->value().abs().toDouble();
     if (moneyLend)
       val = -val;
     calc.setPmt(val);
   }
 
   if (!m_balloonAmount->lineedit()->text().isEmpty()) {
-    val = static_cast<long double>(m_balloonAmount->value().abs().toDouble());
+    val = m_balloonAmount->value().abs().toDouble();
     if (moneyLend)
       val = -val;
     calc.setFv(val);
   }
 
   if (m_termAmount->value() != 0) {
-    calc.setNpp(static_cast<long double>(term()));
+    calc.setNpp(term());
   }
 
   // setup of parameters is done, now do the calculation
@@ -989,7 +988,7 @@ void LoanDetailsPage::slotCalculate(void)
       val = calc.payment();
       m_paymentAmount->setValue(MyMoneyMoney(static_cast<double>(val)).abs());
       // reset payment as it might have changed due to rounding
-      val = static_cast<long double>(m_paymentAmount->value().abs().toDouble());
+      val = m_paymentAmount->value().abs().toDouble();
       if (moneyLend)
         val = -val;
       calc.setPmt(val);
@@ -997,8 +996,8 @@ void LoanDetailsPage::slotCalculate(void)
       result = i18n("KMyMoney has calculated a periodic payment of %1 to cover principal and interest.", m_paymentAmount->lineedit()->text());
 
       val = calc.futureValue();
-      if ((moneyBorrowed && val < 0 && fabsl(val) >= fabsl(calc.payment()))
-          || (moneyLend && val > 0 && fabs(val) >= fabs(calc.payment()))) {
+      if ((moneyBorrowed && val < 0 && qAbs(val) >= qAbs(calc.payment()))
+          || (moneyLend && val > 0 && qAbs(val) >= qAbs(calc.payment()))) {
         calc.setNpp(calc.npp() - 1);
         // updateTermWidgets(calc.npp());
         val = calc.futureValue();
@@ -1006,8 +1005,8 @@ void LoanDetailsPage::slotCalculate(void)
         m_balloonAmount->loadText(refVal.abs().formatMoney("", m_wizard->precision()));
         result += QString(" ");
         result += i18n("The number of payments has been decremented and the balloon payment has been modified to %1.", m_balloonAmount->lineedit()->text());
-      } else if ((moneyBorrowed && val < 0 && fabsl(val) < fabsl(calc.payment()))
-                 || (moneyLend && val > 0 && fabs(val) < fabs(calc.payment()))) {
+      } else if ((moneyBorrowed && val < 0 && qAbs(val) < qAbs(calc.payment()))
+                 || (moneyLend && val > 0 && qAbs(val) < qAbs(calc.payment()))) {
         m_balloonAmount->loadText(MyMoneyMoney(0, 1).formatMoney("", m_wizard->precision()));
       } else {
         MyMoneyMoney refVal(static_cast<double>(val));
@@ -1019,14 +1018,14 @@ void LoanDetailsPage::slotCalculate(void)
       // calculate the number of payments out of the other information
       val = calc.numPayments();
       if (val == 0)
-        throw new MYMONEYEXCEPTION("incorrect fincancial calculation");
+        throw MYMONEYEXCEPTION("incorrect fincancial calculation");
 
       // if the number of payments has a fractional part, then we
       // round it to the smallest integer and calculate the balloon payment
-      result = i18n("KMyMoney has calculated the term of your loan as %1. ", updateTermWidgets(floorl(val)));
+      result = i18n("KMyMoney has calculated the term of your loan as %1. ", updateTermWidgets(qFloor(val)));
 
-      if (val != floorl(val)) {
-        calc.setNpp(floorl(val));
+      if (val != qFloor(val)) {
+        calc.setNpp(qFloor(val));
         val = calc.futureValue();
         MyMoneyMoney refVal(static_cast<double>(val));
         m_balloonAmount->loadText(refVal.abs().formatMoney("", m_wizard->precision()));
@@ -1047,14 +1046,14 @@ void LoanDetailsPage::slotCalculate(void)
       //    less in value than regular payments. That means, that the
       //    future value is to be treated as  (fully payed back)
       // c) the loan is not payed back yet
-      if ((moneyBorrowed && val < 0 && fabsl(val) > fabsl(calc.payment()))
-          || (moneyLend && val > 0 && fabs(val) > fabs(calc.payment()))) {
+      if ((moneyBorrowed && val < 0 && qAbs(val) > qAbs(calc.payment()))
+          || (moneyLend && val > 0 && qAbs(val) > qAbs(calc.payment()))) {
         // case a)
-        qDebug("Future Value is %Lf", val);
-        throw new MYMONEYEXCEPTION("incorrect fincancial calculation");
+        qDebug("Future Value is %f", val);
+        throw MYMONEYEXCEPTION("incorrect fincancial calculation");
 
-      } else if ((moneyBorrowed && val < 0 && fabsl(val) <= fabsl(calc.payment()))
-                 || (moneyLend && val > 0 && fabs(val) <= fabs(calc.payment()))) {
+      } else if ((moneyBorrowed && val < 0 && qAbs(val) <= qAbs(calc.payment()))
+                 || (moneyLend && val > 0 && qAbs(val) <= qAbs(calc.payment()))) {
         // case b)
         val = 0;
       }
@@ -1064,15 +1063,14 @@ void LoanDetailsPage::slotCalculate(void)
 
       if (!m_balloonAmount->lineedit()->text().isEmpty()) {
         if ((m_balloonAmount->value().abs() - refVal.abs()).abs().toDouble() > 1) {
-          throw new MYMONEYEXCEPTION("incorrect fincancial calculation");
+          throw MYMONEYEXCEPTION("incorrect fincancial calculation");
         }
         result = i18n("KMyMoney has successfully verified your loan information.");
       }
       m_balloonAmount->loadText(refVal.abs().formatMoney("", m_wizard->precision()));
     }
 
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     KMessageBox::error(0,
                        i18n("You have entered mis-matching information. Please modify "
                             "your figures or leave one value empty "
@@ -1121,9 +1119,9 @@ int LoanDetailsPage::term(void) const
   return factor;
 }
 
-QString LoanDetailsPage::updateTermWidgets(const long double val)
+QString LoanDetailsPage::updateTermWidgets(const double val)
 {
-  long long vl = static_cast<long long>(floorl(val));
+  long vl = qFloor(val);
 
   QString valString;
   MyMoneySchedule::occurrenceE unit = m_termUnit->currentItem();
@@ -1157,41 +1155,27 @@ bool LoanDetailsPage::isComplete(void) const
   // bool rc = KMyMoneyWizardPage::isComplete();
 
   int fieldCnt = 0;
-  QWidget* calculatedField = 0;
 
   if (m_loanAmount->lineedit()->text().length() > 0) {
     fieldCnt++;
-  } else {
-    calculatedField = m_loanAmount;
   }
 
   if (m_interestRate->lineedit()->text().length() > 0) {
     fieldCnt++;
-  } else {
-    calculatedField = m_interestRate;
   }
 
   if (m_termAmount->value() != 0) {
     fieldCnt++;
-  } else {
-    calculatedField = m_termAmount;
   }
 
   if (m_paymentAmount->lineedit()->text().length() > 0) {
     fieldCnt++;
-  } else {
-    calculatedField = m_paymentAmount;
   }
 
   if (m_balloonAmount->lineedit()->text().length() > 0) {
     fieldCnt++;
-  } else {
-    calculatedField = m_balloonAmount;
   }
 
-  if (fieldCnt == 5)
-    calculatedField = 0;
-
   m_calculateButton->setEnabled(fieldCnt == 4 || (fieldCnt == 5 && m_needCalculate));
 
   m_calculateButton->setAutoDefault(false);
@@ -1477,7 +1461,7 @@ HierarchyFilterProxyModel::HierarchyFilterProxyModel(QObject *parent)
 bool HierarchyFilterProxyModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
 {
   if (!source_parent.isValid()) {
-    QVariant data = sourceModel()->data(sourceModel()->index(source_row, 0, source_parent), AccountsModel::AccountIdRole);
+    QVariant data = sourceModel()->index(source_row, 0, source_parent).data(AccountsModel::AccountIdRole);
     if (data.isValid() && data.toString() == AccountsModel::favoritesAccountId)
       return false;
   }
@@ -1580,29 +1564,31 @@ void AccountSummaryPage::enterPage(void)
   m_dataList->setFontWeight(QFont::Bold);
   m_dataList->append(i18n("Account information"));
   m_dataList->setFontWeight(QFont::Normal);
-  m_dataList->append(QString(i18nc("Account name", "Name") + ": %1").arg(acc.name()));
+  m_dataList->append(i18nc("Account name", "Name: %1", acc.name()));
   if (!acc.isLoan())
     m_dataList->append(i18n("Subaccount of %1", m_wizard->parentAccount().name()));
+  QString accTypeText;
   if (acc.accountType() == MyMoneyAccount::AssetLoan)
-    m_dataList->append(QString(i18n("Type") + ": %1").arg(i18n("Loan")));
+    accTypeText = i18n("Loan");
   else
-    m_dataList->append(QString(i18n("Type") + ": %1").arg(m_wizard->m_accountTypePage->m_typeSelection->currentText()));
+    accTypeText = m_wizard->m_accountTypePage->m_typeSelection->currentText();
+  m_dataList->append(i18n("Type: %1", accTypeText));
 
-  m_dataList->append(QString(i18n("Currency") + ": %1").arg(m_wizard->currency().name()));
-  m_dataList->append(QString(i18n("Opening date") + ": %1").arg(KGlobal::locale()->formatDate(acc.openingDate())));
+  m_dataList->append(i18n("Currency: %1", m_wizard->currency().name()));
+  m_dataList->append(i18n("Opening date: %1", KGlobal::locale()->formatDate(acc.openingDate())));
   if (m_wizard->currency().id() != MyMoneyFile::instance()->baseCurrency().id()) {
-    m_dataList->append(QString(i18n("Conversion rate") + ": %1").arg(m_wizard->conversionRate().rate(QString()).formatMoney("", KMyMoneyGlobalSettings::pricePrecision())));
+    m_dataList->append(i18n("Conversion rate: %1", m_wizard->conversionRate().rate(QString()).formatMoney("", KMyMoneyGlobalSettings::pricePrecision())));
   }
   if (!acc.isLoan() || !m_wizard->openingBalance().isZero())
-    m_dataList->append(QString(i18n("Opening balance") + ": %1").arg(MyMoneyUtils::formatMoney(m_wizard->openingBalance(), acc, sec)));
+    m_dataList->append(i18n("Opening balance: %1", MyMoneyUtils::formatMoney(m_wizard->openingBalance(), acc, sec)));
 
   if (!m_wizard->m_institutionPage->institution().id().isEmpty()) {
-    m_dataList->append(QString(i18n("Institution") + ": %1").arg(m_wizard->m_institutionPage->institution().name()));
+    m_dataList->append(i18n("Institution: %1", m_wizard->m_institutionPage->institution().name()));
     if (!acc.number().isEmpty()) {
-      m_dataList->append(QString(i18n("Number") + ": %1").arg(acc.number()));
+      m_dataList->append(i18n("Number: %1", acc.number()));
     }
-    if (!acc.value("IBAN").isEmpty()) {
-      m_dataList->append(QString(i18n("IBAN") + ": %1").arg(acc.value("IBAN")));
+    if (!acc.value("iban").isEmpty()) {
+      m_dataList->append(i18n("IBAN: %1", acc.value("iban")));
     }
   }
 
@@ -1612,12 +1598,12 @@ void AccountSummaryPage::enterPage(void)
       m_dataList->append(i18n("Brokerage Account"));
       m_dataList->setFontWeight(QFont::Normal);
 
-      m_dataList->append(QString(i18nc("Account name", "Name") + ": %1 " + i18n("(Brokerage)")).arg(acc.name()));
-      m_dataList->append(QString(i18n("Currency") + ": %1").arg(m_wizard->m_brokeragepage->m_brokerageCurrency->security().name()));
+      m_dataList->append(i18nc("Account name", "Name: %1 (Brokerage)", acc.name()));
+      m_dataList->append(i18n("Currency: %1", m_wizard->m_brokeragepage->m_brokerageCurrency->security().name()));
       if (m_wizard->m_brokeragepage->m_accountNumber->isEnabled() && !m_wizard->m_brokeragepage->m_accountNumber->text().isEmpty())
-        m_dataList->append(QString(i18n("Number") + ": %1").arg(m_wizard->m_brokeragepage->m_accountNumber->text()));
+        m_dataList->append(i18n("Number: %1", m_wizard->m_brokeragepage->m_accountNumber->text()));
       if (m_wizard->m_brokeragepage->m_iban->isEnabled() && !m_wizard->m_brokeragepage->m_iban->text().isEmpty())
-        m_dataList->append(QString(i18n("IBAN") + ": %1").arg(m_wizard->m_brokeragepage->m_iban->text()));
+        m_dataList->append(i18n("IBAN: %1", m_wizard->m_brokeragepage->m_iban->text()));
     }
   }
 
@@ -1627,30 +1613,30 @@ void AccountSummaryPage::enterPage(void)
     m_dataList->append(i18n("Loan information"));
     m_dataList->setFontWeight(QFont::Normal);
     if (m_wizard->moneyBorrowed()) {
-      m_dataList->append(QString(i18n("Amount borrowed") + ": %1").arg(m_wizard->m_loanDetailsPage->m_loanAmount->value().formatMoney(m_wizard->currency().tradingSymbol(), m_wizard->precision())));
+      m_dataList->append(i18n("Amount borrowed: %1", m_wizard->m_loanDetailsPage->m_loanAmount->value().formatMoney(m_wizard->currency().tradingSymbol(), m_wizard->precision())));
     } else {
-      m_dataList->append(QString(i18n("Amount lent") + ": %1").arg(m_wizard->m_loanDetailsPage->m_loanAmount->value().formatMoney(m_wizard->currency().tradingSymbol(), m_wizard->precision())));
+      m_dataList->append(i18n("Amount lent: %1", m_wizard->m_loanDetailsPage->m_loanAmount->value().formatMoney(m_wizard->currency().tradingSymbol(), m_wizard->precision())));
     }
-    m_dataList->append(QString(i18n("Interest rate") + ": %1 %").arg(m_wizard->m_loanDetailsPage->m_interestRate->value().formatMoney("", -1)));
+    m_dataList->append(i18n("Interest rate: %1 %", m_wizard->m_loanDetailsPage->m_interestRate->value().formatMoney("", -1)));
     m_dataList->append(i18n("Interest rate is %1", m_wizard->m_generalLoanInfoPage->m_interestType->currentText()));
-    m_dataList->append(QString(i18n("Principal and interest") + ": %1").arg(MyMoneyUtils::formatMoney(m_wizard->m_loanDetailsPage->m_paymentAmount->value(), acc, sec)));
-    m_dataList->append(QString(i18n("Additional fees") + ": %1").arg(MyMoneyUtils::formatMoney(m_wizard->m_loanPaymentPage->additionalFees(), acc, sec)));
-    m_dataList->append(QString(i18n("Payment frequency") + ": %1").arg(m_wizard->m_generalLoanInfoPage->m_paymentFrequency->currentText()));
-    m_dataList->append(QString(i18n("Payment account") + ": %1").arg(m_wizard->m_loanSchedulePage->m_paymentAccount->currentText()));
+    m_dataList->append(i18n("Principal and interest: %1", MyMoneyUtils::formatMoney(m_wizard->m_loanDetailsPage->m_paymentAmount->value(), acc, sec)));
+    m_dataList->append(i18n("Additional Fees: %1", MyMoneyUtils::formatMoney(m_wizard->m_loanPaymentPage->additionalFees(), acc, sec)));
+    m_dataList->append(i18n("Payment frequency: %1", m_wizard->m_generalLoanInfoPage->m_paymentFrequency->currentText()));
+    m_dataList->append(i18n("Payment account: %1", m_wizard->m_loanSchedulePage->m_paymentAccount->currentText()));
 
     if (!m_wizard->m_loanPayoutPage->m_noPayoutTransaction->isChecked() && m_wizard->openingBalance().isZero()) {
       m_dataList->setFontWeight(QFont::Bold);
       m_dataList->append(i18n("Payout information"));
       m_dataList->setFontWeight(QFont::Normal);
       if (m_wizard->m_loanPayoutPage->m_refinanceLoan->isChecked()) {
-        m_dataList->append(QString(i18n("Refinance") + ": %1").arg(m_wizard->m_loanPayoutPage->m_loanAccount->currentText()));
+        m_dataList->append(i18n("Refinance: %1", m_wizard->m_loanPayoutPage->m_loanAccount->currentText()));
       } else {
         if (m_wizard->moneyBorrowed())
           m_dataList->append(i18n("Transfer amount to %1", m_wizard->m_loanPayoutPage->m_assetAccount->currentText()));
         else
           m_dataList->append(i18n("Transfer amount from %1", m_wizard->m_loanPayoutPage->m_assetAccount->currentText()));
       }
-      m_dataList->append(QString(i18n("Payment date") + ": %1 ").arg(KGlobal::locale()->formatDate(m_wizard->m_loanPayoutPage->m_payoutDate->date())));
+      m_dataList->append(i18n("Payment date: %1 ", KGlobal::locale()->formatDate(m_wizard->m_loanPayoutPage->m_payoutDate->date())));
     }
   }
 
@@ -1659,24 +1645,21 @@ void AccountSummaryPage::enterPage(void)
     m_dataList->setFontWeight(QFont::Bold);
     m_dataList->append(i18n("Schedule information"));
     m_dataList->setFontWeight(QFont::Normal);
-    m_dataList->append(QString(i18nc("Schedule name", "Name") + ": %1").arg(sch.name()));
+    m_dataList->append(i18nc("Schedule name", "Name: %1", sch.name()));
     if (acc.accountType() == MyMoneyAccount::CreditCard) {
       MyMoneyAccount paymentAccount = MyMoneyFile::instance()->account(m_wizard->m_schedulePage->m_paymentAccount->selectedItem());
-      m_dataList->append(QString(i18nc("Schedule occurrence", "Occurrence") + ": %1").arg(i18nc("Once per month schedule occurrence", "Monthly")));
+      m_dataList->append(i18n("Occurrence: Monthly"));
       m_dataList->append(i18n("Paid from %1", paymentAccount.name()));
       m_dataList->append(i18n("Pay to %1", m_wizard->m_schedulePage->m_payee->currentText()));
-      m_dataList->append(QString(i18n("Amount") + ": %1").arg(MyMoneyUtils::formatMoney(m_wizard->m_schedulePage->m_amount->value(), acc, sec)));
+      m_dataList->append(i18n("Amount: %1", MyMoneyUtils::formatMoney(m_wizard->m_schedulePage->m_amount->value(), acc, sec)));
       m_dataList->append(i18n("First payment due on %1", KGlobal::locale()->formatDate(sch.nextDueDate())));
-      m_dataList->append(QString(i18n("Payment method") + ": %1").arg(m_wizard->m_schedulePage->m_method->currentText()));
+      m_dataList->append(i18n("Payment method: %1", m_wizard->m_schedulePage->m_method->currentText()));
     }
     if (acc.isLoan()) {
-      m_dataList->append(QString(i18n("Occurrence") + ": %1").arg(m_wizard->m_generalLoanInfoPage->m_paymentFrequency->currentText()));
-      m_dataList->append(QString(i18n("Amount") + ": %1").arg(MyMoneyUtils::formatMoney(m_wizard->m_loanPaymentPage->basePayment() + m_wizard->m_loanPaymentPage->additionalFees(), acc, sec)));
+      m_dataList->append(i18n("Occurrence: %1", m_wizard->m_generalLoanInfoPage->m_paymentFrequency->currentText()));
+      m_dataList->append(i18n("Amount: %1", MyMoneyUtils::formatMoney(m_wizard->m_loanPaymentPage->basePayment() + m_wizard->m_loanPaymentPage->additionalFees(), acc, sec)));
       m_dataList->append(i18n("First payment due on %1", KGlobal::locale()->formatDate(m_wizard->m_loanSchedulePage->firstPaymentDueDate())));
     }
   }
 }
 }
-
-#include "knewaccountwizard.moc"
-#include "knewaccountwizard_p.moc"
