@@ -60,8 +60,7 @@ KEditLoanWizard::KEditLoanWizard(const MyMoneyAccount& account, QWidget *parent)
   try {
     QString id = m_account.value("schedule");
     m_schedule = file->schedule(id);
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
   }
   m_lastSelection = -1;
 
@@ -115,6 +114,9 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
     m_interestTypePage->m_variableInterestButton->animateClick();
   }
 
+  QString institutionName = file->institution(m_account.institutionId()).name();
+  m_loanAttributesPage->setInstitution(institutionName);
+
   MyMoneyMoney ir;
   if (m_schedule.startDate() > QDate::currentDate()) {
     ir = m_account.interestRate(m_schedule.startDate());
@@ -162,8 +164,7 @@ void KEditLoanWizard::loadWidgets(const MyMoneyAccount& /* account */)
         try {
           payee = file->payee(it_s.payeeId());
           setField("payeeEdit", payee.id());
-        } catch (MyMoneyException *e) {
-          delete e;
+        } catch (const MyMoneyException &) {
           qWarning("Payee for schedule has been deleted");
         }
       }
@@ -277,6 +278,9 @@ bool KEditLoanWizard::validateCurrentPage()
       m_pages.clearBit(Page_Schedule);
       m_pages.setBit(Page_Summary);
 
+      // Attributes
+      m_pages.clearBit(Page_LoanAttributes);
+
       m_pages.setBit(Page_EffectiveDate);
 
       if (page(Page_Summary) != 0) {
@@ -315,6 +319,9 @@ bool KEditLoanWizard::validateCurrentPage()
         setPage(Page_Summary, m_summaryPage);
         m_pages.setBit(Page_Summary);
 
+      } else if (field("editAttributesButton").toBool()) {
+        m_pages.setBit(Page_LoanAttributes);
+        m_pages.clearBit(Page_EffectiveDate);
       } else {
         qWarning("%s,%d: This should never happen", __FILE__, __LINE__);
       }
@@ -374,9 +381,8 @@ bool KEditLoanWizard::validateCurrentPage()
             split = (*it).splitByAccount(m_account.id());
             balance += split.value();
 
-          } catch(MyMoneyException *e) {
+          } catch(const MyMoneyException &e) {
             // account is not referenced within this transaction
-            delete e;
           }
         }
         m_loanAmountEdit->setText(balance.formatMoney());
@@ -458,6 +464,22 @@ const MyMoneyAccount KEditLoanWizard::account(void) const
   else
     acc.setInterestCalculation(MyMoneyAccountLoan::paymentDue);
 
+  MyMoneyFile *file = MyMoneyFile::instance();
+
+  QString institution = m_loanAttributesPage->m_qcomboboxInstitutions->currentText();
+  if (institution != i18n("(No Institution)")) {
+    QList<MyMoneyInstitution> list;
+    file->institutionList(list);
+    Q_FOREACH(const MyMoneyInstitution& testInstitution, list) {
+      if (testInstitution.name() == institution) {
+        acc.setInstitutionId(testInstitution.id());
+        break;
+      }
+    }
+  } else {
+    acc.setInstitutionId(QString());
+  }
+
   acc.setFixedInterestRate(field("fixedInterestButton").toBool());
   acc.setFinalPayment(field("finalPaymentEdit").value<MyMoneyMoney>());
   acc.setTerm(m_durationPage->term());
@@ -485,5 +507,3 @@ const MyMoneyTransaction KEditLoanWizard::transaction() const
 
   return t;
 }
-
-#include "keditloanwizard.moc"
